package uk.ac.ox.tall.wcker.wizard;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import org.jdom.Attribute;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;

import uk.ac.ox.tall.wcker.Wcker;

/**
 * ApplyToItem object, describing what questions to ask about which heirarchy items 
 * @author balchd
 *
 */
public class ApplyToItem {
	private NeedToKnow ntk;
	private String targetHook;
	private String prompt;
	private Guidance[] guidance;
	private String[] qGroupRefs;

	/**
	 * Constructor
	 * @param applyEl
	 */
	public ApplyToItem(Element applyEl, NeedToKnow ntkArg) {
        if(Wcker.DEBUG > 1) {
            System.out.println(" Construct an ApplyToItem ");
            /*
            System.out.println("applyEl: " + applyEl);
            System.out.println("applyEl.getAttributeValue(\"targetHook\"): " + applyEl.getAttributeValue("targetHook"));
            System.out.println("applyEl.getChildren(): " + applyEl.getChildren());
            System.out.println("applyEl.getChildText(\"prompt\", NeedToKnow.nsWSI): " + applyEl.getChildText("prompt", NeedToKnow.nsWSI));
            */
        }
		ntk = ntkArg;
		// Get the TargetHook value
		targetHook = applyEl.getAttributeValue("targetHook");
		
		// Set the prompt
		Element promptNode = applyEl.getChild("prompt", NeedToKnow.nsWSI);
		if (promptNode != null) {
			prompt = promptNode.getText();
		}
		

		// Set the questionGroup references
		XPath childPath;
		List qGroupRefNodes = null;
		try {
			childPath = XPath.newInstance("wsi:questionGroup/@ref");
			childPath.addNamespace(NeedToKnow.nsWSI);
			qGroupRefNodes = childPath.selectNodes(applyEl);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		
		qGroupRefs = new String[qGroupRefNodes.size()];
		for (int i=0; i<qGroupRefNodes.size(); i++) {
			qGroupRefs[i] = ((Attribute)qGroupRefNodes.get(i)).getValue();
		}
		
		
		// Set the guidance
		List guidanceNodes = applyEl.getChildren("guidance", NeedToKnow.nsWSI);
		if (guidanceNodes != null) {
			guidance = new Guidance[guidanceNodes.size()];
			for (int i=0; i<guidanceNodes.size(); i++) {
				// TODO
				//guidance[i] = guidanceNodes.item(i).getNodeValue();
			}
		}

        if(Wcker.DEBUG > 1) {
            System.out.println("ApplyToItem.toString(): " + toString());
        }
	}

	/**
	 * Get the prompt for this applyToItem.
	 * @return prompt
	 */
	public String getPrompt() {
		return prompt;
	}

	/**
	 * Get the targetHook for this applyToItem.
	 * @return targetHook
	 */
	public String getTargetHook() {
		return targetHook;
	}

	/**
	 * Get the guidance for this applyToItem.
	 * @return guidance
	 */
	public Guidance[] getGuidance() {
		return guidance;
	}

	/**
	 * Get the qGroupRefs for this applyToItem.
	 * @return qGroupRef[]
	 */
	public String[] getQGroupRefs() {
		return qGroupRefs;
	}
	
	public String toString() {
		String thisObject =
		   "\n targetHook: " + targetHook +
		   "\n prompt:     " + prompt +
		   "\n guidance:   " + guidance +
		   "\n qGroupRef:  " + qGroupRefs;
		
		return thisObject;
	}

	
	/**
	 * Create a Composite containing inputs for a organization Item
	 * @param parent Composite
	 * @param cpItem 
	 * @param applyInfo
	 */
	Composite getAsComposite(Composite parent, Element cpItem) {
        if(Wcker.DEBUG > 1) {
            System.out.println("ApplyToItem.getAsComposite()");
            System.out.println("ApplyToqGroupRefs.length: " + qGroupRefs.length);
            System.out.println("ApplyToqGroupRefs: " + qGroupRefs);
        }
        
		Composite applyItemComp = new Composite(parent, SWT.NONE);
		GridLayout theLayout = new GridLayout();
		theLayout.numColumns = 1;
		applyItemComp.setLayout(theLayout);

		// Apply all the qGroupRefs as Composites
		for (int i=0; i<qGroupRefs.length; i++) {
			// get WQG from it's id
			WckerQuestionGroup qGroup = NeedToKnow.getQuestionGroup(qGroupRefs[i]);
			qGroup.getAsComposite(applyItemComp, cpItem);
		}
		
		return applyItemComp;
	}
}
