package uk.ac.ox.tall.wcker.wizard;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.TreeItem;

import org.jdom.Attribute;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;

import uk.ac.ox.tall.wcker.Wcker;
import uk.ac.ox.tall.wcker.wizard.TreeNavigate;

public class EditControlsComp extends Composite {
	NeedToKnow ntk;
	WckerStep step;
	
	Composite questionComp;
	Button next;
	Button prev;

	public EditControlsComp(Composite parent, NeedToKnow ntkArg, WckerStep stepArg) {
		super(parent, SWT.BORDER);
		this.setLayout(new FillLayout());
		
		ntk = ntkArg;
		step = stepArg;
		
		createQuestionComp();
		createPrevNextComp();
	}
	
	private void createPrevNextComp() {
		// Add a Composite to hold the previous/next buttons
		Composite prevNextComp = new Composite(this, SWT.NONE);
		RowLayout pnLayout = new RowLayout();
		pnLayout.pack = false;
		pnLayout.marginTop = 0;
		pnLayout.marginBottom = 2;
		pnLayout.marginLeft = 0;
		pnLayout.marginRight = 0;
		prevNextComp.setLayout(pnLayout);

		GridData pnCompData = new GridData();
		pnCompData.horizontalAlignment = GridData.FILL;
		pnCompData.grabExcessHorizontalSpace = true;
		prevNextComp.setLayoutData(pnCompData);
		
		// Add previous/next buttons
		prev = new Button(prevNextComp, SWT.PUSH);
		prev.setText("Previous item");
		prev.setToolTipText("Click to go back (up the course structure)");
		
		// Add action to button
		prev.addSelectionListener(
                new SelectionListener(){
                    public void widgetSelected(SelectionEvent e) {
                    	// If a TreeItem is selected, move to the previous TreeItem 
                        TreeItem ti = NeedToKnow.guiTree.getSelection()[0];
                        if (ti != null) {
                        	// Find the previous TreeItem. If there is one, move there.
                            TreeItem newSelection = TreeNavigate.docOrderPrev(ti);
                            if (newSelection != null) {
                                NeedToKnow.guiTree.setSelection(new TreeItem[] {newSelection});
        					    // Update the EditControls with the relevant questions
        					    ntk.editControls.updateControls(newSelection);
        					    // Update the TreeButtons
        					    ntk.tbs.updateTreeButtons(newSelection);	
                            }
                            // TODO: disable button when there are no "prev" items
                        }
                    }

					public void widgetDefaultSelected(SelectionEvent arg0) {
					}
                }
		);
		 
		next = new Button(prevNextComp, SWT.PUSH);
		next.setText("Next item");
		next.setToolTipText("Click to go forwards (down the course structure)");
		
		// Add action to button
		next.addSelectionListener(
                new SelectionListener(){
                    public void widgetSelected(SelectionEvent e) {
                    	// If a TreeItem is selected, move to the next TreeItem 
                        TreeItem ti = NeedToKnow.guiTree.getSelection()[0];
                        if (ti != null) {
                    		// Find the next TreeItem. If there is one, move there.
                    		TreeItem newSelection = TreeNavigate.docOrderNext(ti);
                            if (newSelection != null) {
                                NeedToKnow.guiTree.setSelection(new TreeItem[] {newSelection});
        					    // Update the EditControls with the relevant questions
        					    ntk.editControls.updateControls(newSelection);
        					    // Update the TreeButtons
        					    ntk.tbs.updateTreeButtons(newSelection);	
                            }
                            // TODO: disable button when there are no "next" items

                        }
                    }

					public void widgetDefaultSelected(SelectionEvent arg0) {
					}
                }
		);
	}

	private void createQuestionComp() {
		// Add a Composite to hold the questions
		questionComp = new Composite(this, SWT.NONE);
		GridLayout questionCompLayout = new GridLayout();
		questionCompLayout.marginHeight = 0;
		questionCompLayout.marginWidth = 0;
		questionComp.setLayout(questionCompLayout);
		
		GridData qCompData = new GridData();
		qCompData.horizontalAlignment = GridData.FILL;
		qCompData.verticalAlignment = GridData.FILL;
		qCompData.grabExcessHorizontalSpace = true;
		qCompData.grabExcessVerticalSpace = true;
		questionComp.setLayoutData(qCompData);
	}
	
	Composite getQuestionComp() {
		return questionComp;
	}
	
	void clearQuestions() {
		// remove any existing questions, in preparation for adding new ones.
		Control[] oldChildren = questionComp.getChildren();
		for (int i=0; i<oldChildren.length; i++) {
			oldChildren[i].dispose();
		}
	}
	
	/**
	 * Change the EditControls to whatever's appropriate for a new selected TreeItem
	 * @param selectedItem The new selected TreeItem
	 */
	public void updateControls(TreeItem selectedItem) {
	    if(Wcker.DEBUG > 1) {
	        System.out.println("updateEditControls(" + selectedItem + ")");
	    }
		// Remove the old widgets from questionComp.
	    clearQuestions();
		
		// Get useful the relevant CP item
	    Element cpItem = (Element)selectedItem.getData("cpItem");
	    
	    // Add controls for the cpItem
	    createControlsFromCpItem(questionComp, cpItem);
	    
	    // Refresh the widgets displayed
	    questionComp.layout();
	}

	private void createControlsFromCpItem(Composite parent, Element cpItem) {
	    if(Wcker.DEBUG > 1) {
	        System.out.println("createControlsFromCpItem()");
	        System.out.println("cpItem: " + cpItem);
	    }
	    
		XPath qhPath;
    	Attribute qhAttrib = null;
		try {
			qhPath = XPath.newInstance("cp:metadata/wm:wcker/@questionHook");
			qhPath.addNamespace(NeedToKnow.nsCP);
			qhPath.addNamespace(NeedToKnow.nsWM);
			qhAttrib = (Attribute)qhPath.selectSingleNode(cpItem);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
	    String questionHook = qhAttrib.getValue();
		
		// Add new QuestionGroup Composite to EditControls
	    Composite qComp = new Composite(parent, SWT.NONE);
		FillLayout thisLayout = new FillLayout(SWT.VERTICAL);
		qComp.setLayout(thisLayout);
		
		GridData qCompData = new GridData();
		qCompData.horizontalAlignment = GridData.FILL;
		qCompData.verticalAlignment = GridData.FILL;
		qCompData.grabExcessHorizontalSpace = true;
		qCompData.grabExcessVerticalSpace = true;
		qComp.setLayoutData(qCompData);

		// Add the relevant QuestionGroups
		ApplyToItem[] applyItems = step.applyHierChunk.applyItemChunks;

	    if(Wcker.DEBUG > 1) {
	        System.out.println("step.applyHierChunk: " + step.applyHierChunk);
	        System.out.println("applyItems: " + applyItems);
	        System.out.println("applyItems.length: " + applyItems.length);
	    }
		// Find references to any relevant questionGroups
		// for all applyToItem objects, 
		for(int i = 0; i < applyItems.length; i++) {
			ApplyToItem thisApplyToItem = applyItems[i];

		    if(Wcker.DEBUG > 1) {
		        System.out.println("questionHook: " + questionHook);
		        System.out.println("thisApplyToItem: " + thisApplyToItem);
		        System.out.println("thisApplyToItem.getTargetHook(): " + thisApplyToItem.getTargetHook());
		    }
		    
			// If the targetHook matches the current questionHook
			if (thisApplyToItem.getTargetHook().equals(questionHook)) {
				// Add the ApplyItem questions
				thisApplyToItem.getAsComposite(qComp, cpItem);
			}
		}
	}
}
