package uk.ac.ox.tall.wcker.wizard;

import java.util.List;

import org.jdom.Element;
import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.xpath.XPath;

import uk.ac.ox.tall.wcker.Wcker;

/**
 * LoadWizard class - loads the "wizard.wsi" file which describes 
 * a WCKER wizard, and makes an object for {@link Wizard} to
 * use creating the running wizard.
 *                   
 * @author BalchD
 * @author JohnstonD
 *  
 */

public class LoadWizard {
	Document cacheDocument;
	static NeedToKnow ntn;

	private static Namespace nsWSI = Namespace.getNamespace("wsi", "http://www.tall.ox.ac.uk/schema/wcker/0.2");
	private static Namespace nsDC = Namespace.getNamespace("dc", "http://purl.org/dc/elements/1.1/");
	
	Document getDocument() {
		return cacheDocument;
	}
	
	/**
     * convert an XML document to an array of WckerQuestionGroup objects
	 */
	static WckerQuestionGroup[] documentToQuestionGroups(Document d) {
	    // An array of WckerQuestionGroup objects
		WckerQuestionGroup[] w;
		
	    // Create a List of questionGroup nodes
		XPath qgPath;
		List qNodes = null;
		try {
			qgPath = XPath.newInstance("wsi:wizard/wsi:questions/wsi:questionGroup");
			qgPath.addNamespace(nsWSI);
			qNodes = qgPath.selectNodes(d);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		
		if(Wcker.DEBUG > 0) {
			System.out.println("document_to_question_groups " + qNodes.size());
		}

		// Populate array of questionGroups
		w = new WckerQuestionGroup[qNodes.size()] ;
		for(int i = 0; i < qNodes.size(); ++i) {
			 w[i] = new WckerQuestionGroup((Element)qNodes.get(i));
		}

		if(Wcker.DEBUG > 0) {
			System.out.println("w.length: " + w.length );
		}

		return w;
	}
	
	static String getDescFromDoc(Document d) {
		XPath qgPath;
		Element theNode = null;
		try {
			qgPath = XPath.newInstance("wsi:wizard/wsi:metadata/dc:description");
			qgPath.addNamespace(nsWSI);
			qgPath.addNamespace(nsDC);
			theNode = (Element)qgPath.selectSingleNode(d);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		return theNode.getText();
	}
	
	static Element getTemplatesFromDoc(Document d) {
		XPath qgPath;
		Element theNode = null;
		try {
			qgPath = XPath.newInstance("wsi:wizard/wsi:init/wsi:newItemTemplates");
			qgPath.addNamespace(nsWSI);
			theNode = (Element)qgPath.selectSingleNode(d);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		return theNode;
	}
	
	static String getTitleFromDoc(Document d) {
		XPath qgPath;
		Element theNode = null;
		try {
			qgPath = XPath.newInstance("wsi:wizard/wsi:metadata/dc:title");
			qgPath.addNamespace(nsWSI);
			qgPath.addNamespace(nsDC);
			theNode = (Element)qgPath.selectSingleNode(d);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		return theNode.getText();
	}
	/**
	 * Extract the structure part of the .wsi
	 * @param d
	 * @return structureEl Element
	 */
	static Element structureFromDoc(Document d) {
		Element structureEl = null;
		XPath qgPath;
		try {
			qgPath = XPath.newInstance("wsi:wizard/wsi:init/wsi:structure");
			qgPath.addNamespace(nsWSI);
			structureEl = (Element)qgPath.selectSingleNode(d);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		
		return structureEl;
	}
	
	static Element supportFilesFromDocument(Document d) {
		XPath sfPath;
		Element supportFilesNode = null;
		try {
			sfPath = XPath.newInstance("wsi:wizard/wsi:init/wsi:supportFiles");
			sfPath.addNamespace(nsWSI);
			supportFilesNode = (Element)sfPath.selectSingleNode(d);
		} catch (JDOMException e1) {
			e1.printStackTrace();
		}
		
		return supportFilesNode;
	}
	
	static WckerStep[] documentToSteps(Document d, NeedToKnow ntk) {
	    // Get a list of steps in the wizard document
		XPath stepPath;
		List stepNodes = null;
		try {
			stepPath = XPath.newInstance("wsi:wizard/wsi:steps/wsi:step");
			stepPath.addNamespace(nsWSI);
			stepPath.addNamespace(nsDC);
			stepNodes = stepPath.selectNodes(d);
		} catch (JDOMException e1) {
			e1.printStackTrace();
		}

		// Create an array of WckerStep objects, ready to be populated
		int len  = stepNodes.size();
		WckerStep[] wsa = new WckerStep[len];
		if(Wcker.DEBUG > 0) {
			System.out.println("### document_to_steps() ###");
			System.out.println("# steps: " + len);
		}

		for(int k=0; k < len; ++k) {
		    // Add each step to the array
			if(Wcker.DEBUG > 0) {
				System.out.println("-- add step #" + k + " --");
				System.out.println("nl.item(k): " + stepNodes.get(k));
			}
			wsa[k] = new WckerStep((Element)stepNodes.get(k), ntk);
		}
		//nl.item(1).
		return wsa;
	}
}
