package uk.ac.ox.tall.wcker.wizard;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.output.XMLOutputter;
import org.jdom.output.Format;
import org.jdom.xpath.XPath;

import uk.ac.reload.editor.contentpackaging.CP_EditorHandler;
import uk.ac.reload.editor.contentpackaging.datamodel.CP_Resource;
import uk.ac.reload.editor.contentpackaging.xml.*;
import uk.ac.reload.editor.metadata.xml.*;
import uk.ac.reload.jdom.XMLPath;
import uk.ac.reload.moonunit.schema.SchemaElement;
import uk.ac.reload.moonunit.schema.SchemaException;

import uk.ac.ox.tall.wcker.Wcker;
import uk.ac.ox.tall.wcker.util.LoadXML;

/**
 * NeedToKnow class. 
 *            Contains info from "TALL_short-course.wsi" in an application or
 *            programmer friendly form. Basically, after the document is parsed
 *		      it gets translated into a "NeedToKnow" object, which is then
 *		      used to draw the GUI. Basically, the "in memory" form of the XML
 *            is complex and confusing!
 * @author BalchD
 * @author JohnstonD
 *  
 */

public class NeedToKnow {
    
    /* Global wizard info */
    static public String wizardTitle;
    private String description;
    private File _initialDir; // The directory that the wizard's course is first created in.
    private File _workingDir = null; // The directory within which the wizard's course is being worked on.
    private String _exportDir;
	static Namespace nsWSI = Namespace.getNamespace("wsi", "http://www.tall.ox.ac.uk/schema/wcker/0.2");
    static Namespace nsWM = Namespace.getNamespace("wm", "http://www.tall.ox.ac.uk/schema/wcker/wizMeta"); // Namespace for Wcker metadata added to a Content Package
	static Namespace nsDC = Namespace.getNamespace("dc", "http://purl.org/dc/elements/1.1/");
    static Namespace nsCP;
    static Namespace nsMD;
    
    /* Main data */
    static String courseTitle;
    public static Element newItemTemplates;
    WckerStep[] steps; // Each of the steps in the wizard.
    static ContentPackage cp; // An IMS content package - use Reload functions to populate it.
    WckerTree organization; // The hierarchy for this course.
    static WckerQuestionGroup[] questionGroups; // The groups of questions available in this wizard.
    
    static Metadata md;
    
    int stage; // Which wizard step we are currently on at runtime.
    
    /* graphics stuff */
    static public Tree guiTree; // The clickable GUI tree
    EditControlsComp editControls;
    TreeButtons tbs;    
    HelpPanel helpPanel;

    /* Export stuff */
    public static Element supportFiles; // Files to include at export (eg css for HTML templates)
	
	/**
	 * NeedToKnow constructor
	 * @param wizardWorkingDir The directory where the wizard package was unpacked (and contains wizard.wsi)
	 */
	public NeedToKnow(File wizardWorkingDir, String loadType) {
		if(Wcker.DEBUG > 0) {
			System.out.println("# Building NeedToKnow: " + wizardWorkingDir + " #");
		}
		_workingDir = wizardWorkingDir;
		_initialDir = wizardWorkingDir;
		String wizard_file_name = wizardWorkingDir.toString() + System.getProperty("file.separator") + "wizard.wsi";

		Document d = LoadXML.loadXml(wizard_file_name);
		
		wizardTitle = LoadWizard.getTitleFromDoc(d);
		description = LoadWizard.getDescFromDoc(d);
		newItemTemplates = LoadWizard.getTemplatesFromDoc(d);

		supportFiles = LoadWizard.supportFilesFromDocument(d);
		
		questionGroups = LoadWizard.documentToQuestionGroups(d);
		
		// Create IMS CP, using Reload classes
		if (loadType.equals("newpackage")) {
			// Create from .wsi file
			cp = buildCpFromDocument(d, _workingDir);
		} else {
			// Set namespaces from .wsi.
			// There is a risk of conflict with the namespaces loaded from the manifest,
			// but only if there's been deliberate changes to the manifest.
	        setNamespaces(d);
	        
			// Load from previous work.
			File manifest = new File(_workingDir.toString() + System.getProperty("file.separator") + "imsmanifest.xml");
			try {
				cp = new ContentPackage(manifest);
			} catch (JDOMException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SchemaException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		steps = LoadWizard.documentToSteps(d, this);
		
		if(Wcker.DEBUG > 0) {
			System.out.println("# NeedToKnow built #");
		}
	}

	/**
     * Create a new content package in a working directory, derived from the ntk.organization
     *
     */
    public static ContentPackage buildCpFromDocument(Document d, File targetFolder) {
        if(Wcker.DEBUG > 1) {
            System.out.println("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
            System.out.println("Building Content Package in " + targetFolder + ", from Doc " + d);
        }
        
        setNamespaces(d);
        
        //cpFolder - A folder to put the manifest and other files into.
        File cpFolder = targetFolder;
        
        try {
            //TODO: derive which standards we're working with from the Wizard's .wsi file
            //cpController - a pointer to a particuar version of IMS Content Packaging, and Reload mechanisms to handle it
            CP_SchemaController cpController = new CP112_SchemaController();
            //System.out.println(cpController);
            
            //mdController - a pointer to a particuar version of IMS Metadata, and Reload mechanisms to handle it
            MD_SchemaController mdController = new MD121_SchemaController();
            md = new Metadata(false, mdController);
            //System.out.println(mdController);
            
            // The ContentPackage class is where the action happens, with
            // Reload creating a CP manifest file and in-memory representation.
            // Make a content package:
            cp = new ContentPackage(cpFolder, cpController, mdController);

        } catch (Exception e) {
            System.out.println("Couldn't create new CP_EditorHandler");
            e.printStackTrace();
        }
        
        Element cpRoot = cp.getRootElement();
        
        // The target 'organizations' element 
        Element orgsElement = cpRoot.getChild("organizations", nsCP);
        
        // Create a new child 'organization' element
        XMLPath orgPath = new XMLPath("manifest/organizations/organization");
        SchemaElement newOrgElement = (SchemaElement)cp.getSchemaController().getSchemaNode(orgPath);
        // Add the 'organization' element to the target 'organizations' element
        Element orgElement = cp.addElementBySchema(cp, orgsElement, newOrgElement, false);
        
        // Add metadata/lom elements
        Element metadataEl = addBasicMetaData(orgElement);
        
        /**
         * Create an hierarchy from the .wsi wizard/init/organization 
         */
        // Get the organization's top level item
        XPath qgPath;
        Element topItemEl = null;
        try {
        	qgPath = XPath.newInstance("wsi:wizard/wsi:init/wsi:organization/wsi:item");
        	qgPath.addNamespace(nsWSI);
        	topItemEl = (Element)qgPath.selectSingleNode(d);
        	
        } catch (JDOMException e) {
        	e.printStackTrace();
        }
        
        // Set the organization title to the wizard's course title
        courseTitle = topItemEl.getAttributeValue("title");
        orgElement.getChild("title", nsCP).setText(courseTitle);
        
        // Add WCKER info to the new Node
        addWckerInfo(metadataEl, topItemEl);
        
        // Add relevant metadata fields from the .wsi questions
        addMetaFromQuestions(d.getRootElement(), orgElement);
        
        // Find any child items and itemPatterns
        XPath childPath;
        List childItems = null;
        try {
        	childPath = XPath.newInstance("wsi:item | wsi:itemPattern");
        	childPath.addNamespace(nsWSI);
        	childItems = childPath.selectNodes(topItemEl);
        } catch (JDOMException e) {
        	e.printStackTrace();
        }
        
        // iterate over children
        for(int i = 0; i < childItems.size(); i++) {
        	addWsiItems(orgElement, (Element)childItems.get(i), targetFolder.toString());
        }
        
        if(Wcker.DEBUG > 1) {
        	XMLOutputter output = new XMLOutputter(Format.getPrettyFormat());
        	String string = output.outputString(cp.getDocument());
        	System.out.println("-=-=-=-= CP =-=-=-=-");
        	System.out.println(string);
        	System.out.println("-=-=-=-= end CP =-=-=-=-");
        }
        
        //Save our progress
        try {
        	cp.saveDocument();
        } catch (IOException e) {
        	e.printStackTrace();
        }
        
        //Return the content package
        return cp;
    }

    /**
     * Set namespaces from whatever is defined in the .wsi
     *
     */
    private static void setNamespaces(Document d) {
        // Derive which standards we're working with from the Wizard's .wsi file
        XPath containerPath;
        Element schemaElContainer = null;
        XPath metadataPath;
        Element schemaElMetadata = null;
        try {
        	// Container
        	containerPath = XPath.newInstance("wsi:wizard/wsi:init/wsi:schemas/wsi:schema[@role = 'container']");
        	containerPath.addNamespace(nsWSI);
        	schemaElContainer = (Element)containerPath.selectSingleNode(d);
        	
        	// Metadata
        	metadataPath = XPath.newInstance("wsi:wizard/wsi:init/wsi:schemas/wsi:schema[@role = 'metadata']");
        	metadataPath.addNamespace(nsWSI);
        	schemaElMetadata = (Element)metadataPath.selectSingleNode(d);
        	
        } catch (JDOMException e) {
        	e.printStackTrace();
        }

        nsCP = Namespace.getNamespace(schemaElContainer.getAttributeValue("prefix"),
        		schemaElContainer.getAttributeValue("namespace"));
        nsMD = Namespace.getNamespace(schemaElMetadata.getAttributeValue("prefix"),
        		schemaElMetadata.getAttributeValue("namespace"));
		
	}

	/**
     * Add metadata and/or wcker metadata entries for all the questions 
     * applicable to an item Element 
     * @param itemEl
     */
	private static void addMetaFromQuestions(Element wsi, Element itemEl) {
		Element metaEl = itemEl.getChild("metadata", nsCP);
		String questionHook = metaEl.getChild("wcker", nsWM).getAttributeValue("questionHook");
        
		// Lookup questions relevant to this item
		XPath qXPath;
		List qRefs = null;
		try {
			qXPath = XPath.newInstance("//wsi:applyToItem[@targetHook = '" + questionHook + "']/wsi:questionGroup/@ref");
			qXPath.addNamespace(NeedToKnow.nsWSI);
			qRefs = qXPath.selectNodes(wsi);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		
        if(Wcker.DEBUG > 1) {
        	System.out.println("addMetaFromQuestions()");
        	System.out.println("questionHook: " + questionHook);
        	System.out.println("qRefs: " + qRefs);
        }

		
		// Make a list of fields to be added
		LinkedList fields = new LinkedList();
		// Look at all questionGroups that are referenced
		for (int i=0; i<qRefs.size(); i++) {
			WckerQuestionGroup thisQGroup = getQuestionGroup(((Attribute)qRefs.get(i)).getValue());
			
			// Look at all questions in a questionGroup
			for (int j=0; j<thisQGroup.getLength(); j++) {
				WckerQuestion thisQuestion = thisQGroup.getQuestion(j);

				// Look at all inputs in a question
				for (int k=0; k<thisQuestion.inputs.length; k++) {
					WckerInput thisinput = thisQuestion.inputs[k];
				
					// If the input isn't already on the list, add it.
					if (!fields.contains(thisinput)) {
						// Ignore cp:title targets
						if (!((WckerInput)thisinput).getTarget().equals("cp:title")) {
							fields.add(thisinput);
						}
					}
				}
			}
		}
		
        if(Wcker.DEBUG > 1) {
        	System.out.println("fields.size(): " + fields.size());
        }
		
		// Add the fields
        for (int i=0; i<fields.size(); i++) {
        	addMetaField(itemEl, (WckerInput)fields.get(i));
        }
        
        if(Wcker.DEBUG > 1) {
        	System.out.println("END addMetaFromQuestions()");
        }
	}

	/**
	 * Add a metadata field to a given itemEl, based on the given WckerInput.
	 * @param currentElement
	 * @param input
	 */
	private static void addMetaField(Element currentElement, WckerInput input) {
		addMetaField(currentElement, input, input.getTarget());
	}
	private static void addMetaField(Element currentElement, WckerInput input, String target) {
		if(Wcker.DEBUG > 1) {
			System.out.println("");
			System.out.println("DB: addMetaField()");
			System.out.println("DB: currentElement: " + currentElement);
			//System.out.println("DB: input: " + input);
			System.out.println("DB: target: " + target);
		}

		// Drill down to any existing version of the target element
		// Split the target xpath into elements
		String[] pathElements = target.split("/");
		int numPathElements = pathElements.length;
		int pos = pathElements[0].indexOf(":");
		String firstTarget = pathElements[0].substring(pos + 1);
		String prefix = pathElements[0].substring(0, pos);
    	Namespace ns = getNamespaceFromPrefix(prefix);
		if(Wcker.DEBUG > 1) {
			System.out.println("DB: pathElements.length: " + numPathElements);
			//System.out.println("DB: pathElements[0]: " + pathElements[0]);
			System.out.println("DB: firstTarget: " + firstTarget);
		}
		
		// Find the target
		Element targetEl = currentElement.getChild(firstTarget, ns);
		
		if(Wcker.DEBUG > 1) {
			System.out.println("DB: targetEl: " + targetEl);
		}

		// If target doesn't exist, create it.
		if (targetEl == null) {
			targetEl = new Element(firstTarget, ns);
			currentElement.addContent(targetEl);
			if(Wcker.DEBUG > 1) {
				System.out.println("DB: targetEl '" + firstTarget + "' didn't exist, tried creating it:");
				System.out.println("DB: targetEl: " + targetEl);
			}
		}
				
		// If more elements in target, move into the new element and recurse
		if (numPathElements > 1) {
			String remainingTarget = "";
			for (int i = 1; i < numPathElements; i++) {
				// NB i = 1, as we don't want to include the element we're about to recurse into 
				remainingTarget += pathElements[i];
				if (i < pathElements.length -1) {
					remainingTarget += "/";
				}
			}
			addMetaField(targetEl, input, remainingTarget);
		} else {
			// Add details for the final element of this target
			// Add any attributes needed.
			List outputAttribs = input.getOutputAttribs();
			for (int i=0; i<outputAttribs.size(); i++) {
				String nameField = ((Element)outputAttribs.get(i)).getAttributeValue("name");
				
				int attribSepPos = nameField.indexOf(":");
				// Have attrib name with prefix, so fix it up
				String attribName = nameField.substring(attribSepPos + 1);
				String attribPrefix = nameField.substring(0, attribSepPos);
				Namespace attribNs = getNamespaceFromPrefix(attribPrefix);
				String value = ((Element)outputAttribs.get(i)).getAttributeValue("value");
				targetEl.setAttribute(attribName, value, attribNs);
			}
			
			// Add any values provided
			if (!input.getDefaultValue().equals("")) {
				targetEl.addContent(input.getDefaultValue());
			}
		}
	}

	public static Element addBasicMetaData(Element targetElement) {
        // Add metadata element
        XMLPath metaPath = new XMLPath("manifest/organizations/organization/metadata");
        SchemaElement newMetaElement = (SchemaElement)cp.getSchemaController().getSchemaNode(metaPath);
        Element metadataEl = cp.addElementBySchema(cp, targetElement, newMetaElement, false);

        // Add lom element
        XMLPath lomPath = new XMLPath("lom");
        SchemaElement newMetaContribEl = (SchemaElement)md.getSchemaController().getSchemaNode(lomPath);
        md.addElementBySchema(cp, metadataEl, newMetaContribEl, false);
    	
    	return metadataEl;
    }
    
    /**
     * Add Wcker item information in a new metadata element
     * @param metadataEl
     * @param topItemEl
     */
    private static void addWckerInfo(Element metadataEl, Element ItemEl) {
    	Element wckerEl = new Element("wcker", nsWM);
    	metadataEl.addContent(wckerEl);
    	
    	wckerEl.setAttribute("itemType", ItemEl.getAttributeValue("itemType"));
    	wckerEl.setAttribute("typeLabel", ItemEl.getAttributeValue("typeLabel"));
    	wckerEl.setAttribute("questionHook", ItemEl.getAttributeValue("questionHook"));

        // Get any allowedChildren attribute on the wsi item
        if (ItemEl.getAttributeValue("allowedChildren") != null) {
        	// If there is no allowedChildren, this item is probably a page
        	wckerEl.setAttribute("allowedChildren", ItemEl.getAttributeValue("allowedChildren"));
        }
        // Get any src attributes on the wsi item
        if (ItemEl.getAttributeValue("src") != null) {
        	// If there are no templates refs, this item is a directory
        	wckerEl.setAttribute("src", ItemEl.getAttributeValue("src"));
        }
        // Get any templatesrc attributes on the wsi item
        if (ItemEl.getAttributeValue("templatesrc") != null) {
        	// If there are no templates refs, this item is a directory
        	wckerEl.setAttribute("templatesrc", ItemEl.getAttributeValue("templatesrc"));
        }
	}

	/**
     * Recursively add .wsi item and itemPatterns as CP items
     * @param parentCpItem The CP Element to which we add new children items
     * @param newWsiItem The .wsi item/itemPattern node to add
     */
    static Element addWsiItems(Element parentCpItem, Element newWsiItem, String parentPath) {
        if(Wcker.DEBUG > 1) {
            System.out.println("addWsiItems()");
            System.out.println("newWsiItem: " + newWsiItem);
            System.out.println("newWsiItem.getName(): " + newWsiItem.getName());
        }
        Element newItem = null;

        // Set the new Node title from the wsiItem
        String thisTitle = newWsiItem.getAttributeValue("title");
		List childItems = null;
        
    	// Distinguish between items and itemPatterns
    	if (newWsiItem.getName().equals("item") || newWsiItem.getName().equals("template")) {
    		/* This is an item or an item template - add it directly */

	        // Create the resource and add this item
    		Element newCpItem = null;
    		File thisFile = null;
    		String fileExt = WckerUtility.getTemplateFileExt(newWsiItem);
    		if (fileExt.equals("directory")) {
        		// If it's a directory, just add as an item
    	        XMLPath childXmlPath = new XMLPath("manifest/organizations/organization/item");
    	        SchemaElement newCpEl = (SchemaElement)cp.getSchemaController().getSchemaNode(childXmlPath);
    	        newCpItem = cp.addElementBySchema(cp, parentCpItem, newCpEl, false);

		        thisFile = new File(parentPath + 
		        		System.getProperty("file.separator") + 
		        		WckerUtility.formatTitleToPath(thisTitle)
		        );
    			
    		} else {
    			// If there's a file/template, add as a resource
		        CP_Resource thisResource;
		        thisFile = new File(parentPath + 
		        		System.getProperty("file.separator") + 
		        		WckerUtility.formatTitleToPath(thisTitle) +
		        		fileExt
		        );
		        thisResource = new CP_Resource(thisFile);
		        newCpItem = cp.addCP_Resource(thisResource, parentCpItem);
    		}

            if(Wcker.DEBUG > 1) {
            	System.out.println("parentPath: " + parentPath);
            	System.out.println("thisFile: " + thisFile);
            }
            
	        // Set the new Node title from the wsiItem
	        newCpItem.getChild("title", nsCP).setText(thisTitle);
	        
            // Add metadata/lom elements
            Element metadataEl = addBasicMetaData(newCpItem);
            
	    	// Add WCKER info to the new Node
            addWckerInfo(metadataEl, newWsiItem);
            
            // Add relevant metadata fields from the .wsi questions
            addMetaFromQuestions(newWsiItem, newCpItem);

	    	// Find any child items and itemPatternsXPath childPath;
    		XPath childPath;
    		try {
    			childPath = XPath.newInstance("wsi:item | wsi:itemPattern");
    			childPath.addNamespace(nsWSI);
    			childItems = childPath.selectNodes(newWsiItem);
    		} catch (JDOMException e) {
    			e.printStackTrace();
    		}
    		
    		// Set return value;
    		newItem = newCpItem;
    		
			// iterate over children
	        for(int i = 0; i < childItems.size(); i++) {
	           addWsiItems(newCpItem, (Element)childItems.get(i), thisFile.toString());
	    	}
    	} else if (newWsiItem.getName().equals("itemPattern")) {
    		/* This is an itemPattern, create 'count' items based on it (using the 'count' attribute). */
    		int count = Integer.parseInt(newWsiItem.getAttributeValue("count"));
	        
    		for(int i=0; i < count; i++) {
    	        String localTitle = thisTitle;
				// If localTitle contains the substring, replace it with the itemCount. 
				if (localTitle.contains("%c")) {
					String replaceFrag = new Integer(parentCpItem.getChildren("item", nsCP).size() + 1).toString();
					localTitle = localTitle.replace("%c", replaceFrag);
				}

		        // Create the resource and add this item
	    		Element newCpItem = null;
	    		File thisFile = null;
	    		String fileExt = WckerUtility.getTemplateFileExt(newWsiItem);
	    		if (fileExt.equals("directory")) {
	    			
	        		// If it's a directory, just add as an item
	    	        XMLPath childXmlPath = new XMLPath("manifest/organizations/organization/item");
	    	        SchemaElement newCpEl = (SchemaElement)cp.getSchemaController().getSchemaNode(childXmlPath);
	    	        newCpItem = cp.addElementBySchema(cp, parentCpItem, newCpEl, false);

			        thisFile = new File(parentPath + 
			        		System.getProperty("file.separator") + 
			        		WckerUtility.formatTitleToPath(localTitle)
			        );
	    			
	    		} else {
	    			// If there's a file/template, add as a resource
			        CP_Resource thisResource;
			        thisFile = new File(parentPath + 
			        		System.getProperty("file.separator") + 
			        		WckerUtility.formatTitleToPath(localTitle) +
			        		fileExt
			        );
			        thisResource = new CP_Resource(thisFile);
			        newCpItem = cp.addCP_Resource(thisResource, parentCpItem);
	    		}
	    		
    	        newCpItem.getChild("title", nsCP).setText(localTitle);

                // Add metadata element
                XMLPath metaPath = new XMLPath("manifest/organizations/organization/metadata");
                SchemaElement newMetaElement = (SchemaElement)cp.getSchemaController().getSchemaNode(metaPath);
                Element metadataEl = cp.addElementBySchema(cp, newCpItem, newMetaElement, false);
                
    	    	// Add WCKER info to the new Node
                addWckerInfo(metadataEl, newWsiItem);

                // Add relevant metadata fields from the .wsi questions
                addMetaFromQuestions(newWsiItem, newCpItem);
                
    	    	// Find any child items and itemPatterns
        		XPath childPath;
        		try {
        			childPath = XPath.newInstance("wsi:item | wsi:itemPattern");
        			childPath.addNamespace(nsWSI);
        			childItems = childPath.selectNodes(newWsiItem);
        		} catch (JDOMException e) {
        			e.printStackTrace();
        		}

    			// iterate over children
    	        for(int j = 0; j < childItems.size(); j++) {
    	           addWsiItems(newCpItem, (Element)childItems.get(j), thisFile.toString());
    	    	}
    		}
    	}
    	
    	return newItem;
    }

	/**
	* a dump of everything in a NeedToKnow object
	*
	*/
	public String toString() {
		String acc = "";

	if(Wcker.DEBUG > 0) {
		acc += "needtoknow start ====================++++++ |" + wizardTitle + "|";
		acc +=  wizardTitle + "\n";
		for(int i=0;i<steps.length;++i) {
			acc += steps[i] + "\n";
		}
		acc += ".....................................\n";
		for(int i=0;i<steps.length;++i) {
			acc += steps[i] + "\n";
			acc += ".....................................\n";
		}

		for(int i=0;i<questionGroups.length;++i) {
			acc += questionGroups[i];
		}
		acc += "needtoknow stop ====================++++++ |" + wizardTitle + "|";
	}
		return acc;
	}

	/**
	 * Return the WckerQuestionGroup with the given id.
	 * @param id The WckerQuestionGroup id to look-up.
	 */
	public static WckerQuestionGroup getQuestionGroup(String id) {
	    WckerQuestionGroup theGroup = null;
	    
	    for(int i=0; i<questionGroups.length; i++) {

			if(Wcker.DEBUG > 1) {
				//System.out.println("questionGroups["+i+"]: " + questionGroups[i]);
				System.out.println("id: " + id);
				System.out.println("questionGroups["+i+"].id: " + questionGroups[i].id);
				System.out.println("questionGroups[i].id.equals(id): " + questionGroups[i].id.equals(id));
			}
			
	        if(questionGroups[i].id.equals(id)) {
	            theGroup = questionGroups[i];
	        }
	    }
	    return theGroup;
	}
	
	/**
	 * Return the Directory that the wizard's course was started in.
	 * @return
	 */
	public File getInitialDir() {
		return _initialDir;
	}
	
	/**
	 * Return the Directory within which the wizard's course is being worked on
	 * Regardless of whether the user has saved the wizard in progress to a
	 * Separate directory.
	 * @return
	 */
	public File getOperatingDir() {
		if(Wcker.DEBUG > 1) {
			System.out.println("getOperatingDir():");
			System.out.println("_workingDir: " + _workingDir);
			System.out.println("_initialDir: " + _initialDir);
		}
		File theDir;
		if (_workingDir != null) {
			theDir = _workingDir;
		} else {
			theDir = _initialDir;
		}
		return theDir;
	}

	/**
	 * Set the Directory that the wizard will be exported to
	 * @return
	 */
	public void setExportDir(String exportDir) {
		_exportDir = exportDir;
	}
	
	/**
	 * Return the Directory that the wizard will be exported to
	 * @return
	 */
	public String getExportDir() {
		return _exportDir;
	}

	/**
	 * Find the all Orgs and Items that have a matching questionHook
	 * @param targetHook
	 * @return
	 */
	public List getHierItemsByQuestionHook(String targetHook) {
		List items = null;
		
        if(Wcker.DEBUG > 1) {
            System.out.println("getHierItemByQuestionHook()");
            System.out.println("targetHook: " + targetHook);
            System.out.println("cp: " + cp);
        }
        
		Element orgs = cp.getRootElement().getChild("organizations", nsCP);
		
		XPath itemsPath;
		String xpathString = "descendant::*[cp:metadata/wm:wcker/@questionHook = '" + targetHook + "']";
		try {
			itemsPath = XPath.newInstance(xpathString);
			itemsPath.addNamespace(nsCP);
			itemsPath.addNamespace(nsWM);
			items = itemsPath.selectNodes(orgs);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
        
		return items;
	}

	/**
	 * Find the first Org or Item that has a matching questionHook
	 * @param targetHook
	 * @return
	 */
	public Element getHierItemByQuestionHook(String targetHook) {
		return (Element)getHierItemsByQuestionHook(targetHook).get(0);
	}

	/**
	 * Get the .wsi item type from a TreeItem's CP item 
	 * @param ti
	 * @return
	 */
	String getItemType(TreeItem ti) {
		Element cpItem = (Element)ti.getData("cpItem");
		return getItemType(cpItem);
	}

	/**
	 * Get the .wsi item type from a TreeItem's CP item 
	 * @param cpItem
	 * @return
	 */
	String getItemType(Element cpItem) {
		String itemType = null;
		XPath typePath;
		try {
			typePath = XPath.newInstance("cp:metadata/wm:wcker/@itemType");
			typePath.addNamespace(NeedToKnow.nsCP);
			typePath.addNamespace(NeedToKnow.nsWM);
			itemType = ((Attribute)typePath.selectSingleNode(cpItem)).getValue();
		} catch (JDOMException err) {
			err.printStackTrace();
		}
		return itemType;
	}

	private static Namespace getNamespaceFromPrefix(String prefix) {
		Namespace ns = null;
    	if  (prefix.equals("xml")) {
    		ns = org.jdom.Namespace.XML_NAMESPACE;
    	} else if(prefix.equals(NeedToKnow.nsCP.getPrefix())) {
    		ns = NeedToKnow.nsCP;
    	} else if (prefix.equals(NeedToKnow.nsMD.getPrefix())) {
    		ns = NeedToKnow.nsMD;
    	} else if (prefix.equals(NeedToKnow.nsWM.getPrefix())) {
    		ns = NeedToKnow.nsWM;
    	} else if (prefix.equals(NeedToKnow.nsDC.getPrefix())) {
    		ns = NeedToKnow.nsDC;
    	}else if (prefix.equals(NeedToKnow.nsWSI.getPrefix())) {
    		ns = NeedToKnow.nsWSI;
    	}
		return ns;
	}

	void addTreeListener(WckerStep step) {
		// Listen for selections on the Tree
		NeedToKnow.guiTree.addSelectionListener (
				new SelectionListener() {
				    // Tree listener
					public void widgetSelected(SelectionEvent e) {
					    // a tree item selected
					    if(Wcker.DEBUG > 1) {
					        System.out.println("Selected: " + e.item);
					        System.out.println("Data: " + e.item.getData("cpItem"));
						}
					    
					    // Update the EditControls with the relevant questions
					    editControls.updateControls((TreeItem)e.item);
					    
					    // Update the TreeButtons
					    tbs.updateTreeButtons((TreeItem)e.item);
						
					    // TODO: Update the guidance
						//helpPanel.updateTreeGuidance(step, (Element)e.item.getData("cpItem"));		
						
					}
					public void widgetDefaultSelected(SelectionEvent e)  {
						if(Wcker.DEBUG > 1) {
							System.out.println("Default Selection Event");
						}
					}
				}
		);
	}
}