package uk.ac.ox.tall.wcker.wizard;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import uk.ac.ox.tall.wcker.Wcker;

/**
 * OrganisationPayLoad class - Contains info for each node in an organisation WckerTree
 *                
 * @author JohnstonD   
 * @author BalchD
 *  
 */
public class OrganisationPayLoad {
	public String nodename; // <name>
	public String originalNodeName; // <name>
	public String type; // e.g. "unit", "course", "page"
	String src;
	String templatesrc;
	String templatetype;
	String questionHook;

	/**
	 * Constructor from a String array
	 * @param arg
	 */
	OrganisationPayLoad(String[] arg) {
		nodename 	= arg[0]; // <name>
		type		= arg[1]; // e.g. "unit", "course", "page"
		src		    = arg[2];
		templatesrc	= arg[3];
		templatetype = arg[4];
		questionHook = arg[5];
		
		if(Wcker.DEBUG > 1) {
			System.out.println("Created new OrganisationPayLoad: " + this.hashCode());
			System.out.println(toString());
		}
	}
	
	/**
	 * Constructor from a item Node, with an itemCount string (what number the item is in a sequence)
	 * @param n
	 */
	OrganisationPayLoad(Node n, int dupeCount) {
		// Get the attributes from this node
		NamedNodeMap nnm = n.getAttributes();
		
		// Put the attribute values in the relevant variables
		for (int i=0; i<nnm.getLength(); i++) {
			Node thisNode = nnm.item(i);
			if (Wcker.DEBUG > 0) {
				System.out.println("  " + thisNode.getNodeName() + ": " + thisNode.getNodeValue());
			}
			if (thisNode.getNodeName().equals("title")) {
				String thisString = thisNode.getNodeValue();
				System.out.println("originalNodeName: " + originalNodeName);
				// If we are creating a new payload (originalNodeName is empty)
				// fill it with the title. This allows us to preserve any replacement strings.
				if (originalNodeName == null) {
					originalNodeName = thisString;
				}
				// If thisString contains the substring, replace it with the itemCount. 
				if (originalNodeName.contains("%c")) {
					String replaceFrag = new Integer(dupeCount).toString();
					thisString = originalNodeName.replace("%c", replaceFrag);
				}
				this.nodename = thisString;
			}
			else if (thisNode.getNodeName().equals("type")) {
				this.type = thisNode.getNodeValue();
			}
			else if (thisNode.getNodeName().equals("src")) {
				this.src = thisNode.getNodeValue();
			}
			else if (thisNode.getNodeName().equals("templatesrc")) {
				this.templatesrc = thisNode.getNodeValue();
			}
			else if (thisNode.getNodeName().equals("templatetype")) {
				this.templatetype = thisNode.getNodeValue();
			}
			else if (thisNode.getNodeName().equals("questionHook")) {
				this.questionHook = thisNode.getNodeValue();
			}
		}
		
		if(Wcker.DEBUG > 0) {
			System.out.println("Created new OrganisationPayLoad: " + this.hashCode());
			System.out.println(toString());
		}
	}

	/**
	 * Duplicate this OrganisationPayload
	 * @return
	 */
	OrganisationPayLoad dup() {
		String[] a = new String[6];
		a[0] = new String(nodename);
		a[1] = new String(type);
		a[2] = new String(src);
		a[3] = new String(templatesrc);
		a[4] = new String(templatetype);
		a[5] = new String(questionHook);
		return new OrganisationPayLoad(a);
	}
	
	protected Object clone() {
		return (Object) dup();
	}

	/**
	 * @return A String of node details, for debugging. 
	 */
	public String toString() {
		return  "nodename="	+ nodename +
			", originalNodeName=" + originalNodeName + 
			", type=" + type + 	
			", src=" + src + 
			", templatesrc=" + templatesrc + 
			", templatetype=" + templatetype +
			", questionHook=" + questionHook + "\n";
	}

	/**
	 * @return The String naming this node. 
	 */
	public String getName() {
		return nodename;
	}
	
	/**
	 * @return The String describing the type of node this is.
	 */
	public String getType() {
		return type;
	}
	
	/**
	 * @return The questionHook identifier String.
	 */
	public String getQuestionHook() {
		return questionHook;
	}
}
