package uk.ac.ox.tall.wcker.wizard;

import java.util.List;

import org.eclipse.swt.widgets.TreeItem;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;

import uk.ac.ox.tall.wcker.Wcker;

class TreeAction {
    
    /**
     * Add a new item to the Content Package and the guiTree
     * @param contextItem
     * @param childData
     */
	static void addNewChild(NeedToKnow ntk, TreeItem contextItem, String[] childData) {
        if (Wcker.DEBUG > 0) {
            System.out.println("Hit button: Add new " + childData[0] + " - "  + childData[1]);
        }
        
        // Create and add item after menu choice
        Element contextElement = (Element)contextItem.getData("cpItem");
        
        Element newWsiItem = null;
		XPath qgPath;
		try {
			qgPath = XPath.newInstance("wsi:template[@id='" + childData[1] + "']");
			qgPath.addNamespace(NeedToKnow.nsWSI);
			newWsiItem = (Element)qgPath.selectSingleNode(NeedToKnow.newItemTemplates);
		} catch (JDOMException err) {
			err.printStackTrace();
		}
        
		// Get a useful value for parentPath
		String parentPath = ntk.getOperatingDir() + System.getProperty("file.separator") + CPItemPath(contextElement, "");
		//System.out.println("parentPath: " + parentPath);
		
        Element cpItem = NeedToKnow.addWsiItems(contextElement, newWsiItem, parentPath);
		WckerPage.addTreeItemFromCpItem(contextItem, cpItem);
	}
	
	/**
	 * Create a string representing the file path for a branch of the item hierarchy
	 * @param contextElement
	 * @return
	 */
	private static String CPItemPath(Element contextElement, String pathSoFar) {
		// Make path from the current element title
		
		if (!contextElement.getName().equals("organization")) {
			
			String thisPathFrag = WckerUtility.formatTitleToPath(
					contextElement.getChildText("title", NeedToKnow.nsCP)
					+ System.getProperty("file.separator"));
			
			pathSoFar = thisPathFrag + pathSoFar;
			
			Element parent = contextElement.getParentElement();
			if (parent != null) {
				// If there's a parent element, recurse
				CPItemPath(parent, pathSoFar);
			} else {
				return pathSoFar;
			}
		}
		
		return pathSoFar;
	}

	/**
	 * Delete an item from both the manifest and the gui tree.
	 * @param ti
	 */
	public static void delete(TreeItem ti) {
		if(Wcker.DEBUG > 1) {
		    System.out.println("\ndelete(TreeItem): " + ti);
		}

		// Delete any child elements
		TreeItem[] children = ti.getItems();
		for (int i = 0; i<children.length; i++) {
			delete((TreeItem)children[i]);
		}
		
		// Get the CP manifest's item
		Element CpItem = (Element)ti.getData("cpItem");
		
		// Get the CP manifest's resource
		String resRef = CpItem.getAttributeValue("identifierref");
		
		if (resRef != null) {
			XPath resPath;
			Element resource = null;
			try {
				resPath = XPath.newInstance("/cp:manifest/cp:resources/cp:resource[@identifier = '" + resRef + "']");
				resPath.addNamespace(NeedToKnow.nsCP);
				resource = (Element)resPath.selectSingleNode(CpItem);
			} catch (JDOMException e) {
				e.printStackTrace();
			}
			
			if(Wcker.DEBUG > 1) {
				System.out.println("CpItem: " + CpItem);
				System.out.println("CpItem.getParentElement();: " + CpItem.getParentElement());
				System.out.println("resRef: " + resRef);
				System.out.println("resource: " + resource);
			}
			
			// Delete the manifest item's resource
			// Find out how many references there are to the resource
			XPath refsPath;
			List refs = null;
			try {
				refsPath = XPath.newInstance("//cp:item[@identifierref = '" + resRef + "']");
				refsPath.addNamespace(NeedToKnow.nsCP);
				refs = (List)refsPath.selectNodes(CpItem);
			} catch (JDOMException e) {
				e.printStackTrace();
			}
			
			// If there are no more than one reference (the one we're about to delete), remove the resource.
			if (refs.size() > 1) {
				NeedToKnow.cp.removeElement(null, resource);
			}
		}

		// Delete the manifest item
		NeedToKnow.cp.removeElement(null, CpItem);
		
		// Delete the guiTree's item
		ti.dispose();
	}
}
