package uk.ac.ox.tall.wcker.wizard;

import java.util.LinkedList;
import java.util.List;

import uk.ac.ox.tall.wcker.Wcker;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.graphics.*;

import org.jdom.Attribute;
import org.jdom.Element;
import org.jdom.Content;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;

public class TreeButtons extends Composite{	
    Button[] treeEditButtons = new Button[6];
    final String[] names = {"addContainer", "delContainer", "addLeaf", "delLeaf", "up", "down"};
    NeedToKnow ntk;
    int stepNumber;
    boolean self_insertion_mode;

    /**
     * Constructor for Tree editing buttons, extended from Composite
     * @param stepNumberArg
     * @param ntkArg
     * @param parent
     */
    public TreeButtons(int stepNumberArg, NeedToKnow ntkArg, Composite parent) {
		super(parent, SWT.NONE);
    	stepNumber = stepNumberArg;
    	ntk = ntkArg;
    	
    	// Layout for the Composite which contains the editing buttons
    	FillLayout buttonLayout = new FillLayout(SWT.HORIZONTAL);
        this.setLayout(buttonLayout);
    	
        addButtons(this);
    }

    /**
     * Add buttons to the TreeButtons Composite.
     * @param parent Composite to which buttons are added
     */
    private void addButtons(final Composite parent) {
        // Add buttons to the treeEditButtons array
        for(int i = 0;i < names.length; ++i) {
            final String buttonType = names[i];
            
            // Create and format myButton
            final Button myButton = new Button(parent, SWT.FLAT);
            // XXX Why doesn't this set the background colour?
            // XXX Could be a platform dependant problem.
            myButton.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WHITE));
            //myButton.setBackground(new Color(Display.getDefault(), 255, 255, 255));
            Image buttonImage = new Image(Display.getDefault(), (Wcker.wckerProps.getProperty("wcker.gfx.dir") + buttonType + ".png"));
            myButton.setImage(buttonImage);
            // Set tooltip
            myButton.setToolTipText(getTreeToolTip(buttonType));

            // Add myButton to the array
            treeEditButtons[i] = myButton;
            
            // Setup choice popups for add item buttons
            if (buttonType.equals("addLeaf") || buttonType.equals("addContainer")) {
                myButton.addMouseListener (	
                        new MouseAdapter() {
                            public void mouseDown(MouseEvent e) {
                            	TreeItem contextItem = NeedToKnow.guiTree.getSelection()[0];
                            	String[][] allowedChildren = getItemAllowedChildren(contextItem);

                    			// Filter out inapropriate children
                    			// (e.g. no containers when using the addLeaf button)
                                LinkedList allowedList = new LinkedList();
                        		for (int i=0; i<allowedChildren.length; i++) {
                                    if (buttonType.equals("addLeaf") && allowedChildren[i][0].equals("leaf")) {
                                    	allowedList.add(allowedChildren[i]);
                        			} else if (buttonType.equals("addContainer") && allowedChildren[i][0].equals("container")){
                                    	allowedList.add(allowedChildren[i]);
                        			}
                        		}
                                
                            	// Check number of types of children allowed.
                            	int num = allowedList.size();
                            	if (num > 1) {
                            		// Create a menu containing a button to add each child type allowed
                            		Menu popupmenu = new Menu(myButton);
                            		for (int i=0; i<num; i++) {
                            			// Add each child type
                            			String[] thisChildType = (String[])allowedList.get(i);
                                        MenuItem childButton = new MenuItem(popupmenu,  SWT.PUSH);
                                        childButton.setData("children", thisChildType);
                                        childButton.setData("contextItem", contextItem);
                                        childButton.setText("Add new " + thisChildType[1]);
                                        
                                        // Add action to the button
                                        childButton.addListener (
                                                SWT.Selection, 
                                                new Listener() {
                                                    public void handleEvent(Event e) {
                                                        MenuItem mi = (MenuItem)e.widget;
                                                        String[] childData = (String[])mi.getData("children");
                                                        TreeItem contextItem = (TreeItem)mi.getData("contextItem");
                                                        TreeAction.addNewChild(ntk, contextItem, childData);
                                                        contextItem.setExpanded(true);
                                                    }
                                                }
                                        );
                            		}
                            		
                            		// Show the popup
                                    popupmenu.setEnabled(true);
                                    popupmenu.setVisible(true);
                            	}
                            }
                        }
                );
            }
            
            // Listen for clicks on the buttons
            myButton.addSelectionListener ( 
                    new SelectionListener() {
                        public void widgetSelected(SelectionEvent e) {
                        	// one of 6 buttons selected
                        	TreeItem contextItem = NeedToKnow.guiTree.getSelection()[0];
                        	String[][] allowedChildren = getItemAllowedChildren(contextItem);
                        	
                            if (contextItem != null) {
                            	String itemType = ntk.getItemType(contextItem);
                            	
                            	if (buttonType.equals("up") || buttonType.equals("down")) {
                            		// Move the item
                            		//System.out.println("contextItem.getExpanded(): " + contextItem.getExpanded());
                            		moveGuiTreeItem(contextItem, buttonType);
                            		
                            	}

                                if (buttonType.equals("addLeaf") || buttonType.equals("addContainer")) {
                                    // This should only be activated if there is only one type of 
                                	// child allowed (hence no popup menu).

                        			// Filter out inapropriate children
                        			// (e.g. no containers when using the addLeaf button)
                                    LinkedList allowedList = new LinkedList();
                            		for (int i=0; i<allowedChildren.length; i++) {
                                        System.out.println("allowedChildren["+i+"]: " + allowedChildren[i][0] + ":" + allowedChildren[i][1]);
                                        if (buttonType.equals("addLeaf") && allowedChildren[i][0].equals("leaf")) {
                                        	allowedList.add(allowedChildren[i]);
                            			} else if (buttonType.equals("addContainer") && allowedChildren[i][0].equals("container")){
                                        	allowedList.add(allowedChildren[i]);
                            			}
                            		}
                            		
                            		// Add the child
                            		TreeAction.addNewChild(ntk, contextItem, (String[])allowedList.get(0));
                                	contextItem.setExpanded(true);

                                }
                                
                                if (buttonType.equals("delContainer") && itemType.equals("container")) {
                                    TreeAction.delete(contextItem);
                                } else if (buttonType.equals("delLeaf") && itemType.equals("leaf")) {
                                    TreeAction.delete(contextItem);
                                }
                            }
                        }

						public void widgetDefaultSelected(SelectionEvent e) {
                        }
                    }
            );
        }
    }

    /**
     * Return a tooltip String for TreeButtons, based on the buttonType and structure labels
     * @param buttonType
     * @return
     */
    private String getTreeToolTip(String buttonType) {
		// TODO: Include multiple child options
		// TODO: incorporate structure labels
    	String tip = "";
    	if (buttonType.equals("up")) {
    		tip = "Move selection up";
    	} else if (buttonType.equals("down")) {
    		tip = "Move selection down";
    	} if (buttonType.equals("addLeaf")) {
    		tip = "Add page";
    	} if (buttonType.equals("delLeaf")) {
    		tip = "Delete page";
    	} if (buttonType.equals("addContainer")) {
    		tip = "Add section";
    	} if (buttonType.equals("delContainer")) {
    		tip = "Delete section and contents";
    	}
    	
    	return tip;
	}

	/**
     * Move a TreeItem and its corresponding manifest entry up of down,
     * adhereing to the rules in the .wsi structure section
     * @param item
     * @param direction
     */
    private void moveGuiTreeItem(TreeItem ti, String direction) {
    	//TreeItem parent = ti.getParentItem();
    	TreeItem destTreeItem = null;
    	
		if (direction.equals("up")) {
			// Move the item up
			destTreeItem = TreeNavigate.getPreviousSibling(ti);
		} else {
			// Move the item down
			destTreeItem = TreeNavigate.getFollowingSibling(ti);
		}
		
		if (destTreeItem == null) {
			// TODO: No valid sibling, go up a level and try there
			// For the moment we disallow moving to a different parent
		}

		copyGuiTree(ti, destTreeItem, direction);
	}
	
	/**
	 * Clone a GUI Tree, and move a given TreeItem above or below a reference TreeItem.
	 * @param origTI
	 * @param RefTI
	 * @param direction
	 * @return
	 */
	private void copyGuiTree(TreeItem movingTI, TreeItem refTI, String direction) {
		// Get details from the original tree, ready to transfer to a duplicate Tree
		Tree origTree = NeedToKnow.guiTree;
		
		Tree newTree = new Tree(origTree.getParent(), SWT.NONE);
		GridData treeGridData = new GridData();
		treeGridData.horizontalAlignment = GridData.FILL;
		treeGridData.verticalAlignment = GridData.FILL;
		treeGridData.grabExcessHorizontalSpace = true;
		treeGridData.grabExcessVerticalSpace = true;
		newTree.setLayoutData(treeGridData);
		
		// Create new TreeItem
		TreeItem origTI = origTree.getItems()[0];
		TreeItem newItem = new TreeItem(newTree, SWT.NONE);
		
		// Copy over all data from the original
		newItem.setText(origTI.getText());
		newItem.setImage(origTI.getImage());
		newItem.setData("cpItem", origTI.getData("cpItem"));
		
		// Copy children from the original
		TreeItem[] origTIChildren = origTI.getItems();
		for (int j = 0; j<origTIChildren.length; j++) {
			copyGuiTreeItem(newItem, origTIChildren[j], movingTI, refTI, direction);
		}
		
		// Set the guiTree to the new tree
		NeedToKnow.guiTree = newTree;
		
		// Ensure the tree is initially expanded
		TreeItem rootTreeItem = newTree.getItems()[0];
		rootTreeItem.setExpanded(true);
		
		// TODO: Set selection to the original
        if (Wcker.DEBUG > 0) {
            System.out.println("origTree.getSelection():" + origTree.getSelection());
        }
		newTree.setSelection(origTree.getSelection());
		
		// Remove the old Tree
		origTree.dispose();
		
		// Add Listeners to new Tree
		ntk.addTreeListener(ntk.steps[stepNumber]);
		
		// Redraw the widgets
		newTree.getParent().layout();
	}
	
	/**
	 * Clone a GUI TreeItem, and move a given TreeItem above or below a reference TreeItem.
	 * @param origTI
	 * @param newTI
	 * @param RefTI
	 * @param direction
	 * @return
	 */
	private void copyGuiTreeItem(TreeItem newParent, TreeItem origTI, TreeItem movingTI, TreeItem refTI, String direction) {
        if (Wcker.DEBUG > 1) {
            System.out.println("copyGuiTreeItem(" + newParent + ", " + origTI + ", " + movingTI + ", " + refTI + ", " + direction + ")");
        }
		
		// Move TreeItem above
		// Check to see if this matches the reference TreeItem & up direction
		if (origTI == refTI && direction.equals("up")) {
			// Copy in the movingTI
			// Create new TreeItem
			TreeItem movedItem = new TreeItem(newParent, SWT.NONE);
			
			// Copy over all data from the original TreeItem
			movedItem.setText(movingTI.getText());
			movedItem.setImage(movingTI.getImage());
			
			// Copy children from the original TreeItem
			TreeItem[] origTIChildrenUp = movingTI.getItems();
			for (int j = 0; j<origTIChildrenUp.length; j++) {
				copyGuiTreeItem(movedItem, origTIChildrenUp[j], null, null, null);
			}
			
			// Move the item Element in the CP manifest
			Element cpMovingItem = (Element)movingTI.getData("cpItem");
			Element cpDestItem = (Element)refTI.getData("cpItem");
			Element cpDestParent = cpDestItem.getParentElement();
			
	        if (Wcker.DEBUG > 1) {
	            System.out.println("cpMovingItem: " + cpMovingItem.getName() + " " + cpMovingItem.getChildText("title", NeedToKnow.nsCP));
	            System.out.println("cpDestItem: " + cpDestItem.getName() + " " + cpDestItem.getChildText("title", NeedToKnow.nsCP));
	            System.out.println("cpDestParent: " + cpDestParent);
	            System.out.println("cpDestItem.getParent(): " + cpDestItem.getParent());
	            System.out.println("cpDestParent: " + cpDestParent.getName() + " " + cpDestParent.getChildText("title", NeedToKnow.nsCP));
	        }
	        
			// Find where we want to move the item to (above the reference)
			int upDestIndex = -1; // Need position where we want to insert the copy
			int upOrigIndex = -1; // Need position of the original so we can remove it later
			List cpDestSiblings = (List)cpDestParent.getChildren();
			for (int i=0; i<cpDestSiblings.size(); i++) {
				if (cpDestSiblings.get(i) == cpDestItem) {
					// The position to insert the copy is the position of the ref, 
					// as others will be pushed down the list
					upDestIndex = i;
				}
				if (cpDestSiblings.get(i) == cpMovingItem) {
					// The position of the original will be it's old position,
					//plus one for the copy we add above it.
					upOrigIndex = i+1;
				}
			}

	        if (Wcker.DEBUG > 1) {
	            System.out.println("upDestIndex: " + upDestIndex);
	            System.out.println("upOrigIndex: " + upOrigIndex);
	        }
	        
			// Move a clone to the destination
	        Element clone = (Element)cpMovingItem.clone();
			cpDestParent.addContent(upDestIndex, clone);
			
			// Attatch the clone to the TreeItem
			movedItem.setData("cpItem", clone);
			
	        if (Wcker.DEBUG > 1) {
	            List children = cpDestParent.getChildren();
	            System.out.println("cpDestParent: " + cpDestParent + ", #children: " + children.size());
	            for (int i = 0; i<children.size(); i++) {
	            	System.out.println("  children.get(i): " + ((Element)children.get(i)).getChildText("title", NeedToKnow.nsCP));
	            }
	        }
			
			// Delete the old version
			cpDestParent.removeContent(upOrigIndex);
			
			if (Wcker.DEBUG > 1) {
	            List children = cpDestParent.getChildren();
	            System.out.println("cpDestParent: " + cpDestParent + ", #children: " + children.size());
	            for (int i = 0; i<children.size(); i++) {
	            	System.out.println("  children.get(i): " + ((Element)children.get(i)).getChildText("title", NeedToKnow.nsCP));
	            }
	            System.out.println("clone: " + clone);
	            System.out.println("  clone.getParent(): " + clone.getParent());
	            System.out.println("  clone.getParentElement(): " + clone.getParentElement());
	        }
			
		}
		
		// Copy current TreeItem, if it isn't the one being moved 
		if (origTI != movingTI) {
			TreeItem newItem = new TreeItem(newParent, SWT.NONE);
			
			// Copy over all data from the original
			newItem.setText(origTI.getText());
			newItem.setImage(origTI.getImage());
			newItem.setData("cpItem", origTI.getData("cpItem"));
			
			// Copy children from the original
			TreeItem[] origTIChildren = origTI.getItems();
			for (int i = 0; i<origTIChildren.length; i++) {
				copyGuiTreeItem(newItem, origTIChildren[i], movingTI, refTI, direction);
			}
			newItem.setExpanded(origTI.getExpanded());
		}
		
		// Move TreeItem below
		// Check to see if this matches the reference TreeItem & down direction
		if (origTI == refTI && direction.equals("down")) {
	        if (Wcker.DEBUG > 1) {
	            System.out.println("match refTI, move down");
	        }
			// Copy in the movingTI
			// Create new TreeItem
			TreeItem movedItem = new TreeItem(newParent, SWT.NONE);
			
			// Copy over all data from the original TreeItem
			movedItem.setText(movingTI.getText());
			movedItem.setImage(movingTI.getImage());
			//movedItem.setData("cpItem", movingTI.getData("cpItem"));
			
			// Copy children from the original TreeItem
			TreeItem[] origTIChildrenDown = movingTI.getItems();
			for (int j = 0; j<origTIChildrenDown.length; j++) {
				copyGuiTreeItem(movedItem, origTIChildrenDown[j], null, null, null);
			}
			
			
			// Move the CP item Element in the CP manifest
			Element cpMovingItem = (Element)movingTI.getData("cpItem");
			Element cpDestItem = (Element)refTI.getData("cpItem");
			Element cpDestParent = cpDestItem.getParentElement();
			
			// Find where we want to move the item to
			int downDestIndex = -1;
			int downOrigIndex = -1;
			List cpDestSiblings = (List)cpDestParent.getChildren();
			for (int i=0; i<cpDestSiblings.size(); i++) {
				if (cpDestSiblings.get(i) == cpDestItem) {
					// The position to insert the copy is the position of the ref plus one.
					downDestIndex = i+1;
				}
				if (cpDestSiblings.get(i) == cpMovingItem) {
					// The position of the original will still be it's old position.
					downOrigIndex = i;
				}
			}

	        if (Wcker.DEBUG > 1) {
	            System.out.println("move up:");
	            System.out.println("cpDestSiblings.size(): " + cpDestSiblings.size());
	            System.out.println("cpDestSiblings: " + cpDestSiblings);
	        }
			// Move a clone to the destination
	        Content clone = (Content)cpMovingItem.clone();
			cpDestParent.addContent(downDestIndex, clone);
			
			// Attatch the clone to the TreeItem
			movedItem.setData("cpItem", clone);
			
			// Delete the old version
			cpDestParent.removeContent(downOrigIndex);
			
		}
	}

	int buttonNameToIndex(String name) {
        for(int i = 0; i < names.length; ++i) {
            if (name.equals(names[i])) {
                return i;
            }
        }
        return -1;
    }

    /**
     * Update the state of the buttons below the Tree widget, 
     * to reflect what operations are allowed on the current selection.
     * @param ti - The TreeItem currently selected.
     */
    public void updateTreeButtons(TreeItem ti) {
        if(Wcker.DEBUG > 0) {
            System.out.println("");
            System.out.println("updateTreeButtons()");
            System.out.println("Selected ti: " + ti.getText());
            Element cpItem = (Element)ti.getData("cpItem");
            System.out.println("ti.getData(\"cpItem\"): " + cpItem.getName() + ": " + cpItem.getChildText("title", NeedToKnow.nsCP));
        }
        
        // Disable/enable Adding items, depending on what structure is allowed
		boolean allowContainer = false;
		boolean allowLeaf = false;
		
        if (!ntk.getItemType(ti).equals("leaf")) {
        	String[][] allowedChildren = getItemAllowedChildren(ti);
        	
        	if(allowedChildren != null) {
        		// Children are allowed - check what types
        		
        		for (int i=0; i< allowedChildren.length; i++) {
        			String childType = allowedChildren[i][0];
                    if(Wcker.DEBUG > 1) {
                        System.out.println("childType: " + childType);
                    }
        			if (childType.equals("container")) {
        				allowContainer = true;
        			} else if (childType.equals("leaf")) {
        				allowLeaf = true;
        			}
        		}
        	}
        }

		treeEditButtons[buttonNameToIndex("addContainer")].setEnabled(allowContainer);
		treeEditButtons[buttonNameToIndex("addLeaf")].setEnabled(allowLeaf);
		

        if (ti.getParentItem() == null) {
            // If the selection is the root node disable deletion and movement
        	treeEditButtons[buttonNameToIndex("delContainer")].setEnabled(false);
            treeEditButtons[buttonNameToIndex("delLeaf")].setEnabled(false);
            treeEditButtons[buttonNameToIndex("up")].setEnabled(false);
            treeEditButtons[buttonNameToIndex("down")].setEnabled(false);
        } else {
            // Disable/enable remaining depending on whatever structure is allowed
        	String itemType = ntk.getItemType(ti);
            
            // Delete items
            if (itemType.equals("container")) {
                treeEditButtons[buttonNameToIndex("delContainer")].setEnabled(true);
                treeEditButtons[buttonNameToIndex("delLeaf")].setEnabled(false);
            } else if (itemType.equals("leaf")) {
                treeEditButtons[buttonNameToIndex("delContainer")].setEnabled(false);
                treeEditButtons[buttonNameToIndex("delLeaf")].setEnabled(true);
            }
            
            // Move items
            if (TreeNavigate.getPreviousSibling(ti) == null) {
                treeEditButtons[buttonNameToIndex("up")].setEnabled(false);
            } else {
                treeEditButtons[buttonNameToIndex("up")].setEnabled(true);
            }
            
            if (TreeNavigate.getFollowingSibling(ti) == null) {
                treeEditButtons[buttonNameToIndex("down")].setEnabled(false);
            } else {
                treeEditButtons[buttonNameToIndex("down")].setEnabled(true);
            }
        }
    }
    
    /**
     * Get the .wsi item type from a TreeItem's CP item 
     * @param ti
     * @return
     */
    private String[][] getItemAllowedChildren(TreeItem ti) {
    	// Get values from item's allowedChildren attribute
    	String[][] allowedChildrenSplit = null;
    	
    	Element cpItem = (Element)ti.getData("cpItem");
    	Attribute childrenAttr = null;
    	XPath typePath;
    	try {
    		typePath = XPath.newInstance("cp:metadata/wm:wcker/@allowedChildren");
    		typePath.addNamespace(NeedToKnow.nsCP);
    		typePath.addNamespace(NeedToKnow.nsWM);
    		childrenAttr = (Attribute)typePath.selectSingleNode(cpItem);
    	} catch (JDOMException err) {
    		err.printStackTrace();
    	}
    	
    	if (childrenAttr != null) {
    		// Split values into array of n{type, idref}
    		String[] allowedChildrenVals = childrenAttr.getValue().split(" ");
    		allowedChildrenSplit = new String[allowedChildrenVals.length][];
    		
    		for (int i=0; i<allowedChildrenVals.length; i++) {
    			allowedChildrenSplit[i] = new String[2];
    			String childTypeString[] = allowedChildrenVals[i].split(":");
    			allowedChildrenSplit[i][0] = childTypeString[0];
    			allowedChildrenSplit[i][1] = childTypeString[1];
    		}
    	}
    	
    	return allowedChildrenSplit;
	}
}