package uk.ac.ox.tall.wcker.wizard;
import org.eclipse.swt.widgets.*;

import uk.ac.ox.tall.wcker.Wcker;

class TreeNavigate {
	/**
	 * Return the previous TreeItem (depth-first), or null if there isn't one
	 * @param ti
	 * @return
	 */
	static TreeItem docOrderPrev(TreeItem ti) {
		TreeItem prevTreeItem = null;
		// Look for a previous sibling
		prevTreeItem = getPreviousSibling(ti);
		
		if (prevTreeItem != null) {
			// If there's a previous sibling, check for children
			if (prevTreeItem.getItemCount() > 0) {
				// If there are children, use the last one
				prevTreeItem = getLastChild(getPreviousSibling(ti));
			}
		} else {
			// If there's no previous sibling, get the parent
			prevTreeItem = ti.getParentItem();
		}
		
		return prevTreeItem;
	}

	/**
	 * Return the previous TreeItem (depth-first), or null if there isn't one
	 * @param ti
	 * @return
	 */
	static TreeItem getPreviousSibling(TreeItem ti) {
		TreeItem prevSib = null;
		TreeItem p = null;
		p = ti.getParentItem();
		
		if(p != null) {
			TreeItem[] siblings = p.getItems();
			
			int prevSiblingPosition = 0;
			for (int i = 0 ; i < siblings.length; i++) {
				// If the given TreeItem is this member of the siblings array, the next is member i-1
				if(ti == siblings[i]) {
					prevSiblingPosition = i - 1;
				}
			}

			if(prevSiblingPosition >= 0) {
				// If prevSiblingPosition points to a valid array member, use it.
				prevSib = siblings[prevSiblingPosition];
			}
		}
		
		return prevSib;
	}
	
	static TreeItem getLastChild(TreeItem ti) {
		TreeItem[] children = ti.getItems(); 
		return children[children.length - 1];
	}
	/**
	 * Return the next TreeItem (depth-first), or null if there isn't one
	 * @param ti
	 * @return
	 */
	static TreeItem docOrderNext(TreeItem ti) {
		TreeItem nextTreeItem = null;
		if (ti.getItemCount() > 0) {
        	// If TreeItem has children, open it and get the first child
			ti.setExpanded(true);
			nextTreeItem = ti.getItems()[0];
		} else {
        	// If TreeItem has no children, look for a following sibling
			nextTreeItem = getFollowingSibling(ti);
			// If there's no following sibling, look for a following sibling of the parent
			if (nextTreeItem == null) {
				nextTreeItem = getFollowingSibling(ti.getParentItem());
			}
		}
		
		return nextTreeItem;
	}

	/**
	 * Return the next sibling TreeItem (i.e. with the same parent), or null if there isn't one
	 * @param ti
	 * @return
	 */
	static TreeItem getFollowingSibling(TreeItem ti) {
		TreeItem nextSib = null;
		TreeItem p = ti.getParentItem();
		
		if(p != null) {
			TreeItem[] siblings = p.getItems();

			int nextSiblingPosition = 0;
			for(int i = 0 ; i < siblings.length; i++) {
				// If the given TreeItem is this member of the siblings array, the next is member i+1
				if(ti == siblings[i]) {
					nextSiblingPosition = i + 1;
				}
			}
			
			if(nextSiblingPosition > 0 && nextSiblingPosition < siblings.length) {
				// If nextSiblingPosition points to a valid array member, use it.
				nextSib = siblings[nextSiblingPosition];
			}
		}
		
		return nextSib;
	}
	
// XXX all the following *may* be obsolete
	
	static void collect_expanded_status_top(NeedToKnow local_ntn) {
		TreeItem[] ti = local_ntn.guiTree.getItems();
		for(int i=0;i<ti.length;++i) {
			collect_expanded_status(ti[i]);
		}
	}

	static void collect_expanded_status(TreeItem ti) {
		WckerTree wt = (WckerTree) (( (MyTreeItem ) ti).payload);	
		wt.expanded = ti.getExpanded();

		TreeItem[] ti2 = ti.getItems();
		for(int i=0;i<ti2.length;++i) {
			collect_expanded_status(ti2[i]);
		}
	}
	
	static void collect_selected_status_top(NeedToKnow local_ntn, MyTreeItem current_item) {
		TreeItem[] ti = local_ntn.guiTree.getItems();
		for(int i=0;i<ti.length;++i) {
			collect_selected_status(ti[i], current_item);
		}
	}

	static void collect_selected_status(TreeItem ti, MyTreeItem current_item) {
		WckerTree wt = (WckerTree) (( (MyTreeItem ) ti).payload);	
		wt.selected = ( ti == current_item );

		if(Wcker.DEBUG > 0) {
			System.out.println("248 wt.selected = " + wt.selected);
		}

		TreeItem[] ti2 = ti.getItems();
		for(int i=0;i<ti2.length;++i) {
			collect_selected_status(ti2[i], current_item );
		}
	}
}
