/*
 * Created on 23-Jun-2005
 *
 */
package uk.ac.ox.tall.wcker.wizard;

import java.io.IOException;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

import org.jdom.Element;

import uk.ac.ox.tall.wcker.Wcker;
import uk.ac.reload.editor.contentpackaging.xml.ContentPackage;
import uk.ac.reload.jdom.XMLPath;

/**
 * @author David Balch
 * @author David Johnston
 *
 * Main Wcker wizard page.
 */
public class WckerPage extends WizardPage {
    
    int stepNumber;
    static NeedToKnow ntk;
    WckerStep step;
    
    Text continueWizardTextInput;
    DirectoryDialog directoryDialog;
    Button exportButton;
    
	public WckerPage(NeedToKnow ntk_arg, int stepNumber_arg) {
		super("" + stepNumber_arg);
		ntk = ntk_arg;
		stepNumber = stepNumber_arg;
		step = ntk.steps[stepNumber];

	    if(Wcker.DEBUG > 0) {
	        System.out.println("WckerPage: " + stepNumber_arg);
	    }
	    
		setTitle(step.title);
		setDescription(step.strapLine);
	}

    /* 
     * Create the controls used in this Wizard page
     */
    public void createControl(Composite parent) {
    	// Layout with progress across a top row
    	Composite all = new Composite(parent, SWT.NONE);
    	GridLayout layout = new GridLayout();
    	all.setLayout(layout);
    	
    	// Progress meter
    	//Composite progress = getProgressLabels(all);
    	Composite progress = new ProgressLabels(all, ntk, stepNumber);
    	GridData progGData = new GridData();
    	progGData.heightHint = 20;
    	progress.setLayoutData(progGData);
    	
    	// Interactive widgets (where the action happens)
    	Composite main = mainContent(all);
    	GridData mainGData = new GridData(GridData.FILL_BOTH);
    	main.setLayoutData(mainGData);
    	
    	// Assign the controls to this page
    	setControl(all);
    }
    
	private Composite mainContent(Composite parent) {
		// Draw up the main widgets for this step in their own Composite
	    if(Wcker.DEBUG > 0) {
	        System.out.println("mainContent, step: " + step);
	    }

	    Composite mainComp = null;
		Composite contentComp = null;
		
		// Information pages are special case - just display the guidance.
		if(step.guidance != null && !step.stepType.equals("information")) {
			// If there is guidance, we want it in a resizable SashForm.
			// Guidance, so add SashForm.
			mainComp = new SashForm(parent, SWT.HORIZONTAL);
		} else {
			// No guidance (or intro with only guidance), so no SashForm.
			mainComp = new Composite(parent, SWT.NONE);
		}

		// Maximise use of the space in this Composite. 
		mainComp.setLayout(new FillLayout());
		
		// Add the content.
		if(step.stepType.equals("information")) {
			// Don't do anything for an information step - we add info via the HelpPanel below.
		} else if(step.stepType.equals("applyToItem")) {
			contentComp = applyItemComp(mainComp);
		} else if(step.stepType.equals("applyToHeirarchy")) {
			contentComp = applyHierComp(mainComp);
		} else if(step.stepType.equals("exportSkeleton")) {
			contentComp = exportSkelComp(mainComp);
			this.setPageComplete(false);
		}

		if(step.guidance != null) {
			// Add the Help panel.
			ntk.helpPanel = new HelpPanel(mainComp, ntk, stepNumber);

			// If we're not dealing with an information step, we need to set the sash weights
			if (!step.stepType.equals("information")) {
				// Set weights for the SashForm children
		        int[] weights = {70, 30};
		        ((SashForm)mainComp).setWeights(weights);				
			}
		}
		
		return mainComp;
	}

	/**
	 * Create the controls for a applyToHierarchy section
	 * @param parent - The parent Composite 
	 * @return applyHierComp - A Composite containing the widgets for editing a CP org tree 
	 */
	private Composite applyHierComp(Composite parent) {
		// Composite to hold Hierarchy relevant widgets
		SashForm applyHierComp = new SashForm(parent, SWT.NONE);
		
		// Panel (Composite) for Tree widget
		//Composite treePanel = 
		createTreeComp(applyHierComp);
		
		// Create Composite to hold the questions and prev/next buttons
		ntk.editControls = new EditControlsComp(applyHierComp, ntk, step);
 		
 		GridLayout gridLayout = new GridLayout();
 		gridLayout.numColumns = 1;
 		ntk.editControls.setLayout(gridLayout);

 		// Set relative weights between the tree and the editControls
        int[] weights = {40,60};
        applyHierComp.setWeights(weights);
        
		return applyHierComp;
	}

    /**
	 * Create a Composite containing questions for the item set by this page.
	 * 
	 * @param parent - The parent Composite 
	 * @return applyItemComp - A Composite containing the widgets for editing one CP item 
	 */
	private Composite applyItemComp(Composite parent) {
	    if(Wcker.DEBUG > 0) {
	        System.out.println("applyItemComp()");
	        System.out.println("step.applyItemChunks.length: " + step.applyItemChunks.length);
	    }
		Composite applyItemComp = new Composite(parent, SWT.BORDER);
		FillLayout applyItemCompLayout = new FillLayout(SWT.VERTICAL);
		applyItemComp.setLayout(applyItemCompLayout);
		
		/* For each applytoItem in the step, process the prompt, and questionGroups
		 * applying them to their targetHook.
		 */
		for (int i=0; i<step.applyItemChunks.length; i++) {
		    if(Wcker.DEBUG > 1) {
		        System.out.println("applyItemComp[" + i + "]: " + step.applyItemChunks[i]);
		        System.out.println("applyItemComp[" + i + "].getTargetHook(): " + step.applyItemChunks[i].getTargetHook());
		    }
			// Point this to the targetHook value
		    String targetHook = step.applyItemChunks[i].getTargetHook();
			Element cpItem = ntk.getHierItemByQuestionHook(targetHook);
            step.applyItemChunks[i].getAsComposite(applyItemComp, cpItem);
		}

		return applyItemComp;
	}	
    
    TreeItem get_tree_selection() {
        try {
            TreeItem[] ti = NeedToKnow.guiTree.getSelection();
            return ti[0];
        } catch (Exception e) {
            return (TreeItem) null;
        }
    }

    /**
     * @param parent The parent Composite this will attatch to.
     * @return Composite
     */
    private Composite exportSkelComp(Composite parent) {
        Composite exportSkelComp = new Composite(parent, SWT.BORDER);
        GridLayout gridLayout = new GridLayout();
        gridLayout.numColumns = 2;
        exportSkelComp.setLayout(gridLayout);
        
        // Label
        Label exportLabel = new Label(exportSkelComp, SWT.NONE);
        exportLabel.setText("Select export location");
		GridData labelGridData = new GridData();
		labelGridData.horizontalSpan = 2;
		exportLabel.setLayoutData(labelGridData);
        
        // Directory text input
		continueWizardTextInput = new Text(exportSkelComp, SWT.BORDER);
		GridData inputGridData = new GridData();
		inputGridData.horizontalAlignment = GridData.FILL;
		inputGridData.grabExcessHorizontalSpace = true;
        continueWizardTextInput.setLayoutData(inputGridData);
		
		// File browsing button
        Button continueWizardBrowse = new Button(exportSkelComp, SWT.PUSH);
		continueWizardBrowse.setText("Browse");
		
		// Directory dialogue widget
    	directoryDialog = new DirectoryDialog(exportSkelComp.getShell());
		directoryDialog.setText("Select export location");
		directoryDialog.setMessage("Select a directory to write the course files in.");
		
		// Export button
        exportButton = new Button(exportSkelComp, SWT.PUSH);
        exportButton.setText("Export");
        exportButton.setEnabled(false);
        
		
		// Listen for clicks on the browse button - open the directory dialogue widget onclick
		continueWizardBrowse.addSelectionListener(
				new SelectionAdapter() {
					public void widgetSelected(SelectionEvent e) {
						if(Wcker.DEBUG > 0) {
							System.out.println("Clicked 'Browse' button");
						}
						String selectedDirectory = "";
						selectedDirectory = directoryDialog.open();
						if (selectedDirectory != null) {
							System.out.println("Selected directory: '" + selectedDirectory + "'.");
							continueWizardTextInput.setText(selectedDirectory);
							ntk.setExportDir(selectedDirectory);
							
					        exportButton.setEnabled(true);
					        setPageComplete(true);
						} else {
							System.out.println("No directory selected");
						}
					}
				}
		);
        
		// Export button listener
		exportButton.addSelectionListener ( 
				new SelectionListener() {
					public void widgetSelected(SelectionEvent e) {
						Composite parent = ((Button)e.getSource()).getParent();
						
						// Export to the selected directory.
						String exportDir = continueWizardTextInput.getText();
				        //Save our progress
				        try {
				        	NeedToKnow.cp.saveDocument();
				        } catch (IOException io) {
				        	io.printStackTrace();
				        }
						exportCpAsFiles(exportDir);
						
						// TODO: Ensure that this fails if there's an export problem - 
						// we're currently *assuming* it's ok.
						boolean exportSuccessful = true;
						if (exportSuccessful) {
							new Label(parent, SWT.NONE); // Get past the columns
							Label exportCompleted = new Label(parent, SWT.NONE);
							exportCompleted.setText("Export completed.");
							parent.layout();
						}
					}
					public void widgetDefaultSelected(SelectionEvent e) {
						if(Wcker.DEBUG > 0) {
							System.out.println("Default Selection Event");
						}
					}
				}
		);
		
        return exportSkelComp;
    }
	
    protected void exportCpAsFiles(String exportDir) {
    	// Export
		new SkeletonExporter(ntk, exportDir);
	}

    
    /**
     * Create and layout tree and tree editing buttons with grid layoutdata
     * @param parent
     * @param ntk
     * @return a Composite with tree and buttons
     */
	Composite createTreeComp(Composite parent) {
		Composite treePanel = new Composite(parent, SWT.BORDER);
		GridLayout treeGridLayout = new GridLayout();
		treeGridLayout.marginHeight = 0;
		treeGridLayout.marginWidth = 0;
		treeGridLayout.verticalSpacing = 0;
		treePanel.setLayout(treeGridLayout);
		
		
		// Add a Composite to hold the Tree
	    Composite treeComp = new Composite(treePanel, SWT.NONE);
		GridData treeCompGridData = new GridData();
		treeCompGridData.horizontalAlignment = GridData.FILL;
		treeCompGridData.verticalAlignment = GridData.FILL;
		treeCompGridData.grabExcessHorizontalSpace = true;
		treeCompGridData.grabExcessVerticalSpace = true;
		treeComp.setLayoutData(treeCompGridData);
	    
		GridLayout treeCompLayout = new GridLayout();
		treeCompLayout.marginHeight = 0;
		treeCompLayout.marginWidth = 0;
		treeCompLayout.verticalSpacing = 0;
		treeComp.setLayout(treeCompLayout);
		
		// Add the hierarchy Tree widget
		Tree myTree = guiTreeFromCp(treeComp, NeedToKnow.cp);
		GridData treeGridData = new GridData();
		treeGridData.horizontalAlignment = GridData.FILL;
		treeGridData.verticalAlignment = GridData.FILL;
		treeGridData.grabExcessHorizontalSpace = true;
		treeGridData.grabExcessVerticalSpace = true;
		myTree.setLayoutData(treeGridData);
		
		// Hook the Tree into NeedToKnow
		NeedToKnow.guiTree = myTree;
		
		// Add the tree editing buttons
		final TreeButtons treeButtons = new TreeButtons(stepNumber, ntk, treePanel);
		GridData buttonsGridData = new GridData();
		buttonsGridData.horizontalAlignment = GridData.FILL;
		buttonsGridData.grabExcessHorizontalSpace = true;
		treeButtons.setLayoutData(buttonsGridData);
		ntk.tbs = treeButtons;
		
		// Ensure buttons match default selection
		treeButtons.updateTreeButtons(myTree.getSelection()[0]);

		// Listen for selections on the Tree
		ntk.addTreeListener(step);
		
		return NeedToKnow.guiTree;
	}

	/**
	 * Build the GUI tree which represents the IMSCP hierarchy
	 * @param treePanel The Composite which we add the Tree to.
	 * @param cp The Content Package to use.
	 * @return theGuiTree
	 */
	private Tree guiTreeFromCp(Composite parent, ContentPackage cp) {
	    if(Wcker.DEBUG > 1) {
	        System.out.println("guiTreeFromCp(" + cp + ")");
	    }
	    
		Tree theGuiTree = new Tree(parent, SWT.NONE);
		
		// Get CP organisation
		Element cpOrg = cp.getElement(new XMLPath("manifest/organizations/organization"));
		
		// Add organisation as a TreeItem
		TreeItem rootTreeItem = new TreeItem(theGuiTree, SWT.NONE);
		rootTreeItem.setData("cpItem", cpOrg);
		rootTreeItem.setText(cpOrg.getChildText("title", NeedToKnow.nsCP));
		// Set image
        Image image = new Image(Display.getDefault(), (Wcker.wckerProps.getProperty("wcker.gfx.dir") + "root.png"));
        rootTreeItem.setImage(image);

		// Add child items as treeItems
		java.util.List childItems = cpOrg.getChildren("item", NeedToKnow.nsCP);
		for (int i = 0; i<childItems.size(); i++) {
			addTreeItemFromCpItem(rootTreeItem, (Element)childItems.get(i));
		}
		
		// Ensure the tree is initially expanded
		rootTreeItem.setExpanded(true);
		// Also expand the first child, to give the user an idea what's going on
		rootTreeItem.getItems()[0].setExpanded(true);
		
		// Select the root element in the tree
		theGuiTree.setSelection(new TreeItem[] {rootTreeItem});
		
		// Return the GUI Tree
		return theGuiTree;
	}

	/**
	 * Add a TreeItem for a supplied item Element, then recurse to any child Element items
	 * 
	 * @param parentTreeItem The parent Tree which we attatch to.
	 * @param itemElement The Element that we add as a new TreeItem
	 */
	public static void addTreeItemFromCpItem(TreeItem parentTreeItem, Element itemElement) {
		// Add this item
		TreeItem newTreeItem = new TreeItem(parentTreeItem, SWT.NONE);
		
		// Attatche the cp Item
		newTreeItem.setData("cpItem", itemElement);
	    
		// Set the title
		newTreeItem.setText(itemElement.getChildText("title", NeedToKnow.nsCP));
		
		// Add child items as treeItems
		java.util.List childItems = itemElement.getChildren("item", NeedToKnow.nsCP);
		for (int i = 0; i<childItems.size(); i++) {
			addTreeItemFromCpItem(newTreeItem, (Element)childItems.get(i));
		}
		
		// Set image
		String itemType = ntk.getItemType(itemElement);
		Image image = null;
		if (itemType.equals("container")) {
			image = new Image(Display.getDefault(), (Wcker.wckerProps.getProperty("wcker.gfx.dir") + "Container.png"));
		} else {
			image = new Image(Display.getDefault(), (Wcker.wckerProps.getProperty("wcker.gfx.dir") + "Leaf.png"));
		}
        newTreeItem.setImage(image);
	}
    
	/**
	 * Expand the whole GUI tree
	 * @param tree
	 */
	private static void expandTree(Tree tree) {
		TreeItem[] treeItems = tree.getItems();
		for(int i = 0; i < treeItems.length; ++i) {
		    treeItems[i].setExpanded(true);
		}
	}
}
