package uk.ac.ox.tall.wcker.wizard;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.TreeItem;

import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;

import uk.ac.ox.tall.wcker.Wcker;
/**
 * WckerQuestion class - represents the <question> element in the XML
 *                       forms part of the NeedToKnow object
 * @author BalchD
 * @author JohnstonD
 *  
 */

public class WckerQuestion {
    
    public String promptText; 
    public Guidance guidance;
    public WckerInput[] inputs;
    
	/** GUI INPUT STUFF **/
	private Label prompt;
	
	/**
	 * Constructor.
	 * @return
	 */
	public WckerQuestion(Element qEl) {
		if(Wcker.DEBUG > 0) {
			System.out.println("");
			System.out.println("# Build WckerQuestion #");
			System.out.println("qNode: " + qEl);
			System.out.println("qNode.getNodeName(): " + qEl.getName());
		}
		
		// Set the question prompt
		promptText = qEl.getChildText("prompt", NeedToKnow.nsWSI);
		
		// Create a guidance object - a HTML Document fragment
		guidance = new Guidance(qEl);
		
		// Create a list of the inputs in this question
		XPath childPath;
		List inputNodes = null;
		try {
			childPath = XPath.newInstance("wsi:inputs/wsi:input");
			childPath.addNamespace(NeedToKnow.nsWSI);
			inputNodes = childPath.selectNodes(qEl);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		if(Wcker.DEBUG > 0) {
			System.out.println("inputNodes.size(): " + inputNodes.size());
			System.out.println("inputNodes: " + inputNodes);
		}
		
		inputs = new WckerInput[inputNodes.size()];
		
		// Add the inputs
		for(int j=0; j < inputNodes.size(); j++) {
			inputs[j] = new WckerInput((Element)inputNodes.get(j));
			if(Wcker.DEBUG > 1) {
				System.out.println(" inputs["+j+"]: " + inputs[j]);
				System.out.println(" inputs["+j+"].target: " + inputs[j].getTarget() + ".");
			}
		}
		
		if(Wcker.DEBUG > 0) {
			System.out.println("## New WckerQuestion ##");
			System.out.println(toString());
		}
	}

	public WckerQuestion() {
	    // Empty constructor for dup()
	}

	/**
	 * Return a Node containing all guidance in this question.
	 */
	public Element getGuidance() {
	    return guidance.getInfo();
	}

	/**
	 * produces textual dump of WckerQuestion object for debugging purposes
	 * @return textual dump as a String
	 */
	public String toString() {
		String acc = "\nWckerQuestion.toString()\n";
		acc +=  "PROMPT: " + promptText + "\n";
		acc +=  "GUIDANCE: " + guidance + "\n";
		for(int i = 0; i < inputs.length; ++i) {
		    acc += inputs[i];
		}
		return acc;
	}
	
	/**
	 * Add prompt, input pairs to a specified Composite.
	 * @param parent The Composite to attach the questions to.
	 * @param cpItem 
	 */
	public void getGuiQuestions(Composite parent, Element cpItem) {
        if(Wcker.DEBUG > 1) {
            System.out.println("WckerQuestion.getGuiQuestions()");
        }
        
        // Create a new composite to add the quesion on 
        Composite qComp = new Composite(parent, SWT.NONE);
        GridData qCompLayoutData = new GridData();
        qCompLayoutData.horizontalAlignment = GridData.FILL;
        qCompLayoutData.grabExcessHorizontalSpace = true;
        qComp.setLayoutData(qCompLayoutData);
        
        GridLayout theLayout = new GridLayout();
        theLayout.numColumns = 1;
        theLayout.marginWidth = 0;
        theLayout.marginHeight = 0;
        theLayout.verticalSpacing = 2;
        qComp.setLayout(theLayout);
        
		// Add the question label
        prompt = new Label(qComp, SWT.NONE);
        prompt.setText(promptText);
        GridData promptLayoutData = new GridData();
        promptLayoutData.horizontalAlignment = GridData.FILL;
        promptLayoutData.grabExcessHorizontalSpace = true;
        prompt.setLayoutData(promptLayoutData);
        
        // All question inputs
		for (int i = 0; i < inputs.length; i++) {
			WckerInput thisWI = inputs[i];
			String thisTarget = thisWI.getTarget();
			String data = "";

	        if(Wcker.DEBUG > 1) {
	            System.out.println("thisWI: " + thisWI);
	            System.out.println("thisTarget: " + thisTarget);
	        }
			
			// Find the target
			XPath targetPath;
			Element targetEl = null;
			try {
				targetPath = XPath.newInstance(thisTarget);
				targetPath.addNamespace(NeedToKnow.nsCP);
				targetPath.addNamespace(NeedToKnow.nsMD);
				targetPath.addNamespace(NeedToKnow.nsWM);
				targetEl = (Element)targetPath.selectSingleNode(cpItem);
			} catch (JDOMException e) {
				e.printStackTrace();
			}

            if (targetEl != null) {
            	// If the target exists, get any existing value
            	data = targetEl.getText();
            }
			
			if (!thisWI.getType().equals("hidden")) {
	            if(Wcker.DEBUG > 1) {
	                System.out.println("targetEl: " + targetEl);
	            }
	            
		        // Create a text input and set any existing value
	            boolean multiLine = thisWI.getConditions().contains("multiline");
				Text thisInput = null;
				if (multiLine) {
					thisInput = new Text(qComp, SWT.MULTI|SWT.WRAP|SWT.V_SCROLL|SWT.BORDER);
				} else {
					thisInput = new Text(qComp, SWT.SINGLE|SWT.BORDER);
				}
		        thisInput.setText(data);
		        
		        // Connect the input to the manifest values
		        thisInput.setData("cpItem", cpItem);
		        thisInput.setData("targetEl", targetEl);
		        
		        // Set layout
		        GridData inputLayoutData = new GridData();
		        inputLayoutData.horizontalAlignment = GridData.FILL;
		        inputLayoutData.grabExcessHorizontalSpace = true;
		        inputLayoutData.horizontalIndent = 5;
				if (multiLine) {
			        inputLayoutData.grabExcessVerticalSpace = true;
			        inputLayoutData.heightHint = 80;
				}
		        thisInput.setLayoutData(inputLayoutData);

		        // Add listener
		        thisInput.addModifyListener(
		        		new ModifyListener() {
		        			public void modifyText(ModifyEvent e) {
		        				Text input = (Text)e.getSource();
		        				Element targetEl = (Element)input.getData("targetEl");
		        				
		        				// What was the title before the edit
		        				String origTitle = targetEl.getText();

    							// What is the title after the edit
	        					String newTitle = input.getText();
		        				
		        				// Update the target
		        				targetEl.setText(newTitle);
		        				
	        					// If we're dealing with a title
		        				if (targetEl.getName().equals("title")) {
		        					TreeItem[] theTreeItem = NeedToKnow.guiTree.getSelection();
		        					
		        					// Update GUI tree
		        					if (theTreeItem != null) {
		        						//We've got a TreeItem selected so use that.
		        						theTreeItem[0].setText(newTitle);
		        					}

		        					// Update the resource filename on the targetEl
	    							String origTitlePath = WckerUtility.formatTitleToPath(origTitle);
	    							String newTitlePath = WckerUtility.formatTitleToPath(newTitle);
		        					
			        				Element targetItem = (Element)input.getData("cpItem");
		        					Element resource = NeedToKnow.cp.getReferencedElement(targetItem);
		        					
		        					if (resource != null) {
			        					// Get existing value
		        						String oldPath = resource.getAttributeValue("href");
		        						// Construct new path
		        						String newPath = oldPath.replaceAll(origTitlePath, newTitlePath);
		        						
		        						// Add new path to relevant points:
		        						resource.setAttribute("href", newPath);
		        				    	// Find any child items and itemPatterns XPath childPath;
		        						Element fileEl = null;
		        			    		XPath childPath;
		        			    		try {
		        			    			childPath = XPath.newInstance("cp:file[@href = '" + oldPath + "']");
		        			    			childPath.addNamespace(NeedToKnow.nsCP);
		        			    			fileEl = (Element)childPath.selectSingleNode(resource);
		        			    		} catch (JDOMException e1) {
		        			    			e1.printStackTrace();
		        			    		}
		        			    		fileEl.setAttribute("href", newPath);
		        			    		
		        					} else {
		        						// No resource, so it's a container
		        						// Update the foldername for all the child items. Woot!
		        						updateChildPaths(targetItem, origTitlePath, newTitlePath);
		        					}
		        					
		        				}
		        			}
		        		}
		        );
			}
		}
        
	}

	/**
	 * Update the resource paths for all child items of the supplied targetItem, 
	 * replacing the substring origTitlePath with newTitlePath.
	 * @param targetItem
	 * @param origTitlePath
	 * @param newTitlePath
	 */
	protected void updateChildPaths(Element targetItem, String origTitlePath, String newTitlePath) {
		if(Wcker.DEBUG > 1) {
			System.out.println("DB:");
			System.out.println("DB: updateChildPaths()");
			System.out.println("DB: targetItem: " + targetItem);
			System.out.println("DB: origTitlePath: " + origTitlePath);
			System.out.println("DB: newTitlePath: " + newTitlePath);
		}
		List childItems = targetItem.getChildren();
		for (int i=0; i<childItems.size(); i++) {
			Element childItem = (Element)childItems.get(i);
			Element resource = NeedToKnow.cp.getReferencedElement(childItem);
			String oldPath;
			String newPath;
			
			if (resource != null) {
				// Change the resources' href value
				oldPath = resource.getAttributeValue("href");
				newPath = oldPath.replaceAll(origTitlePath, newTitlePath);
				resource.setAttribute("href", newPath);
				
				// Change all the file href values for this resource
				List childFiles = resource.getChildren();
				if(Wcker.DEBUG > 1) {
					System.out.println("DB: childFiles.size(): " + childFiles.size());
					System.out.println("DB: childFiles.get(0): " + childFiles.get(0));
				}
				for (int j=0; j<childFiles.size(); j++) {
					Element fileEl = (Element)childFiles.get(j);
					oldPath = fileEl.getAttributeValue("href");
					newPath = oldPath.replaceAll(origTitlePath, newTitlePath);
					fileEl.setAttribute("href", newPath);
				}
			} else {
				// No resource, so it's a container
				updateChildPaths(childItem, origTitlePath, newTitlePath);
			}
		}
		
	}
}
