package uk.ac.ox.tall.wcker.wizard;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;

import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;

import uk.ac.ox.tall.wcker.Wcker;

/**
 * WckerQuestionGroup class - represents the <questionGroup> element in the XML
 *                            forms part of the NeedToKnow object
 * @author BalchD
 * @author JohnstonD
 *  
 */

public class WckerQuestionGroup {
	String id = "";
	public WckerQuestion[] questions;
	
	/**
	 * Constructor for a questionGroup (a Java Node representing a .wsi XML element) 
	 */
	public WckerQuestionGroup(Element qGroupNode) {
		if(Wcker.DEBUG > 1) {
			System.out.println(" ### New WckerQuestionGroup ###");
		}
	    // Set id
		id = qGroupNode.getAttributeValue("id");
		
		// Populate "questions" array with .wsi questions
		// Get list of .wsi questions
		XPath qgPath;
		List qNodes = null;
		try {
			//qgPath = XPath.newInstance("wsi:wizard/wsi:questions/wsi:questionGroup");
			qgPath = XPath.newInstance("wsi:question");
			qgPath.addNamespace(NeedToKnow.nsWSI);
			qNodes = qgPath.selectNodes(qGroupNode);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		
		// Set "questions" array size to the number of .wsi questions
		questions = new WckerQuestion[qNodes.size()];
		
		// Add the .wsi questions to the "questions" array
		for(int i = 0; i < qNodes.size(); i++) {
			questions[i] = new WckerQuestion((Element)qNodes.get(i));
		}

		if(Wcker.DEBUG > 1) {
			System.out.println(" ### What's in this qGroup? ###");
			System.out.println(toString());
			System.out.println(" ### That's all ###");
			System.out.println("");
		}
	}
	
	public String toString() {
		String acc = "---- WckerQuestionGroup ----\n";
		acc += "ID: " + id;

		if(questions != null) {
		    for (int i = 0;i < questions.length;++i) {
		        acc += questions[i];
		    }
		}
		return acc;
	}

	public WckerQuestionGroup() {
	    // Empty constructor
	}

	/*
	 * Return all guidance for this question group and child questions.
	 */
	public Element getGuidance() {
		Element thisGuidance = null;

	    if(Wcker.DEBUG > 1) {
	        System.out.println("questions: " + questions);
	        System.out.println("questions.length: " + questions.length);
	    }
	    // Iterate over all questions in this questiongroup 
	    for(int i=0; i<questions.length; i++) {
		    if(Wcker.DEBUG > 1) {
		        System.out.println("questions[i].prompt: " + questions[i].promptText);
		    }
	        // Add the guidance in the child questions
		    thisGuidance = questions[i].getGuidance();
			if(Wcker.DEBUG > 1) {
				System.out.println("thisGuidance: " + thisGuidance);
			}
	    }
	    
	    return thisGuidance;
	}
	
	/**
	 * Return how many questions are in this group
	 */
	public int getLength() {
		return questions.length;
	}
	
	/**
	 * Return how many questions are in this group
	 */
	public WckerQuestion getQuestion(int i) {
		return questions[i];
	}
	
	public Composite getAsComposite(Composite parent, Element cpItem) {
        if(Wcker.DEBUG > 1) {
            System.out.println("WckerQuestionGroup.getAsComposite()");
            System.out.println("questions.length: " + questions.length);
        }
		// Create a composite
		Composite qGroupComp = new Composite(parent, SWT.NONE);
		
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		qGroupComp.setLayoutData(gridData);

		GridLayout theLayout = new GridLayout();
		theLayout.numColumns = 1;
        theLayout.marginWidth = 0;
        theLayout.marginHeight = 0;
		qGroupComp.setLayout(theLayout);
        
		// Add each question from question group to the composite
		int count = questions.length;
		for(int i =0; i <count; ++i) {
			questions[i].getGuiQuestions(qGroupComp, cpItem);
		}
		
		return qGroupComp;
	}
}
