package uk.ac.ox.tall.wcker.wizard;

import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Attribute;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;

import uk.ac.ox.tall.wcker.Wcker;
import uk.ac.ox.tall.wcker.util.LoadXML;

/**
 * WckerStep class - represents the <step> element in the XML
 *                   forms part of the NeedToKnow object
 * @author BalchD
 * @author JohnstonD
 *  
 */

public class WckerStep {
    //Parent NeedToKnow object.
    // Could we look this up from the step itself, without this explicit reference?
    NeedToKnow parentNtk;
    
	// Step description
	public String title;
	public String strapLine;
	public String stepType;  
	
	// Step guidance/help text
	public Document guidance = null;
	public String plaintext; // top-level plaintext
	String structure_guidance;
	String structure_prompt;
	
	// Step questions
	public ApplyToItem[] applyItemChunks;
	public ApplyToHier applyHierChunk;
	public String questionGroup_ref;
	public WckerQuestionGroup wqg; // a unique copy of Question Group to modify at will!!!!
	
	/**
	 * Create a WckerStep from a XML node relating to a .wsi step element.
	 * @param stepEl
	 */
	public WckerStep(Element stepEl, NeedToKnow ntkArg) {
	    parentNtk = ntkArg;
	    
		// Set title, strapLine, and type for this step
		title = stepEl.getAttributeValue("title");
		strapLine = stepEl.getAttributeValue("strapLine");
		stepType = stepEl.getAttributeValue("type");
		
		// Set help info
		setGuidance(stepEl);
		
		if(stepType.equals("information")) {
			infoStep(stepEl); 
		} else if(stepType.equals("applyToItem")) {
			applyItemStep(stepEl);
		} else if(stepType.equals("applyToHeirarchy")) {
			applyHierStep(stepEl);
		} else if(stepType.equals("exportSkeleton")) {
			exportSkelStep(stepEl);
		}
		
		plaintext = extract_plaintext(stepEl);
		
		if(Wcker.DEBUG > 1) {
			List nodes = stepEl.getChildren();
			System.out.println("# child nodes:" + nodes.size());
			for (int c=0; c<nodes.size(); c++) {
				System.out.println("# child node:" + c + " = " + nodes.get(c));
			}
			System.out.println("YYY READING IN |" + questionGroup_ref + "|");
			System.out.println("PLAINTEXT:" + extract_plaintext(stepEl));
		}
	}

	/**
	 * Method to set WckerStep info for "information"-type steps.
	 * 
	 * Doesn't actually do anything, as the information is handled by the help pages.
	 */
	private void infoStep(Element stepEl) {
		// All the work is done in the guidance elements, so there's nothing needed here.
        
	}

	/**
	 * Method to set WckerStep info for "applyHierStep"-type steps
	 */
	private void applyHierStep(Element stepEl) {
        // Get all applyToItem nodes in this setp
        Element hierEl = stepEl.getChild("applyToHeirarchy", NeedToKnow.nsWSI);
        
        applyHierChunk = new ApplyToHier(hierEl, parentNtk);
        
        /*OLD:
		// Get the questionGroup nodes
		String[] path = {"applyToHeirarchy", "applyToItem", "questionGroup"};
		Node[] ns = Navigate.elements_by_path(stepNode, path);
		if(Wcker.DEBUG > 0) {
			System.out.println("ns.length: " + ns.length);
			for (int c=0; c<ns.length; c++) {
				System.out.println("ns[" + c + "] = " + ns[c]);
			}
		}
		questionGroup_ref = ns[0].getAttributes().getNamedItem("ref").getNodeValue();

		try {
			String[] path2 = {"applyToHeirarchy", "guidance", "plainText"};
			Node[] ns2 = Navigate.elements_by_path(stepNode, path2);
			structure_guidance = ns2[0].getChildNodes().item(0).getNodeValue();
		} catch(Exception e) {
			structure_guidance = "";
		}
		
		try {
			String[] path3 = {"applyToHeirarchy", "prompt"};
			Node[] ns3 = Navigate.elements_by_path(stepNode, path3);
			structure_prompt = ns3[0].getChildNodes().item(0).getNodeValue();
		} catch(Exception e) {
			structure_prompt = "";
		}
        */
	}

	/**
	 * Method to set WckerStep info for "applyItemStep"-type steps
	 */
	private void applyItemStep(Element stepEl) {
		// Create controls for editing one item
		
		// Get all applyToItem nodes in this setp
		List applyNodes = stepEl.getChildren("applyToItem", NeedToKnow.nsWSI);
		if(Wcker.DEBUG > 0) {
			System.out.println("--- applyItemStep ---");
			System.out.println("applyNodes.getLength(): " + applyNodes.size());
		}
		
		// Set up the applyToItem objects
		applyItemChunks = new ApplyToItem[applyNodes.size()];
		for (int i=0; i<applyNodes.size(); i++) {
			if(Wcker.DEBUG > 0) {
				System.out.println("applyNodes.item(" + i + "): " + applyNodes.get(i));
			}
			applyItemChunks[i] = new ApplyToItem((Element)applyNodes.get(i), parentNtk);
		}
	}

	/**
	 * Method to set WckerStep info for "exportSkelStep"-type steps
	 */
	private void exportSkelStep(Element stepEl) {
		if(Wcker.DEBUG > 0) {
			System.out.println("Do exportSkelStep()");
		}
		
	}

	/*
	 * Set the guidance for this step
	 */
	void setGuidance(Element stepEl) {
	    // Create a HTML Document, then add guidance nodes from the 
	    // step page, and from questionGroups referenced.

		//Load HTML help template from file
		String helpTemplatePath = Wcker.wckerProps.getProperty("wcker.res.dir") + "help.html";
		Document helpDoc = LoadXML.loadXml(helpTemplatePath);

		XPath bodyPath;
    	Element helpBody = null;
		try {
			bodyPath = XPath.newInstance("/html/body");
			helpBody = (Element)bodyPath.selectSingleNode(helpDoc);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		
		// Get guidance from steps and referred questions		
		// Create a list of all the child guidance and questionGroup elements, document order (depth first)
		XPath childPath;
		List questAndGuideNodes = null;
		try {
			//childPath = XPath.newInstance("descendant-or-self::wsi:guidance | descendant-or-self::wsi:questionGroup");
			childPath = XPath.newInstance("descendant-or-self::wsi:guidance");
			childPath.addNamespace(NeedToKnow.nsWSI);
			questAndGuideNodes = childPath.selectNodes(stepEl);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		
		// Add each guidance (inc guidance from questionGroups) to the helpBody
		for(int i=0; i<questAndGuideNodes.size(); i++) {
		    Element thisElement = (Element)questAndGuideNodes.get(i);
		    String thisElementName = thisElement.getName();
		    Element thisGuidance = null;
	        
			if(Wcker.DEBUG > 0) {
				System.out.println("thisElement: " + thisElement);
				System.out.println("thisElementName: " + thisElementName);
				
				List attribs = thisElement.getAttributes();
				System.out.println("attribs.size(): " + attribs.size());
				
				for(int j=0; j<attribs.size(); j++) {
					System.out.println("attribs.item("+j+").getName(): " + ((Attribute)attribs.get(j)).getName());
					System.out.println("attribs.item("+j+").toString(): " + attribs.get(j).toString());
					System.out.println("attribs.item("+j+").getNodeValue(): " + ((Attribute)attribs.get(j)).getValue());
				}
			}
		    
		    if(thisElementName.equals("guidance")) {
		        // If it's a guidance element, get its info.
		    	Guidance stepGuidance = new Guidance(thisElement);
		        thisGuidance = stepGuidance.getInfo();
		    }
		    else if (thisElementName.equals("questionGroup")) {
		        // If it's a questionGroup element, get it's referred guidance part...
		        String qRef = thisElement.getAttribute("ref").getValue();
		        
				if(Wcker.DEBUG > 0) {
					System.out.println("qRef: " + qRef);
				}
		        WckerQuestionGroup qGroup = NeedToKnow.getQuestionGroup(qRef);

				if(Wcker.DEBUG > 0) {
					System.out.println("qGroup.id: " + qGroup.id);
				}
				
		        //...and get the guidance.
		        thisGuidance = qGroup.getGuidance();
		    }
		    
		    // Set the guidance
		    if(thisGuidance != null) {
		    	// Adopt the guidance from its own Document to this Step Document
		        helpBody.addContent((Element)thisGuidance.clone());
		    }
		}
		
		// Assign the helpDoc to the guidance for this step.
		guidance = helpDoc;
	}
		
	/**
	* extract contents of children plainText nodes
	* these will be concatinated if there is more than one match
	*/
	static String extract_plaintext(Element el) {
		XPath childPath;
		List textEls = null;
		try {
			childPath = XPath.newInstance("descendant::wsi:plainText");
			childPath.addNamespace(NeedToKnow.nsWSI);
			textEls = childPath.selectNodes(el);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		
		String pt = "";
		for(int j=0; j < textEls.size(); ++j) {
			 pt += ((Element)textEls.get(j)).getText();
		}
		return pt;
	}
	
	/**
	 * produces textual dump of WckerStep object for debugging purposes
	 *
	 * @return textual dump as a String
	 */
	public String toString() {
		String acc = "";
		acc += "TITLE: " + title + "\n";
		acc += "GUIDANCE: " +  guidance + "\n";
		acc += "QUESTIONGROUP_REF: " + questionGroup_ref + "\n";
		acc += "wqg: " + wqg + "\n";
		return acc;
	}
}
