package uk.ac.ox.tall.wcker.wizard;
/**
 * WckerTree class - a simple class for storing Tree-like data structures
 *              as described in the wizrad xml file
 *              forms part of the NeedToKnow object
 *                   
 * @author BalchD
 * @author JohnstonD
 *  
 */

import uk.ac.ox.tall.wcker.Wcker;

public class WckerTree implements Cloneable {
	WckerTree src;	// for duplication purposes only
	WckerTree dst;  // for duplication purposes only
	boolean expanded;		// to take account of TreeItem expansions when copying! :-)
	boolean selected;		// to take account of TreeItem expansions when copying! :-)
	WckerTree parent;		// for reverse traversals
	public WckerTree[] children = new WckerTree[0];
	public Object payload;
 
	/**
	 * Get the root WckerTree Node
	 * @return WckerTree
	 */
	WckerTree get_root_node() {
		if(parent == (WckerTree) null) {
			return this;
		} else {
			return parent.get_root_node();
		}
	}
	
	WckerTree dup(WckerTree parent, WckerTree src, WckerTree dst) { // and all the little chidren too!
		WckerTree wt;
		try {
			wt = (WckerTree) this.clone();	
		}
		catch ( Exception e) {
			e.printStackTrace();
			wt = (WckerTree) null; // to get around compilation non-initialsation error
		}
		
		wt.parent = parent;
		if(this == src) {
			wt.src = wt;
		}
		
		if(this == dst) {
			wt.dst = wt;
		}

		wt.children = new WckerTree[children.length];
		for(int i = 0; i < children.length; ++i) {
	
			wt.children[i] = children[i].dup(wt,src,dst);
			if(wt.children[i].src != null) {
			    // propagate src upwards 
				wt.src = wt.children[i].src;
			}
			if(wt.children[i].dst != null) {
			    // propagate dst upwards
				wt.dst = wt.children[i].dst;
			}
		}
		return wt;
	}

	WckerTree dup(WckerTree parent)	{
	    // and all the little chidren too!
		WckerTree wt;
		try {
			wt = (WckerTree) this.clone();	
		}
		catch (Exception e) {
			e.printStackTrace();
			wt = (WckerTree) null; // to get aroun compilation non-initialsation error
		}
		wt.parent = parent;
		wt.children = new WckerTree[children.length];
		for(int i = 0; i < children.length; ++i) {
			wt.children[i] = children[i].dup(wt);
		}
		return wt;
	}

	WckerTree (WckerTree parent_arg, Object payload_arg) {
		parent =  parent_arg;
		payload = payload_arg;

		String ps = payload_to_string() ;
		if (Wcker.DEBUG > 1) {
			System.out.println("Wcker Tree Creation payload_to_string() = " +  ps);
		}
		
		if(ps.equals("????")) {
			Exception c;
			int num = 5;
			int denom = 0;
			int quot = num / denom;
			//throw new Exception(" help ");
		}
		
	}
/**************************************************************
	WckerTree (String name)
	{
		nodename = name;
		payload  = new StructurePayLoad(name);
	}
**************************************************************/

	/**
	 * double string argument contructor
         * to create tree node with two
	 * values
	 *
	 * @name first string argument
	 * @type_arg second string argument
	 */
/**************************************************************
	WckerTree (String name, String type_arg)
	{
		nodename = name;
		type     = type_arg;
		payload  = new OrganisationPayLoad(name, type_arg);
	}
*************************************************************/

	/**
	* recursive print routine for WckerTree object
	*/
	public void print() {
		System.out.println("tttttttttttttttttttttttttttttttttttttttttt");
		print0("");
	}
	
	String payload_to_string() {
		Class c = payload.getClass();
		String class_name = c.getName();
		
		String ret;
		if(class_name.equals("uk.ac.ox.tall.wcker.wizard.OrganisationPayLoad")) {
			ret = ((OrganisationPayLoad) payload).toString();
		}
		else if(class_name.equals("uk.ac.ox.tall.wcker.wizard.StructurePayLoad")) {
			ret = ((StructurePayLoad) payload).toString();
		}
		else {
			ret = "????";
		}
		//System.out.println("HEY: " + ret.hashCode());
		//System.out.println(ret);
		return ret;
	}

	private void print0(String tabs) {
		
		System.out.println(tabs + payload_to_string() );
		for(int i = 0; i < children.length; ++i) {
			children[i].print0(tabs + "\t");
		}

	}

	/**
	* produces a genuine duplicate of a WckerTree object in memory
	* otherwise object references values within the structure just 
	* get replicated
	*/
	WckerTree duplicate(WckerTree p) {
		WckerTree t = new WckerTree(p, payload); // keep the same parent !
		//t.nodename = new String(nodename);
		//t.type = new String(type);

		//t.payload = payload;
		
		Class c = payload.getClass();
		String class_name = c.getName();

		if(Wcker.DEBUG > 1) {
			System.out.println("Class name is " + class_name );
		}

		if(class_name.equals("uk.ac.ox.tall.wcker.OrganisationPayLoad")) {
			t.payload =  ((OrganisationPayLoad) payload).clone();
		}
		else if(class_name.equals("uk.ac.ox.tall.wcker.StructurePayLoad")) {
			t.payload =  ((StructurePayLoad) payload).clone();
		}
		else {
		}

		//t.payload = payload.clone();
		t.children = new WckerTree[children.length];

		for(int i=0;i<children.length;++i) {
			t.children[i] = children[i].duplicate(t);
		}
		return t;
	}
	
	/**
	* recursive routine to turn a WckerTree object into a string
	* @return string representation of a WckerTree
	*/
	public String toString() {	
		return "\n### TREE START ###\n " 
			+ toString0("")
			+ "### TREE STOP ###\n" ;
	}
	
	private String toString0(String tabs) {
		String s = tabs + payload_to_string();
		for(int i = 0; i < children.length; ++i) {
			try {
				s += children[i].toString0(tabs + "\t");
			}
			catch (Exception e) {
				s += tabs + "\t";
			}
		}
		return s;
	}
	
	String getName() {
		Class c = payload.getClass();
		String class_name = c.getName();
		String ret;
		if(class_name.equals("uk.ac.ox.tall.wcker.OrganisationPayLoad")) {
			ret = ((OrganisationPayLoad) payload).getName();
		}
		else if(class_name.equals("uk.ac.ox.tall.wcker.StructurePayLoad")) {
			ret = ((StructurePayLoad) payload).getName();
		}
		else {
			ret =   "????";
		}
		return ret;
	}
	String getType() {
		
		Class c = payload.getClass();
		String class_name = c.getName();
		String ret;
		if(class_name.equals("uk.ac.ox.tall.wcker.OrganisationPayLoad")) {
			ret =  ( (OrganisationPayLoad) payload).getType();
		}
		else if(class_name.equals("uk.ac.ox.tall.wcker.StructurePayLoad")) {
			ret =  "????";
		}
		else {
			ret =   "????";
		}
		return ret;
	}

	/**
	 * Return the questionHook String for this WckerTree
	 * @return
	 */
	public String getQuestionHook() {
		String hook = null;
		
		if (payload.getClass().getName().equals("uk.ac.ox.tall.wcker.wizard.OrganisationPayLoad")) {
			String qh = ((OrganisationPayLoad)payload).getQuestionHook();
			if (qh != null) {
				hook = qh;
			} else {
				hook = "";
			}
		} else {
			hook = "";
		}
		
		return hook;
	}
	
	/**
	 * Return the WckerTree matching a given questionHook
	 */
	public WckerTree getFirstWckerTreeByQHook(String hook) {
		WckerTree theTree = null;
		
		// Check to see if this WckerTree has a matching hook
		if (getQuestionHook().equals(hook)) {
			// The hook matches, return this tree
			theTree = this;
		} else if (children.length > 0) {
			// When the hook doesn't match, recurse through the children
			// NB: Uses depth first traversal - breadth first would probably be better 
			for (int i=0; i<children.length; i++) {
				children[i].getFirstWckerTreeByQHook(hook);
			}
		}
		
		return theTree;
	}
}
