package uk.ac.ox.tall.wcker.wizard;

import org.jdom.Element;

import uk.ac.ox.tall.wcker.Wcker;

public class WckerUtility {
	/**
     * Filter a title string, to make it suitable for using as part of a file path
     * @param titleString The original string
     * @return formatted string
     */
	static String formatTitleToPath(String titleString) {
		String formattedString = "";
		String removeChars = "[\\W]";

		if (titleString != "") {
			formattedString = titleString.replaceAll(removeChars, "");
		}
		
		return formattedString;
	}
	
	/**
	 * Get a file extension via the templateType of a wsi item.
	 * @param wsiItem
	 * @return fileExt
	 */
    static String getTemplateFileExt(Element wsiItem) {
    	String fileExt = "";

		if(Wcker.DEBUG > 1) {
			System.out.println("wsiItem: " + wsiItem);
			System.out.println("wsiItem.getAttributeValue(\"templatetype\"): " + wsiItem.getAttributeValue("templatetype"));
		}
    	String templateType = wsiItem.getAttributeValue("templatetype");
		
    	if (templateType == null) {
    		// No matching value, assume it's a directory
    		fileExt = "directory";
    	} else if (templateType.equals("text/html")) {
    		fileExt = ".html";
    	} else {
    		// No matching value, assume it's a directory
    		fileExt = "directory";
    	}

		if(Wcker.DEBUG > 1) {
			System.out.println("getTemplateFileExt()");
			System.out.println("templateType: " + templateType);
			System.out.println("fileExt: " + fileExt);
		}
		
		return fileExt;
    }
    
    /**
     * Get a file extension from a string.
     * @param fileName
     * @return fileExt
     */
    static String getFileExt(String fileName) {
    	int i = fileName.lastIndexOf('.');
    	if(i > 0 && i < fileName.length() - 1) {
    		return fileName.substring(i);
    	}
    	else {
    		return "";
    	}
    }
    
    /**
     * Get a directory path without the filename.
     * @param fileName
     * @return fileExt
     */
    static String getFilePath(String fileName) {
    	// XXX: Suspicious behaviour from System.getProperty("file.separator"), so using "/".
    	//int i = fileName.lastIndexOf(System.getProperty("file.separator"));
    	int i = fileName.lastIndexOf("/");
		if(Wcker.DEBUG > 1) {
			System.out.println("DB: getFilePath()");
			System.out.println("DB: fileName: " + fileName);
			System.out.println("DB: sep: " + System.getProperty("file.separator"));
			System.out.println("DB: i: " + i);
		}
    	if(i > 0 && i < fileName.length() - 1) {
    		return fileName.substring(0, i+1);
    	}
    	else {
    		return fileName;
    	}
    }
	
	static String string_list_list(String[][] l ) {
		String acc = "";
		for(int i = 0 ; i < l.length; i++) {
			acc += string_list_to_path(l[i]) + "\n";
		}
		return acc;
	}
	
	public static String string_list_to_path(String[] l ) {
		if ( l.length == 0 ) {
			return "/";
		}
		String acc = "";
		for(int i = 0 ; i < l.length; i++) {
			acc += "/" + l[i];
		}
		return acc;
	}
	
	public static String[][] string_3D_to_2D(String[][][] o) {
		int count = 0;
		for(int i = 0; i < o.length; ++i) {	
			count += o[i].length;
		}
		
		String[][] o2 = new String[count][];
		int l = 0;
		for(int i =0; i <o.length; ++i) {	
			for(int j =0; j < o[i].length; ++j) {
				o2[l] = o[i][j];
				l++;
			}
		}
		return o2;
	}
	
	public static String [][] map_prepend(String word, String[][] sentences) {
		int n_sentences = sentences.length;

		String [][] s = new String[n_sentences][];

		for(int i = 0; i < sentences.length;++i) {
			s[i] = prepend(word, sentences[i]);
		}
		return s;
	}
	
	public static String[] make_unique( String[] words_in) {
		int length = words_in.length;
		if(length <= 1) {
			return words_in;
		}
		
		// initislaise head
		String head = words_in[0];
		// initislaise tail
		String[] tail = new String[length-1];
		
		for(int i = 0; i < tail.length;++i) {
			tail[i] = words_in[i + 1];
		}
		for(int i = 0; i < tail.length;++i) {
			if(head.equals(tail[i])) {
				return make_unique(tail);
			}
		}
		return prepend( head, make_unique(tail) );
		
	}
	
	public static String[] prepend(String word, String[] old_sentence) {
			int n_words =  old_sentence.length;
			String [] new_sentence = new String[ n_words + 1];
			
			new_sentence[0] = word;
			for (int j = 0; j < n_words; ++j) {
				new_sentence[j + 1] = old_sentence[j];
			}
			return new_sentence;
	}
	


    // Add a DOM fragment from a .wsi XML prompt element
    static void boldEl(Element qEl, Element target) {
        Element eBold = new Element("b");
        eBold.setText(qEl.getValue());
        target.addContent(eBold);
		target.addContent(new Element("br"));
    }
}
