/*
 * Created on 22-Jun-2005
 *
 */
package uk.ac.ox.tall.wcker.wizardchooser;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.widgets.*;

import uk.ac.ox.tall.wcker.Wcker;

/**
 * @author David Balch
 * 
 * Class implementing a wizard page for WCKER wizard selection
 */
public class ChooserPage extends WizardPage implements Listener {

    // Wizard page details
	String pageTitle = "Choose a course wizard";
	String pageDesc = "Start a new course from a wizard below.";
   	
	WizardCache wc;		         // list of wizard file names
 	List wizardList;	         // List of GUI elements in Menu
	int selectedNewWizard;       // new wizard selected to run
	
	//GUI parts
	Composite startNewWizardSection; //Container for new wizard widgets
   	Button newWizardRadio;           // radio buttion for new Wizard
   	Composite continueWizardSection; //Container for existing wizard widgets
    Button continueWizardRadio;      // radio button to continue an existing Wizard
   	String continueWizardPathname;   // Store the path to the existing wizard in this String
	Button continueWizardBrowse;     // Browse button to find an existing Wizard
    Text continueWizardTextInput;    // Text input widget for existing Wizard path
	DirectoryDialog directoryDialog; // directory dialog widget if browse button pressed
	
	protected ChooserPage() {
		super("ChooserPage");
		setTitle(pageTitle);
		setDescription(pageDesc);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
	 */
	public void handleEvent(Event arg0) {
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		// Assign a Layout to our shell
	    FormLayout mainLayout = new FormLayout();
	    mainLayout.marginWidth = 3;
	    mainLayout.marginHeight = 3;
	    parent.setLayout(mainLayout);
		
		// Add Wizard chooser components:
		// Add "New wizard" parts
		startNewWizardSection = addStartNewWizardSection(parent);
		
		// Add "Load existing wizard" parts:
		continueWizardSection = addContinueWizardSectionComposite(parent);
		
		// Add listeners to detect when a wizard has been chosen.
		addSelectionListener();
		addDblClkListener();
		addNewWizardRadioListener();
		addContinueWizardRadioListener();
		addContinueWizardTextInputListener();
		addContinueWizardBrowseListener();
		
		// Page not complete until a wizard is selected.
	    setPageComplete(false);
	    
		//Assign as the control for this page.
		setControl(startNewWizardSection);
	}
	

	/* New wizard section components */
	Composite addStartNewWizardSection(Composite parent) {
		// "Start a new wizard" composite
		Composite startNewWizardSectionComposite = new Composite(parent, SWT.NONE);
		startNewWizardSectionComposite.setLayout(new FormLayout());

		//Add StartNewWizardSection radio button
		Button newWizardRadio = addNewWizardRadio(startNewWizardSectionComposite);
		
		//Add list of wizards
		// TODO: Setup a "Please wait, checking for available wizards" message.
		// After the wizard cache/available wizards are checked, display them.
		wizardList = addWizardList(startNewWizardSectionComposite);

		// Positioning within parent
		FormData startNewWizardSectionLayoutData = new FormData();
		startNewWizardSectionLayoutData.top = new FormAttachment(0,0);
		startNewWizardSectionLayoutData.bottom = new FormAttachment(100,-80);
		startNewWizardSectionLayoutData.left = new FormAttachment(0,0);
		startNewWizardSectionLayoutData.right = new FormAttachment(100,0);
		startNewWizardSectionComposite.setLayoutData(startNewWizardSectionLayoutData);
		
		return startNewWizardSectionComposite;
	}
	
	// Radio button to enable the existing wizard select box (a list)
	Button addNewWizardRadio(Composite parent){
		// Radio button and text prompt
		newWizardRadio = new Button(parent, SWT.RADIO);
		newWizardRadio.setText("Start a new wizard:");
		newWizardRadio.setSelection(true);
		return newWizardRadio;
	}

	// The list of wizards available to WCKER 
	List addWizardList (Composite parent) {
		// List of available wizards
		wizardList = new List(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER | SWT.SINGLE );
		// Positioning within parent
		FormData wizardListLayoutData = new FormData();
		wizardListLayoutData.height = 150;
		wizardListLayoutData.top = new FormAttachment(0,25);
		wizardListLayoutData.bottom = new FormAttachment(100,0);
		wizardListLayoutData.left = new FormAttachment(0,20);
		wizardListLayoutData.right = new FormAttachment(100,-3);
		wizardList.setLayoutData(wizardListLayoutData);
		
		// Add items to wizard list
		// Load up Wizard cache file	
		wc = new WizardCache(Wcker.wckerProps.getProperty("wcker.wizards.dir") + "wwz-cache.xml" );
		// For each wizard in the cache list
		for(int i=0; i < wc.size(); ++i) {
			if (Wcker.DEBUG > 1) {
				System.out.println("wc.size() " + wc.size());
				System.out.println("list index: " + i);
				System.out.println(wc.get(i).toString());
				System.out.println("");
			}
			//Add an item to the List widget 
			//wizardList.add(wc.get(i).toString());
			wizardList.add(((WizardCacheItem)wc.get(i)).getTitle());
		}
		
		return wizardList;
	}
	
	/* Continue wizard section components */
	Composite addContinueWizardSectionComposite(Composite parent) {
		// "Continue an existing wizard" composite
		Composite continueWizardSectionComposite = new Composite(parent, SWT.NONE);
		continueWizardSectionComposite.setLayout(new FormLayout());

		// Continue section Radio button
		Button continueWizardRadio = addContinueWizardRadio(continueWizardSectionComposite);
		
		// Continue section composite with file selection text and browse button
		Composite chooseExistingWizComposite = new Composite(continueWizardSectionComposite, SWT.NONE);
		// Positioning within parent (left indent)
		FormData chooseExistingLayoutData = new FormData();
		chooseExistingLayoutData.top = new FormAttachment(0,0);
		chooseExistingLayoutData.bottom = new FormAttachment(100,0);
		chooseExistingLayoutData.left = new FormAttachment(0,-10);
		chooseExistingLayoutData.right = new FormAttachment(100,0);
		chooseExistingWizComposite.setLayoutData(chooseExistingLayoutData);
		
		// Positioning of children
		GridLayout chooseExistingLayout = new GridLayout();
		chooseExistingLayout.numColumns = 2;
		chooseExistingWizComposite.setLayout(chooseExistingLayout);

		// Text input box
		continueWizardTextInput = new Text(chooseExistingWizComposite, SWT.BORDER);
		continueWizardTextInput.setEnabled(false);
		GridData inputGridData = new GridData(GridData.FILL_HORIZONTAL);
		inputGridData.horizontalIndent = 20;
		inputGridData.grabExcessHorizontalSpace = true;
		inputGridData.horizontalAlignment = GridData.FILL;
		continueWizardTextInput.setLayoutData(inputGridData);
		
		// File browsing button
		continueWizardBrowse = new Button(chooseExistingWizComposite, SWT.PUSH);
		continueWizardBrowse.setText("Browse");
		continueWizardBrowse.setEnabled(false);
		
		// Create directory dialogue widget
		directoryDialog = new DirectoryDialog(parent.getShell());
		directoryDialog.setText("Select a wizard to continue");
		directoryDialog.setMessage("Select the directory of a wizard in progress.");
		
		FillLayout continueWizardSectionCompositeLayout = new FillLayout();
		continueWizardSectionCompositeLayout.type = SWT.VERTICAL;
		continueWizardSectionComposite.setLayout(continueWizardSectionCompositeLayout);
		
		// Positioning of continueWizardSectionComposite within parent
		FormData continueWizardSectionLayoutData = new FormData();
		continueWizardSectionLayoutData.top = new FormAttachment(startNewWizardSection,5);
		continueWizardSectionLayoutData.bottom = new FormAttachment(100,0);
		continueWizardSectionLayoutData.left = new FormAttachment(0,0);
		continueWizardSectionLayoutData.right = new FormAttachment(100,0);
		continueWizardSectionComposite.setLayoutData(continueWizardSectionLayoutData);
		
		return continueWizardSectionComposite;
	}	
	
	// Radio button and text prompt
	Button addContinueWizardRadio(Composite parent){
		continueWizardRadio = new Button(parent, SWT.RADIO);
		continueWizardRadio.setText("Continue an existing wizard:");
		continueWizardRadio.setSelection(false);
		return continueWizardRadio;
	}
		
	Text addContinueWizardTextInput(Composite parent) {
		// text input
		continueWizardTextInput = new Text(parent, SWT.BORDER);
		// Positioning within parent
		FormData continueWizardTextInputLayoutData = new FormData();
		continueWizardTextInputLayoutData.top = new FormAttachment(0,0);
		continueWizardTextInputLayoutData.left = new FormAttachment(0,0);
		continueWizardTextInputLayoutData.right = new FormAttachment(100,0);
		continueWizardTextInput.setLayoutData(continueWizardTextInputLayoutData);
		continueWizardTextInput.setEnabled(false);
		
		return continueWizardTextInput;
	}
	
	Button addContinueWizardBrowse(Composite parent, Composite composite) {
		// File browsing button
		continueWizardBrowse = new Button(composite, SWT.PUSH);
		continueWizardBrowse.setText("Browse");
		continueWizardBrowse.setEnabled(false);
		
		// Create directory dialogue widget
		directoryDialog = new DirectoryDialog(parent.getShell());
		directoryDialog.setText("Select a wizard to continue");
		directoryDialog.setMessage("Select the directory of a wizard in progress.");
		
		return continueWizardBrowse;
	}
	
	/* Listeners */
	
	private void addNewWizardRadioListener() {
		// Listen for clicks on the radio button - enable/disable sections.
		newWizardRadio.addSelectionListener(
				new SelectionAdapter() {
					public void widgetSelected(SelectionEvent e) {
						///disable "continue" sections
						continueWizardRadio.setSelection(false);
						continueWizardTextInput.setEnabled(false);
						continueWizardBrowse.setEnabled(false);
						
						//enable "new" sections
						continueWizardRadio.setSelection(false);
						wizardList.setEnabled(true);
						
						//Enable or disable the select button, depending on whether there's an entry in the text box
						//selectedNewWizard = wizardList.getFocusIndex();
						selectedNewWizard = wizardList.getSelectionIndex();

						System.out.println("selected:" + selectedNewWizard);
						if (selectedNewWizard == -1) {
						    setPageComplete(false);
						} else {
						    setPageComplete(true);
						}
					}
				}
		);
		
	}
	
	private void addContinueWizardRadioListener() {
		// Listen for clicks on the radio button - enable/disable sections.
		continueWizardRadio.addSelectionListener(
				new SelectionAdapter() {
					public void widgetSelected(SelectionEvent e) {
						///disable "new" sections
						newWizardRadio.setSelection(false);
						wizardList.setEnabled(false);
						
						//enable "continue" sections
						continueWizardRadio.setSelection(true);
						continueWizardTextInput.setEnabled(true);
						continueWizardBrowse.setEnabled(true);
						
						//Enable or disable the select button, depending on whether there's an entry in the text box
						continueWizardPathname = continueWizardTextInput.getText();
						if (continueWizardPathname == "") {
						    setPageComplete(false);
						} else {
						    setPageComplete(true);
						}
						
						// Tell Wcker that we're loading an old wizard
						Wcker.setLoadType("reopen");
					}
				}
				
		);
	}
	
	private void addContinueWizardBrowseListener() {
		// Listen for clicks on the browse button - open the directory dialogue widget onclick
		continueWizardBrowse.addSelectionListener(
				new SelectionAdapter() {
					public void widgetSelected(SelectionEvent e) {
						String selectedDirectory = "";
						selectedDirectory = directoryDialog.open();
						if (selectedDirectory != null) {
							System.out.println("Selected directory: '" + selectedDirectory + "'.");
							continueWizardTextInput.setText(selectedDirectory);
							doSelection();
						} else {
							System.out.println("No directory selected");
						}
					}
				}
		);
	}
	private void addContinueWizardTextInputListener() {
		// Listen for an entry in the text input
		continueWizardTextInput.addModifyListener(
				new ModifyListener() {
					public void modifyText(ModifyEvent e) {
						if (continueWizardTextInput.getCharCount() > 0) {
							// Now we have some text, enable the select button
							//*
							// * TODO: We need entry validation of this text at some point,
							// * to ensure that it points to a real wizard course - 
							// * Proabably after hitting the "Select" button
							// * 
						    setPageComplete(true);
						} else {
						    setPageComplete(false);
							
						}
					}
				}
		);
	}
	// Listen for doubleclicks on list items, perform finish
	// (i.e. run the selected wizard)
	// XXX not attatched to the right widget, I think.
	private void addDblClkListener() {
	    wizardList.addMouseListener(
	    		new MouseAdapter() {
	    			public void mouseDoubleClick(MouseEvent e) {
	    				if (Wcker.DEBUG > 0) {
	    					System.out.println("Double-click");
	    				}
	    				doSelection();
	    				getWizard().performFinish();
	    			}
	    		}
	    	);
	    };
	
	private void addSelectionListener() {
		// Listen for clicks on the list of wizard packages
	    wizardList.addSelectionListener(
				new SelectionAdapter() {
					public void widgetSelected(SelectionEvent e) {
						// Tell Wcker that we're opening a new wizard package
						Wcker.setLoadType("newpackage");
						
						doSelection();
					}
				}
		);
	}
	
	private void doSelection() {
		System.out.println("doSelection()");
		System.out.println("Wcker.getLoadType(): " + Wcker.getLoadType());
		if (Wcker.getLoadType().equals("newpackage")) {
			// Load a new wizard from zip package.
			int wizard_index = wizardList.getSelectionIndex();
			WizardCacheItem wi = (WizardCacheItem) wc.get(wizard_index);
			
			if (Wcker.DEBUG > 1) {
				System.out.println("A new wizard has been selected (index=" 
						+ wizard_index + "|" + 	wizardList +  "|" + ": '");
			}
			
			if (wizardList.getSelection() != null) {
				setPageComplete(true);
				
				Wcker.setWizardChoice(wi.getFile());
				getWizard().canFinish();
			}
		} else {
			// Continue an existing wizard from directory
			Wcker.setWizardChoice(continueWizardTextInput.getText());
		}
	}
}
