/*
 * Created on 24-Jan-2005
 *
 */
package uk.ac.ox.tall.wcker.wizardchooser;

import java.io.*;
import java.util.*;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.xpath.XPath;
import org.jdom.output.*;

import uk.ac.ox.tall.wcker.Wcker;
import uk.ac.ox.tall.wcker.util.LoadXML;
import uk.ac.ox.tall.wcker.wizardloader.WizPackageHandler;
import uk.ac.reload.diva.util.ZipUtils;

/**
 * WizardCache - Deals with loading, modifying, and saving the 
 * wizard cache file on behalf of {@link ChooseWizard}.
 * <p>
 * The wizard cache file lists {@link WizardCacheItem}s, to
 * avoid having to load, unzip and parse an arbitrary number 
 * of .wwz wizard packages upon each run of WCKER.
 * 
 * @author BalchD
 *
 */
public class WizardCache extends LinkedList {

	/**
	 * 
	 */
	private static final long serialVersionUID = -8119404394377885051L;
	private File wizardCacheFile = null;
	private Document cacheDocument = null;
	private static Namespace nsWSI = Namespace.getNamespace("wsi", "http://www.tall.ox.ac.uk/schema/wcker/0.2");
	private static Namespace nsDC = Namespace.getNamespace("dc", "http://purl.org/dc/elements/1.1/");
	
	/**
	 * Turns a file containing wizard names into an
	 * in-memory list of names.
	 *
	 * @param cacheLocation the filepath of the wizard cache XML file
	 */
	
	public WizardCache(String cacheLocation) {
		if (Wcker.DEBUG > 1) {
			System.out.println("WizardCache class started");
			System.out.println("cacheLocation: " + cacheLocation);
		}
		
		wizardCacheFile = new File(cacheLocation);
		
		//Check if a cachefile exists, if it does - load it, if it doesn't - create one.
		if(wizardCacheFile.exists()){
			//load the cache file
			cacheDocument = LoadXML.loadXml(wizardCacheFile);
			
			//Parse the cache file into memory (this object)
			parseCache(cacheDocument);
			// Update cache against files in the wizard package directory
			updateCache();
		} else {
			//Generate the cache file from scratch
			buildCache();
			saveCache();
		}
	}

	/**
	 * Add all wizard packages to this WizardCache LinkedList
	 */
	private void buildCache() {
		// Get a array of the wizard package Files
		File[] WizardFiles = WizPackageHandler.getWizardFiles();
		
		// For each file in the array, add it to this WizardCache LinkedList
		for(int i =0; i < WizardFiles.length; ++i) {
			try {
				add(CacheItemFromPackage(WizardFiles[i]));
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public static WizardCacheItem CacheItemFromPackage(File newPackage) throws IOException {
		WizardCacheItem item = null;

		String file = newPackage.getName();
		Document wsiDoc;
		String title = null;
		String description = null;
		String version = null;
		
		if (Wcker.DEBUG > 1) {
			System.out.println("");
			System.out.println("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
			System.out.println("Adding wizard package to cache: " + newPackage.getName());
		}
		
		/* Doesn't work
			// TODO: Make this work
			// Extract the wizard.wsi into memory and create a Document object directly
			// from the zip file. I.e. don't load from disk, save to disk, load from disk.
			final int bufSize = 1024;
			
			// The .wsi file in the package zipfile
			ZipFile zipFile = new ZipFile(newPackage);
			ZipEntry zipWsi = zipFile.getEntry("wizard.wsi");
			int zipWsiSize = (int)zipWsi.getSize();
			
			String ZipEntryString = ZipUtils.extractZipEntry(newPackage, "wizard.wsi");
			
			InputStream wsiInputStream = zipFile.getInputStream(zipWsi);
			
			
			//InputStream wsiInputStream = ZipUtils.extractZipEntryStream(newPackage, "wizard.wsi");
			
			if (debug) {
				//System.out.println("ZipEntryString: " + ZipEntryString);
				System.out.println("zip: " + zipFile.getName());
				System.out.println("    zipWsi.getName(): " + zipWsi.getName());
				System.out.println("    zipWsiSize: " + zipWsiSize);
				System.out.println("    wsiInputStream: " + wsiInputStream);
				
				byte[] wsiInputStreamData = new byte[zipWsiSize];
				int n_read = wsiInputStream.read(wsiInputStreamData, 0, zipWsiSize);
				System.out.println("    Data node: " + wsiInputStreamData);
				System.out.println("    Bytes read from wsiInputStream (n_read): " + n_read);
				
				String someString = new String(wsiInputStreamData);
				System.out.println("someString: " + someString);
				
				 InputStreamReader contents = new InputStreamReader(wsiInputStream);
				 byte[] wsiInputStreamReaderData = new byte[zipWsiSize];
				 int n_read_reader = wsiInputStream.read(wsiInputStreamReaderData, 0, zipWsiSize);
				 System.out.println("    data node: " + wsiInputStreamReaderData);
				 System.out.println("    Bytes read from wsiInputStreamReaderData (n_read_reader): " + n_read_reader);
				 
				 String someString2 = new String(wsiInputStreamReaderData);
				 System.out.println("someString: " + someString2);
				 
			}
			
			//load the .wsi file
			wsiDoc = LoadXML.loadXml(wsiInputStream);
			
			if (debug) {
				System.out.println("wsiInputStream: " + wsiInputStream);
				System.out.println("wsiXml: " + wsiDoc);
			}
			
			*/
		
		File tempFile = new File(Wcker.wckerProps.getProperty("wcker.temp.dir") + "newWizTmp.xml");
		// Ensure that the parent Folder exists
		if(!tempFile.getParentFile().exists()) {
			tempFile.getParentFile().mkdirs();
		}
		
		File wsiFile = ZipUtils.extractZipEntry(newPackage, "wizard.wsi", tempFile);
		
		//load the .wsi file as an XML Document
		wsiDoc = LoadXML.loadXml(wsiFile);
		//Element rootEl = wsiDoc.getRootElement();

		XPath qgPath;
		Element metaEl = null;
		try {
			qgPath = XPath.newInstance("wsi:wizard/wsi:metadata");
			qgPath.addNamespace(nsWSI);
			qgPath.addNamespace(nsDC);
			metaEl = (Element)qgPath.selectSingleNode(wsiDoc);
			
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		
		version = metaEl.getChildText("version", nsWSI);
		title = metaEl.getChildText("title", nsDC);
		description = metaEl.getChildText("description", nsDC);
		
		if (Wcker.DEBUG > 1) {
			System.out.println("");
			System.out.println("Adding details from package: " + newPackage);
			System.out.println("  Title:       '" + title + "'");
			System.out.println("  Description: '" + description + "'");
			System.out.println("  Version:     '" + version + "'");
			System.out.println("  File:        '" + file + "'");
		}
		
		item = new WizardCacheItem(title, description, file, version);
		return item;
	}
	
	/**
	 * Save the cache list of wizards to an XML file
	 */
	public void saveCache() {
		if (Wcker.DEBUG > 1) {
			System.out.println("WizardCache.saveCache() started");
		}

		// Clear cacheDocument, and create from scratch 
		cacheDocument = new Document();
		
		// Set up basic XML file contents
		Element root = new Element("wizards");
		cacheDocument.setRootElement(root);
		
		// Convert LinkedList cache to an DOM Document
		for (int i = 0; i < size(); i++) {
			//Get values
			String versionVal = ((WizardCacheItem)get(i)).getVersion();
			String fileVal = ((WizardCacheItem)get(i)).getFile();
			String titleVal = ((WizardCacheItem)get(i)).getTitle();
			String descriptionVal = ((WizardCacheItem)get(i)).getDescription();
			
			if (Wcker.DEBUG > 1) {
				System.out.println("");
				System.out.println("versionVal: " + versionVal);
				System.out.println("fileVal: " + fileVal);
				System.out.println("titleVal: " + titleVal);
				System.out.println("descriptionVal: " + descriptionVal);
			}
			
			//Construct a "wizard" element for this cache entry, with attributes
			Element wizardEl = new Element("wizard");
			wizardEl.setAttribute("version", versionVal);
			wizardEl.setAttribute("file", fileVal);
			
			//Add the title child element
			Element titleEl = new Element("title");
			titleEl.setText(titleVal);
			wizardEl.addContent(titleEl);
			
			//Add the description child element
			Element descriptionEl = new Element("description");
			descriptionEl.setText(descriptionVal);
			wizardEl.addContent(descriptionEl);

			//Add this wizard element to the Document
			root.addContent(wizardEl);
		}
		
		// Save DOM Document
		writeXmlFile(cacheDocument, wizardCacheFile);
	}
	
	/** Parse the cache file (XML)
	 * Convert "in-memory" document to a LinkedList
	 */

	private void parseCache(Document cacheDoc){
		XPath wizardPath;
		List wizardCacheItems = null;
		try {
			wizardPath = XPath.newInstance("wizards/wizard");
			wizardCacheItems = wizardPath.selectNodes(cacheDoc);
		} catch (JDOMException e) {
			e.printStackTrace();
		}
		
		if (Wcker.DEBUG > 1) {
			System.out.println("~~~~~~~~~~~ Started parseCache() ~~~~~~~~~~~~~");
			System.out.println("Number of wizardCacheItems in the cacheDoc: " + wizardCacheItems.size() );
		}
		
		//Add each wizard in the cache file to a LinkedList 
		for(int i=0; i < wizardCacheItems.size(); ++i) {
			Element thisWizCacheItem = (Element)wizardCacheItems.get(i);
			String title = null;
			String description = null;
			String file = null;
			String version = null;
			
			if (Wcker.DEBUG > 1) {
				System.out.println("wizardCacheItems.item(i): " + wizardCacheItems.get(i));
			}
			
			title = thisWizCacheItem.getChildText("title");
			file = thisWizCacheItem.getAttributeValue("file");
			version = thisWizCacheItem.getAttributeValue("version");
			description = thisWizCacheItem.getChildText("description");
			
			if (Wcker.DEBUG > 1) {
				System.out.println("");
				System.out.println("Title:       '" + title + "'");
				System.out.println("Description: '" + description + "'");
				System.out.println("File:        '" + file + "'");
				System.out.println("Version:     '" + version + "'");
			}
			
			// Add details as a item in the list 
			add( new WizardCacheItem(title, description, file, version));
		}
		
	}

	/**
	 * Update cache file from wizard package files
	 * @param cacheLocation file name
	 */
	public void updateCache(){
		if (Wcker.DEBUG > 1) {
			System.out.println("");
			System.out.println("WizardCache.updateCache() started");
		}
		boolean cacheUpdated = false;
		
		/*
		 *  Compare the cache file LinkedList to the wizard directory contents
		 */
		
		File[] WizardFiles = WizPackageHandler.getWizardFiles();
		
		//For each file in the array,
		for(int i =0; i < WizardFiles.length; ++i) {
			boolean packageCached = false;
			
			// Get the file name
			String fileName = WizardFiles[i].getName();
			if (Wcker.DEBUG > 1) {
				System.out.println("");
				System.out.println("Wizard file: " + fileName );
			}
			
			//For each cache entry
			for(int j =0; j < size(); ++j){
				// Get the cached file name
				String cacheFileName = ((WizardCacheItem)get(j)).getFile();
				if (Wcker.DEBUG > 1) {
					System.out.println("Wizard filename found in cache: " + cacheFileName );
				}
				
				// Check whether the filenames between the file list and the cache entry match
				if (fileName.equals(cacheFileName)) {
					// If no entry exists, mark that we want to add it
					packageCached = true;
					break;
				}
			}

			if (Wcker.DEBUG > 1) {
				System.out.println("packageCached: " + packageCached );
			}
			//If we need to, update the cache, and mark the cache as updated
			if (!packageCached){
				try {
					// update the cache
					add(CacheItemFromPackage(WizardFiles[i]));
				} catch (IOException e) {
					// Auto-generated catch block
					e.printStackTrace();
				}
				cacheUpdated = true;
			}
		}
		/*
		 * TODO: Check for packages listed in the cache but are no longer in 
		 * the package directory, and remove them from the cache.
		 */
		
		
		// If changed, save the cache to an XML file
		if (cacheUpdated){
			if (Wcker.DEBUG > 1) {
				System.out.println("Cache updated, so save it.");
			}
			saveCache();
		};
		
	}
	
	// This method writes a DOM document to a file
	// TODO: put this in a sensible XML utility class.
	public static void writeXmlFile(Document doc, String filename) {
		// Prepare the output file
		File file = new File(filename);
		
		writeXmlFile(doc, file);
	}
	
	//	 This method writes a DOM document to a file
	public static void writeXmlFile(Document doc, File file) {
	    // Initialize FileWriter
	    FileWriter writer = null;
	      try {
	      writer = new FileWriter(file);
	    } catch(Exception e) {}

	    // initialize JDOM XMLOutputter with 2 spaces indent and newlines on
	    XMLOutputter prettyOutput = new XMLOutputter();
	    
	    // output the document to the file (via writer) and close writer 
	    try {
	    	prettyOutput.output(doc, writer);
	    	writer.close();
	    } catch(Exception e) {};
	}
}
