/*
 * Created on 14-Mar-2005
 *
 * @author David Balch david.balch@conted.ox.ac.uk
 * 
 */
package uk.ac.ox.tall.wcker.wizardloader;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.Date;

import uk.ac.ox.tall.wcker.Wcker;
import uk.ac.reload.diva.util.ZipUtils;

/**
 * A class to deal with loading data from WCKER Wizard packages (.wwz)
 * @author David Balch david.balch@conted.ox.ac.uk
 *
 */
public class WizPackageHandler {
	/**
	 * Get a list of the wizard (.wwz) files in the wizard directory.
	 * @return File[]
	 */
	public static File[] getWizardFiles() {
		//Create File array
		File WizardDirectory = new File(Wcker.wckerProps.getProperty("wcker.wizards.dir"));
		
		// Filter the file array - only include .wwz files.
		File[] WizardFiles = WizardDirectory.listFiles(
				new FilenameFilter() {
					public boolean accept(File WizardDirectoryFiles, String name) {
						return name.endsWith(".wwz");
					}
				});
		
		return WizardFiles;
	}

	/**
	 * Unzip a .wwz wizard package to the working directory
	 * @return File - the folder to which the package was extracted
	 */
	public static File extractPackage(String wizardPackage) {
		
		// The package to be unzipped
		File packageFile = new File(Wcker.wckerProps.getProperty("wcker.wizards.dir") + wizardPackage) ;
		
		// The output directory name, based on the wizard package name and the current datetime
		
		// Remove the ".wwz" file extension
		String wizardName = wizardPackage.substring(0, wizardPackage.lastIndexOf(".wwz"));
		
		// Get the current datetime, and format it.
		Date date = new Date();
		Format formatter = new SimpleDateFormat("yyyyMMddHHmmss");
		String dateString = formatter.format(date);
		
		// Concatenate into a path
		File targetFolder = new File(
		        Wcker.wckerProps.getProperty("wcker.working.dir") +
				wizardName + 
				"_" + 
				dateString
		);
		
		if (Wcker.DEBUG > 1) {
			System.out.println("    wizardName: " + wizardName);
			System.out.println("  targetFolder: " + targetFolder);
		}
		
		try {
			// Use the ZipUtils in Reload Diva to unpack the wizard package    	
			ZipUtils.unpackZip(packageFile, targetFolder);
		} catch (IOException e) {
			e.printStackTrace();
			if (Wcker.DEBUG > 0) {
				System.out.println("Couldn't unzip wizard package: " + wizardPackage);
			}
		}
		
		return targetFolder;
	}
}
