/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.editor.learningdesign.editor.environment;

import uk.ac.reload.editor.datamodel.DataComponent;
import uk.ac.reload.editor.datamodel.DataModel;
import uk.ac.reload.editor.gui.Editor_Tree;
import uk.ac.reload.editor.gui.Editor_TreeRenderer;
import uk.ac.reload.editor.learningdesign.datamodel.components.environments.Environment;


/**
 * The Tree to display Environments
 * 
 * @author Phillip Beauvoir
 * @version $Id: EnvironmentTree.java,v 1.8 2004/11/18 23:56:04 phillipus Exp $
 */
public class EnvironmentTree
extends Editor_Tree  {
    
    /**
     * Default Constructor
     */
    public EnvironmentTree() {
        setEditor_TreeModel(new EnvironmentTreeModel(this));
        
		// Tree Renderer
		setCellRenderer(new EnvironmentTreeRenderer());
    }

    /**
     * Set the LD_DataModel
     * @param dataModel The LD_DataModel
     */
    public void setDataModel(DataModel dataModel) {
        super.setDataModel(dataModel);

        // Expand node
		expandNode(getEditor_TreeModel().getRootNode(), true);
        
		// Select first node
		selectRootNode();
    }
    
    class EnvironmentTreeRenderer extends Editor_TreeRenderer {
        
        /**
         * Set the font according to the DataComponent
         * @param dc
         */
        protected void setFont(DataComponent dc) {
            if(dc instanceof Environment) {
                Environment env = (Environment)dc;
                if(env.getLearningObject_Grouping().hasChildren()
                        || env.getService_Grouping().hasChildren()
                        || env.getEnvironmentRef_Grouping().hasChildren()) {
                    setFont(getPlainFont());
                }
                else {
                    setFont(getItalicFont());
                }
            }
            else {
                super.setFont(dc);
            }
        }

    }
}
