/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */
package uk.ac.reload.scormplayer;

import java.io.File;
import java.io.IOException;

import org.jdom.JDOMException;

import uk.ac.reload.scormplayer.client.generic.contentpackaging.DocumentHandlerException;
import uk.ac.reload.scormplayer.client.generic.contentpackaging.NoItemFoundException;
import uk.ac.reload.scormplayer.tomcat.IServerHandler;

/**
 * Interface for methods that a ControlStation must implement.
 * These methods are concerned with handling a content package.
 * 
 * @author Paul Sharples
 * @version $Id: IControlStation.java,v 1.1 2004/07/19 16:05:41 ps3com Exp $
 */
public interface IControlStation extends IServerHandler{
    
    /**
     * check for valid manifest
     */
    public boolean isValidManifest(File file) throws DocumentHandlerException,
            JDOMException, IOException;

    /**
     * parse the manifest - load the content into something - ie JDOM
     * @param man
     * @throws NoItemFoundException
     * @throws JDOMException
     * @throws IOException
     */
    public void parseManifest(File man) throws NoItemFoundException,
            JDOMException, IOException;
    
    /**
     * check if a file ref is a zip
     * @param aFile
     * @return
     */
    public boolean isZip(File aFile);
}
