/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.generic.prefs;

import java.io.File;

import uk.ac.reload.diva.prefs.UserPrefs;
import uk.ac.reload.scormplayer.server.tomcat.ScormTomcatHandler;

/**
 * The User Prefs file.  Implementers will have to fulfil the abstract methods
 *
 * @author Phillip Beauvoir
 * @version $Id: ScormPlayerPrefs.java,v 1.2 2004/07/08 12:15:56 phillipus Exp $
 */
public class ScormPlayerPrefs extends UserPrefs {
	
	// GENERAL KEYS
	
	// The default Open folder
	public static final String GENERAL_PREFS_DEFAULT_FOLDER = "default_folder";
	// Look and Feel
	public static final String GENERAL_PREFS_LF = "look_and_feel";
	// default userid
	public static final String DEFAULT_USERID = "userid";
	// default username
	public static final String DEFAULT_USERNAME = "username";
	
	public static final String SHOW_TREE_WIDGET = "show_tree";
	
	public static final String SHOW_NAVIGATION = "show_navigation";
	
	public static final String AUTO_PROGRESSION = "auto_progress";
		
	public static final String PORT_NUMBER = "port_number";
    
	// Check prefs - starts up a little faster if set to false,
    // but more error-prone if you fool with the reload prefs folder
    // default is true
    public static final String GENERAL_CHECK_SUPPORT = "check_support";
	
	/**
	 * The Prefs Filename
	 */
	public static final String USER_PREFS_FILENAME = "reload_prefs.xml";
	
	/**
	 * The Prefs root element name
	 */
	public static final String USER_PREFS_ELEMENTNAME = "reload_prefs";
	
    /**
	 * Our singleton User Prefs
	 */
	private static ScormPlayerPrefs _userPrefs = new ScormPlayerPrefs();
	
	/**
	 * @return the Singleton
	 */
	public static ScormPlayerPrefs getInstance() {
		return _userPrefs;
	}
	
	/**
	 * Set some default values
	 */
	protected void setDefaultValues() {
		String value = getValue(DEFAULT_USERID);
		if(value == null) putValue(DEFAULT_USERID, "Reload-001");
		
		value = getValue(DEFAULT_USERNAME);
		if(value == null) putValue(DEFAULT_USERNAME, "User, Reload");
		
		value = getValue(PORT_NUMBER);
		if(value == null) putValue(PORT_NUMBER, "8080");

		// check prefs at startup
        value = getValue(GENERAL_CHECK_SUPPORT);
        if(value == null) putBooleanValue(GENERAL_CHECK_SUPPORT, true);
        
        value = getValue(SHOW_TREE_WIDGET);
        if(value == null) putBooleanValue(SHOW_TREE_WIDGET, true);
        
        value = getValue(SHOW_NAVIGATION);
        if(value == null) putBooleanValue(SHOW_NAVIGATION, true);
        
        value = getValue(AUTO_PROGRESSION);
        if(value == null) putBooleanValue(AUTO_PROGRESSION, false);
	}
	
	/**
	 * @return the Preferences File
	 */
	public File getPrefsFile() {
		return new File(getPrefsFolder(), USER_PREFS_FILENAME);
	}
	
	/**
	 * Location of prefs folder
	 * @return the Preferences Folder
	 */
	public File getPrefsFolder() {
		File reloadHome = ScormTomcatHandler.getSharedInstance().getPrefsFolder();
		reloadHome.mkdirs();
		return reloadHome;
	}
	
	/**
	 * @return the root Element name
	 */
	public String getElementRootName() {
		return USER_PREFS_ELEMENTNAME;
	}
	
}