/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.mainwindow.navigationwindow;

import java.util.Hashtable;
import java.util.Iterator;

import org.jdom.Element;

import uk.ac.reload.moonunit.contentpackaging.CP_Core;

import uk.ac.reload.dweezil.gui.tree.DweezilTreeModel;

import uk.ac.reload.scormplayer.client.generic.contentpackaging.NavigationViewer;

/**
 * Our tree model for the navigation tree (shows the organizations)
 * 
 * @author Paul Sharples
 * @version $Id: NavigationTreeModel.java,v 1.3 2004/07/19 16:40:22 ps3com Exp $
 */
public class NavigationTreeModel extends DweezilTreeModel {

    /**
     * A temporary hashtable used to work out the tree structure for each
     * organization. Its cleared once the model is loaded
     */
    protected static Hashtable items = new Hashtable();

    protected NavigationViewer _navViewer;

    /**
     * default constructor
     */
    public NavigationTreeModel(){}

    /**
     * Give the tree an organization element and build the view
     * 
     * @param theOrg
     */
    public void setTree(Element theOrg, NavigationViewer nv) {
        _navViewer = nv;
        if(theOrg != null) {
            root = new NavigationTreeNode(theOrg, nv);
            String orgID = _navViewer.getTitleOfElement(theOrg);
            items.put(orgID, root);
            buildView((NavigationTreeNode)root);
        } 
        else {
            this.setRoot(null);
        }
        reload();
        items.clear();
    }

    /**
     * Build up child nodes
     */
    public void buildView(NavigationTreeNode node) {
        if(node == null)
            return;
        Iterator children = node.getElement().getChildren().iterator();
        while(children.hasNext()) {
            Element child = (Element)children.next();
            String name = child.getName();
            if(name.equals(CP_Core.ITEM)) {
                NavigationTreeNode newNode = new NavigationTreeNode(child,
                        _navViewer);
                String itemID = _navViewer.getTitleOfElement(child);
                items.put(itemID, newNode);
                node.add(newNode);
                
                if(_navViewer.findScoType(child).equals(CP_Core.MANIFEST)) {
                    // get the referenced node
                    Element ref_element = _navViewer.getScormCore().getReferencedElement(child);
                    if(ref_element != null && ref_element.getName().equals(CP_Core.MANIFEST)) {
                        if(child.getChild(CP_Core.ITEM) == null){
                            System.out.println("adding submanifest");
                            Element orgsElement = ref_element.getChild(CP_Core.ORGANIZATIONS, ref_element.getNamespace());
	                        // Now we have to get the default ORGANIZATION
	                        if(orgsElement != null) {
	                            ref_element = _navViewer.getScormCore()
	                                    .getDefaultOrganization(orgsElement);
	                        }
	                        // Get the children of the referenced <organization>
	                        // element and graft clones
	                        if(ref_element != null) {
	                            Iterator it = ref_element.getChildren().iterator();
	                            while(it.hasNext()) {
	                                Element ref_child = (Element)it.next();
	                                child.addContent((Element)ref_child.clone());
	                            }
	                        }
                        }
                    }                                                            
                }
                buildView(newNode);
            }
        }

    }
}
/*
 * Element ref_element =
 * _navViewer.getScormCore().getReferencedElement(element); if(ref_element !=
 * null) { String ref_name = ref_element.getName(); // A RESOURCE
 * if(ref_name.equals(CP_Core.RESOURCE)) {
 *  } // A sub-MANIFEST else if(ref_name.equals(CP_Core.MANIFEST)) { // Get
 * ORGANIZATIONS Element Element orgsElement =
 * ref_element.getChild(CP_Core.ORGANIZATIONS, ref_element.getNamespace()); //
 * Now we have to get the default ORGANIZATION if(orgsElement != null)
 * ref_element = _navViewer.getScormCore().getDefaultOrganization(orgsElement); //
 * Get the children of the referenced <organization> element and graft clones
 * if(ref_element != null) { Iterator it = ref_element.getChildren().iterator();
 * while(it.hasNext()) { Element ref_child = (Element) it.next();
 * element.addContent((Element) ref_child.clone()); } } }
 * 
 *  /*
 * 
 * Iterator children = node.getElement().getChildren().iterator();
 * while(children.hasNext()) { Element child = (Element)children.next();
 * if(doShowNode(child)) { ManifestTreeNode newNode = new
 * ManifestTreeNode(child); node.add(newNode); buildChildren(newNode); } }
 *  
 */

