/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.mainwindow.navigationwindow;

import java.io.File;

import org.jdom.Element;

import uk.ac.reload.dweezil.gui.tree.DweezilTreeNode;
import uk.ac.reload.scormplayer.client.generic.contentpackaging.NavigationViewer;

import uk.ac.reload.scormplayer.client.gui.mainwindow.MainWindow;
import uk.ac.reload.scormplayer.client.gui.mainwindow.packagewindow.PackageTreeNode;
import uk.ac.reload.scormplayer.server.tomcat.ScormTomcatHandler;

import uk.ac.reload.moonunit.contentpackaging.SCORM12_Core;

/**
 * A class to hold a JDOM element as a Navigation tree node.
 * @author Paul Sharples
 * @version $Id: NavigationTreeNode.java,v 1.3 2004/07/19 16:37:52 ps3com Exp $
 */
public class NavigationTreeNode extends DweezilTreeNode{

    protected File _CMIModel = null;
    
    protected NavigationViewer _navViewer;

    public NavigationTreeNode(Element name, NavigationViewer nv) {
        super(name);
        _navViewer = nv;
        String scoType = whatIsThis();
        if (scoType.equals(SCORM12_Core.SCO) && name.getName().equals(SCORM12_Core.ITEM)) {
            // get the item ID
            String id = name.getAttributeValue(SCORM12_Core.IDENTIFIER);

            // get the path
            PackageTreeNode node = (PackageTreeNode) MainWindow._treePanel.
                getPackageTree().getSelectionPath().getLastPathComponent();
            // now get file reference            
            File scoFile = ScormTomcatHandler.getSharedInstance().getScoSettings(node.toString(), id);
            if (scoFile.exists()) {            
                _CMIModel = scoFile;
            }
        }
    }

    public String getLaunch(){
        Element aNode = (Element) getUserObject();
        String name = aNode.getName();        
        if (name.equals(SCORM12_Core.ITEM) && _navViewer.isReferencingElement(aNode)){
            String launch = _navViewer.getLaunch(aNode);
            if (launch != null) {
                if (launch != ""){
                    return launch;
                }
                else return "none";
            }
            else return "none";
        }
        else return "none";
    }


    public String getPrerequistes(){
        Element aNode = (Element) getUserObject();
        String name = aNode.getName();
        if (name.equals(SCORM12_Core.ITEM)){
            String prereq = _navViewer.getPrerequisites(aNode);
            if (prereq != null) {                
                    return prereq;
            }
            else return "none";
        }
        else return "";
    }



    /**
     * Accessor method to return the Identifier for this item/org
     * @return
     */
    public String getIdentifier(){
        Element aNode = (Element) getUserObject();
        String name = aNode.getName();
       // If this element is an ITEM..
       if (name.equals(SCORM12_Core.ITEM)){
           String id = aNode.getAttributeValue(SCORM12_Core.IDENTIFIER);
           if (id != null) return id;
           else return "";
       }
       // If this is the organization
       else if (name.equals(SCORM12_Core.ORGANIZATION)){
           return aNode.getAttributeValue(SCORM12_Core.IDENTIFIER);
       }
       else return "";
    }


    /**
     * Accessor Method to return the file reference to the CMI xml file for this
     * sco.
     * @return - file reference
     */
    public File getSCOFile(){
        return _CMIModel;
    }


    /**
     * Accessor Method to return if an item is visible or not
     * @return
     */
    public boolean isVisible() {
        Element aNode = (Element) getUserObject();
        String name = aNode.getName();
        // If this element is an ITEM..
        if (name.equals(SCORM12_Core.ITEM)) {
            String answer = aNode.getAttributeValue(SCORM12_Core.ISVISIBLE);
            if (answer != null) {
                if (answer.equals("false")) {
                    return false;
                }
                else {
                    return true;
                }
            }
            else {
                return true;
            }
        }
        else {
            return true;
        }
    }

    /**
    * Get the JDOM Element
    * @return The JDOM Element
    */
   public Element getElement() {
       return (Element)getUserObject();
   }


   /**
    * Method to find out if this is a sco/asset/neither
    * @return - a string identifying what this is
    */
   public String whatIsThis(){
       Element aNode = (Element) getUserObject();
       String name = aNode.getName();
       // If this element is an ITEM..
       if (name.equals(SCORM12_Core.ITEM)){
           String scoType = _navViewer.findScoType(aNode);
           if (scoType != null){               
               return scoType;               
           }
           else {
               return "none";
           }
       }
       return "none";
   }

   /**
    * Accessor method to get the title of the item/org
    * @return a string representing the title of this item/org
    */
   public String toString(){
       Element aNode = (Element) getUserObject();
       String name = aNode.getName();
       // If this element is an ITEM..
       if (name.equals(SCORM12_Core.ITEM)){
           String scoType = _navViewer.findScoType(aNode);
           // is this a sco/asset/neither
           if (scoType != "" && scoType != null && scoType != "none"){               
               return _navViewer.getTitleOfElement(aNode) + " (" + scoType + ")";                              
           }
           else{
               return _navViewer.getTitleOfElement(aNode);
           }
        // else is this an Org
       }
       else if (name.equals(SCORM12_Core.ORGANIZATION)){
           return _navViewer.getTitleOfElement(aNode);
       // shouldn't get to here, but just in case...
       }
       else{
           return "unknown";
       }
   }

}

