/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.mainwindow.navigationwindow;

import java.awt.Component;

import javax.swing.JTree;

import uk.ac.reload.dweezil.gui.tree.DweezilTreeRenderer;
import uk.ac.reload.dweezil.util.DweezilUIManager;
import uk.ac.reload.scormplayer.client.gui.IIcons;

/**
 * Our tree renderer for the Navigation (organizations) tree. In charge of setting icons
 * and tool tips
 *
 * @author Paul Sharples
 * @version $Id: NavigationTreeRenderer.java,v 1.3 2004/07/20 00:05:45 phillipus Exp $
 */
public class NavigationTreeRenderer
extends DweezilTreeRenderer
implements IIcons
{

    public NavigationTreeRenderer() {}

    public Component getTreeCellRendererComponent(JTree tree, Object value,
                                                  boolean selected,
                                                  boolean expanded,
                                                  boolean leaf, int row,
                                                  boolean hasFocus) {
        NavigationTreeNode selNode = (NavigationTreeNode) value;
        // Hilight node as we drag over it
        if (selNode.isHiLited)
            selected = true;
            // Set some icons
        if (selNode.getParent() == null) { // Root Node
            closedIcon = DweezilUIManager.getIcon(ICON_CP_SCORM_CLOSED);
            openIcon = DweezilUIManager.getIcon(ICON_CP_SCORM_OPENED);
            leafIcon = closedIcon;
            setToolTipText("Course Structure");
        }
        else if (!selNode.isVisible()){
            closedIcon = DweezilUIManager.getIcon(ICON_HIDDEN_ITEM_CLOSED);
            openIcon = DweezilUIManager.getIcon(ICON_HIDDEN_ITEM_LEAF);
            leafIcon = DweezilUIManager.getIcon(ICON_HIDDEN_ITEM_LEAF);
            setToolTipText("Hidden Item - "+ selNode.toString());
        }
        else {
            closedIcon = DweezilUIManager.getIcon(ICON_SCORM_FOLDER_CLOSED);
            openIcon = DweezilUIManager.getIcon(ICON_SCORM_FOLDER_OPENED);
            leafIcon = DweezilUIManager.getIcon(ICON_SCORM_ITEM);
            setToolTipText("Item - "+ selNode.toString());
        }
        return super.getTreeCellRendererComponent(tree, value, selected,
                                                  expanded, leaf, row, hasFocus);
    }
}
