/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.mainwindow.navigationwindow;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ToolTipManager;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreePath;

import org.jdom.Element;

import uk.ac.reload.moonunit.contentpackaging.SCORM12_Core;
import uk.ac.reload.dweezil.gui.DweezilComboBox;
import uk.ac.reload.scormplayer.client.generic.contentpackaging.NavigationViewer;
import uk.ac.reload.scormplayer.client.gui.mainwindow.packagewindow.PackageTreeNode;
import uk.ac.reload.scormplayer.properties.ScormPlayerProperties;


/**
 * A class to show, update organizations using swing components.
 * The OrganizationPanel show the the Organizations available within
 * a scorm package.  It also allows the user to change/view the default
 * organization, using a combobox and lets them set another organization
 * as default if they wish to.
 * 
 * @author Paul Sharples
 * @version $Id: OrganizationPanel.java,v 1.3 2004/07/19 16:36:06 ps3com Exp $
 */
public class OrganizationPanel extends JPanel
implements TreeSelectionListener, ActionListener {

    /**
     * A helper class that allows us to trverse the navigation xml file
     * using JDOM
     */
    protected NavigationViewer _navView;

    /**
     * Our actual JTree show the items in an organization
     */
    protected NavigationTree _navTree;

    /**
     * A combobox to allow the user to change organizations
     */
    protected DweezilComboBox _organizationsComboBox;


    protected JPanel _contentPanel;

    /**
     * A panel holding the checkbox which allows the user to set an
     * organization as default
     */
    protected ChooseDefaultOrgPanel _defaultOrgPanel;

    /**
     * A list of items to show in the combobox
     */
    protected String[] _orgs = {"none"};

    /**
     * Default constructor which sets various layouts and add listeners
     */
    public OrganizationPanel() {
        _navTree = new NavigationTree();
        ToolTipManager.sharedInstance().registerComponent(_navTree);
        _organizationsComboBox = new DweezilComboBox(_orgs);
        _defaultOrgPanel = new ChooseDefaultOrgPanel(this);
        _organizationsComboBox.addActionListener(this);
        setLayout(new BorderLayout());

        _contentPanel = new JPanel();
        _contentPanel.setLayout(new BorderLayout());
        _contentPanel.setBorder(null);
        _defaultOrgPanel.setBorder(null);

        _navTree.setBorder(null);
        _contentPanel.add(_navTree, BorderLayout.CENTER);
        _contentPanel.add(_defaultOrgPanel, BorderLayout.SOUTH);

        this.add(_organizationsComboBox, BorderLayout.NORTH);

        this.add(new JScrollPane(_contentPanel), BorderLayout.CENTER);

        _organizationsComboBox.setEnabled(false);
    }

    /**
     * ActionPerformed.  Is listening to change events on the JComboBox
     * and also the JCheckbox
     * @param e
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == _organizationsComboBox){
            DweezilComboBox combo = (DweezilComboBox) e.getSource();
            int index = combo.getSelectedIndex();
            if (_navView != null){
                Element[] orgs = _navView.getOrganizationList();
                if (orgs != null && orgs.length > 0) {
                    _navTree.setTree(orgs[index], _navView);
                    boolean isDefaultSelected = _navView.getDefaultOrgByIndex(
                        index);
                    if (isDefaultSelected) {
                        _defaultOrgPanel.setCheckBoxState(true);
                    }
                    else {
                        _defaultOrgPanel.setCheckBoxState(false);
                    }
                }
            }
        }
        else{
            int index = _organizationsComboBox.getSelectedIndex();
            if (_defaultOrgPanel._setDefOrg.isSelected() && _navView != null){
                _navView.updateDefaultOrg(index);
            }
        }
         _navTree.setSelectionRow(0);
    }


    /**
     * Is listening to change events on the package tree. Once the user
     * selects a different package, then the organization tree must automatically
     * update to show that packages default organization
     * @param event
     */
    public void valueChanged(TreeSelectionEvent event) {
        // first remove any listeners for the combox box because we are going
        // to change the organization list, which will fire action performed().
        //  We will put the ActionListener back when we have finished.
        _navTree.setSelectionRow(0);
        _organizationsComboBox.removeActionListener(this);
        TreePath selPath = event.getPath();
        PackageTreeNode selectedNode = (PackageTreeNode)selPath
                .getLastPathComponent();
        File navFile = selectedNode.getNavigationFile();        
        if(navFile != null && navFile.exists()) {
            try {
                _navView = new NavigationViewer(navFile);                              
                // check to see if there any organizations..               
                if(!_navView.getDefaultOrg().equals("") && _navView.getDefaultOrg()!=null) {                    
                    updateView(_navView.getOrganizationList(), _navView
                            .getDefaultOrg());
                    _organizationsComboBox.setEnabled(true);
                    _navTree.setSelectionRow(0);
                    _defaultOrgPanel.setEditable(true);
                } 
                else {
                    noOrgsFound();
                }
            } 
            catch(Exception ex) {               
                System.out.println("Error reading in Organizations list: " + ex);
                if(ScormPlayerProperties.getString("DEBUG").equals("true")){
                    ex.printStackTrace();
                }
            }
        } 
        else {
            noOrgsFound();
        }
        // Put the Action listener back now we have finished populating
        // the combox box with this packages organizations.
        _organizationsComboBox.addActionListener(this);
    }


    private void noOrgsFound(){      
      _organizationsComboBox.setEnabled(false);
      _organizationsComboBox.setItems(_orgs);
      _navTree.setTree(null, null);
      _defaultOrgPanel.setEditable(false);
    }

    /**
     * Method to update the item list in the combobox
     * @param orgs
     * @param defaultOrg
     */
    private void updateView(Element[] orgs, String defaultOrg) {        
        String[] titles = new String[orgs.length];
        String atitle;
        int indexOfDefaultOrg = 0;
        for (int i = 0; i < orgs.length; i++) {
            atitle = _navView.getTitleOfElement(orgs[i]);                
            if (atitle != null || atitle != "") {
                titles[i] = atitle;
            }
            else {
                titles[i] = "Untitled";
            }            
            if (orgs[i].getAttributeValue(SCORM12_Core.IDENTIFIER).toString().equals(defaultOrg)) {
                indexOfDefaultOrg = i;
            }
        }
        _organizationsComboBox.setItems(titles);
        _organizationsComboBox.setSelectedIndex(indexOfDefaultOrg);
        _navTree.setTree(orgs[indexOfDefaultOrg], _navView);
        _defaultOrgPanel.setCheckBoxState(true);
    }


    public NavigationTree getNavTree(){
        return _navTree;
    }

}