/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.mainwindow.packagewindow;

import java.io.File;

import javax.swing.JOptionPane;

import uk.ac.reload.dweezil.gui.tree.DweezilTreeModel;
import uk.ac.reload.scormplayer.properties.ScormPlayerProperties;
import uk.ac.reload.scormplayer.server.tomcat.ScormTomcatHandler;
import uk.ac.reload.scormplayer.client.generic.contentpackaging.NavigationViewer;
import uk.ac.reload.scormplayer.client.generic.contentpackaging.NoItemFoundException;
import uk.ac.reload.scormplayer.client.gui.ScormPlayerFrame;
import uk.ac.reload.scormplayer.ControlStation;
/**
 * Our tree model for the package tree of currently imported packages
 * 
 * @author Paul Sharples
 * @version $Id: PackageTreeModel.java,v 1.4 2004/07/19 16:33:45 ps3com Exp $
 */
public class PackageTreeModel extends DweezilTreeModel{

    public PackageTreeModel() {}

    public void setTree(){
        File appFolder = ScormTomcatHandler.getSharedInstance().getCoursesPath();
        root = new PackageTreeNode(appFolder);        
        buildView( (PackageTreeNode) root, appFolder);
        reload();
    }

    /**
     * Build up child nodes
     */
    public void buildView(PackageTreeNode node, File start) {
        if(node == null || start == null) return;
        File[] children = start.listFiles();
        if(children == null) return;

        for(int i = 0; i < children.length; i++) {
            if (children[i].isDirectory()){
                // check to see if the sco files are there..                
                checkScoSettings(children[i]);
                PackageTreeNode newNode = new PackageTreeNode(children[i]);
                node.add(newNode);
            }
        }
    }

    /**
     * In this method we are trying to ascertain whether or not the sco xml files
     * exist for a particular package - if they are not there then we reset the package
     * forcing re-interpretation of the manifest and recreating the sco xml files.
     * @param aPackageFolder
     */
    public void checkScoSettings(File aPackageFolder){
        try {
            boolean hasBeenReset = false;
            NavigationViewer _navView = new NavigationViewer(ScormTomcatHandler.getSharedInstance().getSettings(aPackageFolder.getName()));
            if(!_navView.doScoSettingsExist() || !_navView.doesNavFileExist()){
                ControlStation.getSharedInstance().resetPackage(aPackageFolder);
                hasBeenReset = true;
            }
            if(_navView.hasManifestChanged() && !hasBeenReset){
                if(ScormPlayerProperties.getString("DEBUG").equals("true")){
                JOptionPane.showMessageDialog(ScormPlayerFrame.getInstance(),
                        "Manifest in package ("+aPackageFolder.getName()+") has been modified, resetting course.");
                }
                ControlStation.getSharedInstance().resetPackage(aPackageFolder);
            }
            
        }
        catch (NoItemFoundException ex) {
            System.out.println("No Items found in package "  + aPackageFolder.getName());
        }
        catch(Exception ex) {
            if(ScormPlayerProperties.getString("DEBUG").equals("true")){
                ex.printStackTrace();
            }
            System.out.println("Could not determine if manifest has changed: " + ex);
        }
        
    }
}