/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.menu;

import java.awt.Toolkit;
import java.awt.event.KeyEvent;

import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import uk.ac.reload.dweezil.gui.DweezilToolBar;
import uk.ac.reload.diva.util.GeneralUtils;


/**
 * The Main Menu for the Application.  It extends JMenuBar.  It also takes care
 * of the Toolbar and any main Popup menus.  It can listen to any refresh requests
 * so that for example, a change will set enabled and disabled menu items and buttons.
 * This class is tied in with the MainMenuAction and MenuAction classes.
 *
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version $Id: MainMenu.java,v 1.6 2004/07/19 16:31:17 ps3com Exp $
 */
public class MainMenu
extends JMenuBar
{
	/**
	 * The Singleton instance
	 */
	public static MainMenu sharedInstance = new MainMenu();
	
	public MenuAction_StartServer _startServer = new MenuAction_StartServer();
	public MenuAction_StopServer _stopServer = new MenuAction_StopServer();
	public MenuAction_StartPackage _startPackage = new MenuAction_StartPackage();
	public MenuAction_Refresh _stopPackage = new MenuAction_Refresh();
	public MenuAction_ResetPackage _resetPackage = new MenuAction_ResetPackage();
	public MenuAction_RemovePackage _removePackage = new MenuAction_RemovePackage();
	
	/**
	 * The main application toolbar
	 */
	private DweezilToolBar toolBar;
	
	/**
	 * The Key Mask will be 'Meta' for Mac and 'Ctrl' for PC/Unix
	 */
	private int keyMask = Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();
	
	/**
	 * Constructor
	 */
	private MainMenu() {
		// Attach Actions to Menu Items
		createMenuBar();
		
		// Create the ToolBar
		toolBar = createToolBar();
	}
	
	
	// =========================== MENU BAR ==================================
	/**
	 * The File Menu
	 */
	public JMenu fileMenu;
	
	/**
	 * The Run Menu
	 */
	public JMenu runMenu;
	
	/**
	 * The Server Menu
	 */
	public JMenu serverMenu;
	
	/**
	 * The Tools Menu
	 */
	public JMenu toolsMenu;
	
	/**
	 * The View Menu
	 */
	public JMenu viewMenu;
	
	/**
	 * The Help Menu
	 */
	public JMenu helpMenu;
	
	/**
	 * Create the Menu Bar
	 */
	private void createMenuBar() {
		// File Menu
		fileMenu = createFileMenu();
		//run menu
		runMenu = createRunMenu();
		// Server menu
		serverMenu = createServerMenu();
		// Tools menu
		toolsMenu = createToolsMenu();
		// View Menu
		viewMenu = createViewMenu();
		// Help Menu
		helpMenu = createHelpMenu();
	}
	
	/**
	 * Create the File Menu
	 * @return The JMenu
	 */
	private JMenu createFileMenu() {
		JMenu jmenu = add(new JMenu("File"));
		jmenu.setMnemonic('f');
		
		
		// Import CP
		
		JMenuItem item = jmenu.add(new MenuAction_OpenContentPackage());
		item.setMnemonic('i');
		
		
		// Remove package
		item = jmenu.add(_removePackage);
		item.setMnemonic('r');
		
        if(GeneralUtils.getOS() != GeneralUtils.MACINTOSH) {
            jmenu.addSeparator();

            // Exit
            item = jmenu.add(new MenuAction_Exit());
            item.setMnemonic('x');
        }

		return jmenu;
	}
	
	
	/**
	 * Create the Run Menu
	 * @return The JMenu
	 */
	private JMenu createRunMenu() {
		JMenu jmenu = add(new JMenu("Run"));
		jmenu.setMnemonic('r');
		
		JMenuItem item = new JMenuItem(_startPackage);
		jmenu.add(item);
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F9, 0));
		item.setMnemonic('r');
		
		jmenu.addSeparator();
		
		item = new JMenuItem(_resetPackage);
		jmenu.add(item);
		item.setMnemonic('t');
		
		return jmenu;
	}
	
	
	
	
	/**
	 * Create the Server Menu
	 * @return The JMenu
	 */
	private JMenu createServerMenu() {
		JMenu jmenu = add(new JMenu("Server"));
		jmenu.setMnemonic('s');
		JMenuItem item = jmenu.add(_startServer);
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));
		item.setMnemonic('s');						
		return jmenu;
	}
	
	/**
	 * Create the Options Menu
	 * @return The JMenu
	 */
	private JMenu createToolsMenu() {
		JMenu jmenu = null;
		
		if(GeneralUtils.getOS() != GeneralUtils.MACINTOSH) {
			jmenu = add(new JMenu("Tools"));
			jmenu.setMnemonic('t');						
			JMenuItem item = jmenu.add(new MenuAction_Prefs());
			item.setMnemonic('o');
		}
		
		return jmenu;
	}
	
	/**
	 * Create the View Menu
	 * @return The JMenu
	 */
	private JMenu createViewMenu() {
		JMenu jmenu = add(new JMenu("View"));
		jmenu.setMnemonic('v');
		
		JMenuItem item = jmenu.add(new MenuAction_StatusWindow());
		item.setMnemonic('s');						
		return jmenu;
	}
	
	
	/**
	 * Create the Help Menu
	 * @return The JMenu
	 */
	private JMenu createHelpMenu() {
		JMenu jmenu = add(new JMenu("Help"));
		jmenu.setMnemonic('h');
		
		// Help
		JMenuItem item = jmenu.add(new MenuAction_Help());
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));
		item.setMnemonic('h');
		
        if(GeneralUtils.getOS() != GeneralUtils.MACINTOSH) {
            jmenu.addSeparator();

            // About
            item = jmenu.add(new MenuAction_About());
            item.setMnemonic('a');
        }
		
		return jmenu;
	}
	
	// =========================== TOOLBAR ==================================
	
	/**
	 * Create the Toolbar.
	 * @return The Toolbar
	 */
	private DweezilToolBar createToolBar() {
		DweezilToolBar tBar = new DweezilToolBar();
		
		// import a SCORM CP
		MenuAction_OpenContentPackage actionOpenCp = new
		MenuAction_OpenContentPackage();
		tBar.add(actionOpenCp);
		
		// remove a SCORM CP
		tBar.add(_removePackage);
		
		// Add a separator
		tBar.addSeparator();
		
		// Run a SCORM Package
		tBar.add(_startPackage);
		
		// Reset a SCORM Package
		tBar.add(_resetPackage);
		
		// Add a separator
		tBar.addSeparator();
		
		// Start Tomcat Server
		tBar.add(_startServer);
				
		// Refresh screen
		tBar.add(_stopPackage);
		
		return tBar;
	}
	
	/* 
	 * Workaround for Sun Java bug #4910521 where buttons grow big when changing L&F
	 * http://developer.java.sun.com/developer/bugParade/bugs/4910521.html
	 * @see javax.swing.JComponent#updateUI()
	 */
	public void updateUI() {
	    super.updateUI();
	    if(toolBar != null) toolBar.updateUI();
	}

	/**
	 * Get the ToolBar.
	 * @return The main toolbar
	 */
	public DweezilToolBar getToolBar() {
		return toolBar;
	}
	
}

