/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.menu;

import java.awt.event.ActionEvent;

import javax.swing.JOptionPane;

import uk.ac.reload.dweezil.util.DweezilUIManager;
import uk.ac.reload.dweezil.menu.MenuAction;
import uk.ac.reload.scormplayer.ControlStation;
import uk.ac.reload.scormplayer.client.gui.IIcons;
import uk.ac.reload.scormplayer.client.gui.ScormPlayerFrame;
import uk.ac.reload.scormplayer.properties.ScormPlayerProperties;
import uk.ac.reload.scormplayer.server.tomcat.ScormTomcatHandler;

/**
 * Menu option to start/stop the web server
 * 
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version $Id: MenuAction_StartServer.java,v 1.4 2004/05/24 21:56:21 ps3com
 *          Exp $
 */
public class MenuAction_StartServer
extends MenuAction
implements IIcons
{

    public boolean isSelected = false;

    public MenuAction_StartServer() {
        super("Start Server", ICON_TOMCAT_START);
        setEnabled(true);
    }

    public void actionPerformed(ActionEvent e) {
        // Start the loader thread
        Thread thread = new Thread() {

            public void run() {
                try {
                    if(isSelected == false) {
                        setButtonIcon(DweezilUIManager
                                .getIcon(ICON_TOMCAT_END));
                        setMenuIcon(DweezilUIManager
                                .getIcon(ICON_TOMCAT_END));
                        setEnabled(false);

                        ControlStation.getSharedInstance().startServer();
                        JOptionPane.showMessageDialog(ScormPlayerFrame
                                .getInstance(), ScormTomcatHandler
                                .getSharedInstance().getServerInfo()
                                + " Started on Port:"
                                + ScormTomcatHandler.getSharedInstance()
                                        .getPortNumber());

                        setText("Stop Server");
                        setEnabled(true);
                        isSelected = true;
                    } 
                    else {
                        setButtonIcon(DweezilUIManager
                                .getIcon(ICON_TOMCAT_START));
                        setMenuIcon(DweezilUIManager
                                .getIcon(ICON_TOMCAT_START));
                        setEnabled(false);
                        ControlStation.getSharedInstance().stopServer();
                        JOptionPane.showMessageDialog(ScormPlayerFrame
                                .getInstance(), ScormTomcatHandler
                                .getSharedInstance().getServerInfo()
                                + " Stopped on Port:"
                                + ScormTomcatHandler.getSharedInstance()
                                        .getPortNumber());

                        setEnabled(true);
                        setText("Start Server");
                        isSelected = false;
                    }
                }
                catch(Exception ex) {
                    if(ScormPlayerProperties.getString("DEBUG").equals("true")) {
                        ex.printStackTrace();
                    }
                    if(ex.getMessage().indexOf("Address already in use") == -1) {
                        JOptionPane.showMessageDialog(ScormPlayerFrame
                                .getInstance(), "Server could not be started."
                                + System.getProperty("line.separator")
                                + "Reason:" + ex.getMessage());
                    } 
                    else {
                        JOptionPane.showMessageDialog(ScormPlayerFrame
                                .getInstance(), "Server could not be started."
                                + System.getProperty("line.separator")
                                + "Reason: current port number ("
                                + ScormTomcatHandler.getSharedInstance()
                                        .getPortNumber()
                                + ") is already in use."
                                + System.getProperty("line.separator")
                                + "Please change this in the preferences.");

                    }
                    setButtonIcon(DweezilUIManager
                            .getIcon(ICON_TOMCAT_START));
                    setMenuIcon(DweezilUIManager
                            .getIcon(ICON_TOMCAT_START));
                    setEnabled(true);
                }
                finally {
                }
            }
        };
        thread.start();

    }
}