/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.prefs;

import java.awt.Component;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.UIManager;

import uk.ac.reload.diva.prefs.UserPrefs;
import uk.ac.reload.dweezil.gui.layout.XYConstraints;
import uk.ac.reload.dweezil.gui.layout.XYLayout;
import uk.ac.reload.dweezil.prefs.PrefsPanel;
import uk.ac.reload.dweezil.util.DweezilUIManager;
import uk.ac.reload.scormplayer.client.generic.prefs.ScormPlayerPrefs;
import uk.ac.reload.scormplayer.client.gui.ScormPlayerFrame;



/**
 * An Appearance Settings User Prefs Panel
 *
 * @author Phillip Beauvoir
 * @version $Id: AppearancePrefsPanel.java,v 1.4 2004/07/08 12:18:52 phillipus Exp $
 */
public class AppearancePrefsPanel
extends PrefsPanel
{
	/**
	 * Combo box
	 */
	private JComboBox lfComboBox;
	
	private static final String LF_LABEL = "Look & Feel: ";
	
	private UIManager.LookAndFeelInfo[] systemLookAndFeels = UIManager.getInstalledLookAndFeels();
	
	/**
	 * Previous L&F
	 */
	private String oldLF;
	
	private boolean updateLFCombo;
	
	/**
	 * Constructor
	 */
	public AppearancePrefsPanel() {
		setLayout(new XYLayout());
		initGUI();
	}
	
	/**
	 * Init the GUI
	 */
	private void initGUI() {
		LookAndFeel[] installedLookAndFeels = new LookAndFeel[systemLookAndFeels.length];
		
		for(int i = 0; i < systemLookAndFeels.length; i++) {
			installedLookAndFeels[i] = new LookAndFeel(systemLookAndFeels[i]);
		}
		lfComboBox = new JComboBox(installedLookAndFeels);
		
		int x = 10;
		int y = 10;
		
		add(new JLabel(LF_LABEL), new XYConstraints(x, y, 0, 0));
		add(lfComboBox, new XYConstraints(x += 100, y, 100, COMBOBOX_HEIGHT));

		lfComboBox.addActionListener(new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				if(updateLFCombo) {
				    setAppearance(((LookAndFeel) lfComboBox.getSelectedItem()).getClassName());
				}
			}
		});
	}
	
	/**
	 * Update UserPrefs according to the controls' settings
	 */
	public void saveToUserPrefs(UserPrefs prefs) {
		prefs.putValue(ScormPlayerPrefs.GENERAL_PREFS_LF,
				((LookAndFeel) lfComboBox.getSelectedItem()).getClassName()
		);
	}
	
	/**
	 * Set the controls' settings
	 */
	public void setFields(UserPrefs prefs) {
		// Save current L&F
		oldLF = UIManager.getLookAndFeel().getClass().getName();
		
	    // Don't listen to combo
		updateLFCombo = false;

	    if(oldLF != null) {
		    for (int i = 0; i < systemLookAndFeels.length; i++) {
				if (systemLookAndFeels[i].getClassName().equals(oldLF)) {
					lfComboBox.setSelectedIndex(i);
					break;
				}
			}
		}
		
		updateLFCombo = true;
	}
	
	public void setAppearance(String className) {
		DweezilUIManager.setLookAndFeel(className, new Component[]{ ScormPlayerFrame.getInstance(),
		        ScormPlayerFrame.getInstance().getStatusWindow(),
				AppearancePrefsPanel.this.getParent().getParent()});
		
	}
	
	private class LookAndFeel {
		private UIManager.LookAndFeelInfo lookAndFeelInfo;
		
		public LookAndFeel(UIManager.LookAndFeelInfo lookAndFeelInfo) {
			this.lookAndFeelInfo = lookAndFeelInfo;
		}
		
		public String toString() {
			return lookAndFeelInfo.getName();
		}
		
		public String getClassName() {
			return lookAndFeelInfo.getClassName();
		}
		
	}

	/* 
	 * Cancel any changes
	 */
	public void cancel() {
		if(oldLF != null) {
			String currentLF = UIManager.getLookAndFeel().getClass().getName();
			if(!oldLF.equals(currentLF)) setAppearance(oldLF);
		}
	}
}
