/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */


package uk.ac.reload.scormplayer.client.gui.prefs;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTextField;

import uk.ac.reload.diva.prefs.UserPrefs;
import uk.ac.reload.dweezil.gui.layout.XYConstraints;
import uk.ac.reload.dweezil.gui.layout.XYLayout;
import uk.ac.reload.dweezil.prefs.PrefsPanel;
import uk.ac.reload.scormplayer.client.generic.prefs.ScormPlayerPrefs;

/**
 * A LMS Settings User Prefs Panel
 *
 * @author Paul Sharples
 * @version $Id: LMS_PrefsPanel.java,v 1.3 2004/07/08 12:18:52 phillipus Exp $
 */
public class LMS_PrefsPanel
extends PrefsPanel{

    /**
     * Default UserId
     */
    private JTextField _tfDefaultUserID;

    /**
     * Default Username
     */
    private JTextField _tfDefaultUserName;
    
    
    private JCheckBox _showTreeWidgetCheckBox;
    
    private JCheckBox _showNavigationCheckBox;

    private JCheckBox _enableAutoProgressionCheckBox;
    
    
    public LMS_PrefsPanel() {
        setLayout(new XYLayout());
        constructFields();
    }

    /**
   * Set up the fields
   */
  private void constructFields() {
      int x = 10;
      int y = 10;

      // ============ Default User id to use ==================
      JLabel idLabel = new JLabel("UserID to use in player:");
      add(idLabel, new XYConstraints(x, y + 2, 0, 0));
      _tfDefaultUserID = new JTextField();
      add(_tfDefaultUserID, new XYConstraints(x + 200, y, 300, TEXTBOX_HEIGHT));

      y += 30;

      // ============ Default User id to use ==================
      JLabel userLabel = new JLabel("Username to use in player:");
      add(userLabel, new XYConstraints(x, y + 2, 0, 0));
      _tfDefaultUserName = new JTextField();
      add(_tfDefaultUserName, new XYConstraints(x + 200, y, 300, TEXTBOX_HEIGHT));
      
      _showTreeWidgetCheckBox = new JCheckBox("Show tree widget in player"); 
      _showNavigationCheckBox = new JCheckBox("Show navigation in player");
      _enableAutoProgressionCheckBox = new JCheckBox("Enable auto progression of items in player");
      
      y += 30;
      add(_showTreeWidgetCheckBox, new XYConstraints(x, y , 0, 0));
      y += 30;
      add(_showNavigationCheckBox, new XYConstraints(x, y, 0, 0));
      y += 30;
      add(_enableAutoProgressionCheckBox, new XYConstraints(x, y, 0, 0));      
  }

  /**
    * Update UserPrefs according to the controls' settings
    */
   public void saveToUserPrefs(UserPrefs prefs) {
       prefs.putValue(ScormPlayerPrefs.DEFAULT_USERID, _tfDefaultUserID.getText());
       prefs.putValue(ScormPlayerPrefs.DEFAULT_USERNAME, _tfDefaultUserName.getText());
       prefs.putBooleanValue(ScormPlayerPrefs.SHOW_TREE_WIDGET, _showTreeWidgetCheckBox.isSelected());
       prefs.putBooleanValue(ScormPlayerPrefs.SHOW_NAVIGATION, _showNavigationCheckBox.isSelected());
       prefs.putBooleanValue(ScormPlayerPrefs.AUTO_PROGRESSION, _enableAutoProgressionCheckBox.isSelected());       
   }


   /**
    * Set the fields' settings from those found in UserPrefs
    */
   public void setFields(UserPrefs prefs) {
       // Set value in field
       _tfDefaultUserID.setText(prefs.getValue(ScormPlayerPrefs.DEFAULT_USERID));
       _tfDefaultUserName.setText(prefs.getValue(ScormPlayerPrefs.DEFAULT_USERNAME));
       _showTreeWidgetCheckBox.setSelected(prefs.getBooleanValue(ScormPlayerPrefs.SHOW_TREE_WIDGET));
       _showNavigationCheckBox.setSelected(prefs.getBooleanValue(ScormPlayerPrefs.SHOW_NAVIGATION));
       _enableAutoProgressionCheckBox.setSelected(prefs.getBooleanValue(ScormPlayerPrefs.AUTO_PROGRESSION));
   }

   /* 
    * Cancel any cahnges
    */
   public void cancel() {
   	
   }
}