/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.client.gui.prefs;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;

import javax.swing.*;

import uk.ac.reload.diva.prefs.UserPrefs;
import uk.ac.reload.dweezil.gui.ErrorDialogBox;
import uk.ac.reload.dweezil.util.DweezilUIManager;
import uk.ac.reload.scormplayer.client.generic.prefs.ScormPlayerPrefs;
import uk.ac.reload.scormplayer.client.gui.IIcons;
import uk.ac.reload.scormplayer.client.gui.ScormPlayerFrame;

/**
 * The Preferences Dialog Window
 *
 * @author Phillip Beauvoir
 * @author Paul Sharples
 * @version $Id: PrefsDialog.java,v 1.5 2004/07/20 00:05:44 phillipus Exp $
 */
public class PrefsDialog
extends JDialog
implements IIcons
{

    /**
     * The General Prefs Panel
     */
    private General_PrefsPanel _generalPrefsPanel;

    /**
     * The LMS Prefs Panel
     */
    private LMS_PrefsPanel _lmsPrefsPanel;

    /**
     * The User Preferences
     */
    private UserPrefs _userPrefs;

    /**
     * The server preference panel
     */
    private Server_PrefsPanel  _serverPrefsPanel;

    /**
     * The Look & Feel
     */
    private AppearancePrefsPanel _appearancePrefsPanel;

	private static PrefsDialog _instance;
	
	/**
	 * @return Static instance
	 */
	public static PrefsDialog getSharedInstance() {
	    if(_instance == null) _instance = new PrefsDialog();
	    return _instance;
	}

	/**
     * Constructor
     */
    public PrefsDialog() {
        super(ScormPlayerFrame.getInstance(), "Preferences", true);
        // Dialog Size
        setSize(580, 350);
        setLocationRelativeTo(ScormPlayerFrame.getInstance());
        setResizable(false);

        // Make some panels
        _generalPrefsPanel = new General_PrefsPanel();
        _lmsPrefsPanel = new LMS_PrefsPanel();
        _serverPrefsPanel = new Server_PrefsPanel();
        _appearancePrefsPanel = new AppearancePrefsPanel();
        
        // Get the User Prefs from file
        _userPrefs = ScormPlayerPrefs.getInstance();

        // Tabbed Pane
        JTabbedPane tabPane = new JTabbedPane();
        tabPane.addTab("General", DweezilUIManager.getIcon(ICON_APP16), _generalPrefsPanel);
        tabPane.addTab("LMS Options", DweezilUIManager.getIcon(ICON_CP_SCORM_CLOSED), _lmsPrefsPanel);
        tabPane.addTab("Server Options",DweezilUIManager.getIcon(ICON_TOMCAT_START), _serverPrefsPanel);
        tabPane.addTab("Appearance", DweezilUIManager.getIcon(ICON_THREAD), _appearancePrefsPanel);
        getContentPane().add(tabPane, BorderLayout.CENTER);

        // Button Panel
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));

        // OK Button
        JButton btnOK = new JButton("OK");
        btnOK.addActionListener(new OKClick());
        getRootPane().setDefaultButton(btnOK);
        buttonPanel.add(btnOK);

        // Cancel Buttton
        JButton btnCancel = new JButton("Cancel");
        btnCancel.addActionListener(new CancelClick());
        buttonPanel.add(btnCancel);

        // Add bottom Panel
        JPanel bottomPanel = new JPanel(new BorderLayout());
        bottomPanel.add(buttonPanel, BorderLayout.EAST);
        getContentPane().add(bottomPanel, BorderLayout.SOUTH);
    }

    /**
     * Over-ride this so we can set up the panels
     */
    public void show() {
        // Do this later because a modal dialog will block
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                // Update the Panels
                _generalPrefsPanel.setFields(_userPrefs);
                _lmsPrefsPanel.setFields(_userPrefs);
                _serverPrefsPanel.setFields(_userPrefs);
                _appearancePrefsPanel.setFields(_userPrefs);
            }
        });
        super.show();
    }

    /**
     * User pressed OK
     */
    protected void finish() {
        // Update the User Prefs
        _generalPrefsPanel.saveToUserPrefs(_userPrefs);
        _lmsPrefsPanel.saveToUserPrefs(_userPrefs);
        _serverPrefsPanel.saveToUserPrefs(_userPrefs);
        _appearancePrefsPanel.saveToUserPrefs(_userPrefs);
        
        try {
            _userPrefs.save();
        }
        catch(Exception ex) {
            ErrorDialogBox.showWarning("Could not save Preferences: ", "Preferences", ex);
        }

        dispose();
    }

    /**
     * OK handler
     */
    private class OKClick extends AbstractAction {
        public void actionPerformed(ActionEvent e) {
            finish();
        }
    }

    /**
     * Cancel handler
     */
    private class CancelClick extends AbstractAction {
        public void actionPerformed(ActionEvent e) {
        	// Cancel anything
        	_generalPrefsPanel.cancel();
        	_lmsPrefsPanel.cancel();
        	_serverPrefsPanel.cancel();
        	_appearancePrefsPanel.cancel();
        	
        	dispose();
        }
    }
}