/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.server.servlets;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import uk.ac.reload.scormplayer.properties.ScormPlayerProperties;
import uk.ac.reload.scormplayer.server.manager.ScormManager;
import uk.ac.reload.scormplayer.server.tomcat.ScormTomcatHandler;

/**
 * A servlet class used to start the whole (server side) ball rolling.
 * Its passed a package name and a couple of boolean flag (show tree widget or
 * not - show navigation or not).  These are passed as param to the scorm
 * manager class which will try to load in all of the xml data files and
 * populate our various models needed.
 * 
 * @author Paul Sharples
 * @version $Id: ScormLaunch.java,v 1.2 2004/07/19 16:01:55 ps3com Exp $
 */
public class ScormLaunch extends HttpServlet {
    
    /**
     * Our singleton scorm package to play.
     */
    public static ScormManager scormManager;
    
    public static final String CONTENT_TYPE = "text/html";
          
    protected static HttpSession _session;
    
    public static HttpSession getHttpSession(){
        return _session;
    }
       
    //Process the HTTP Get request
    public void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {     
        // grab all of the name value pairs from the url.
        String scormID = request.getParameter("scormPackageID");
        boolean nav = Boolean.valueOf(request.getParameter("nav")).booleanValue();
        boolean tree = Boolean.valueOf(request.getParameter("tree")).booleanValue();
        boolean autoProg = Boolean.valueOf(request.getParameter("ap")).booleanValue();
        // make sure there is something in the "scormID" parameter, otherwise we
        // can't carry on...
        if (scormID == "" || scormID == null) {
            throw new ServletException();
        }
        else {
            try {
                _session = request.getSession();
                // Instantiate our *singleton* scorm manager class with
                // the appropriate parameters
                scormManager = new ScormManager(scormID, tree, nav, autoProg);
                // If the scorm manager class could not find the correct xml files
                // then forward user to an error page
                if(scormManager.isErrorFound()) {
                    response.sendRedirect(ScormTomcatHandler.getSharedInstance().getErrorPage());
                }
                else if(!scormManager.hasItemsFound()) {
                    response.sendRedirect(ScormTomcatHandler.getSharedInstance().getNoItemsFoundPage());
                }                
                // otherwise, the scorm manager class didn't find any errors, so
                // lets try and load the frameset and API etc.
                else {
                    response.sendRedirect(ScormTomcatHandler.getSharedInstance().getLMSPage());
                }
            }
            catch (Exception ex) {
                System.out.println("Error trying to play package." + ex);
                if(ScormPlayerProperties.getString("DEBUG").equals("true")){
                    ex.printStackTrace();
                }
                response.sendRedirect(ScormTomcatHandler.getSharedInstance().getErrorPage());
            }
        }
        // shouldn't really get this far, but just in case...
        response.setContentType(CONTENT_TYPE);
    }
    
    
    //Process the HTTP Post request
    public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException {
        response.setContentType(CONTENT_TYPE);
        PrintWriter out = response.getWriter();
        out.println("<html>");
        out.println("<head><title>ScormLaunch</title></head>");
        out.println("<body bgcolor=\"#ffffff\">");
        out.println("<p>This servlet has received a POST request, but it can only handle GET requests.</p>");
        out.println("</body></html>");
    }
    
    //Clean up resources
    public void destroy() {}
    
}