/**
 *  RELOAD TOOLS
 *
 *  Copyright (c) 2003 Oleg Liber, Bill Olivier, Phillip Beauvoir, Paul Sharples
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *  Project Management Contact:
 *
 *  Oleg Liber
 *  Bolton Institute of Higher Education
 *  Deane Road
 *  Bolton BL3 5AB
 *  UK
 *
 *  e-mail:   o.liber@bolton.ac.uk
 *
 *
 *  Technical Contact:
 *
 *  Phillip Beauvoir
 *  e-mail:   p.beauvoir@bolton.ac.uk
 *
 *  Paul Sharples
 *  e-mail:   p.sharples@bolton.ac.uk
 *
 *  Web:      http://www.reload.ac.uk
 *
 */

package uk.ac.reload.scormplayer.tomcat;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import org.apache.catalina.Connector;
import org.apache.catalina.Context;
import org.apache.catalina.Engine;
import org.apache.catalina.Host;
import org.apache.catalina.LifecycleException;
import org.apache.catalina.core.StandardHost;
import org.apache.catalina.logger.SystemOutLogger;
import org.apache.catalina.startup.Embedded;
import org.apache.catalina.valves.RemoteAddrValve;

import uk.ac.reload.scormplayer.properties.ScormPlayerProperties;


/**
 * An embedded version of Tomcat server 4.1.18.  This class is used
 * to invoke Tomcat and is based on James Goodwill example found at
 * http://www.onjava.com/pub/a/onjava/2002/04/03/tomcat.html
 *
 * @author James Goodwill. Modifications made by Paul Sharples.
 * @version $Id: EmbeddedTomcat.java,v 1.4 2004/07/20 09:53:43 ps3com Exp $
 */

public class EmbeddedTomcat {

    // keep a track of all loaded contexts
    private Vector loadedContexts;

    // Boolean to see if server already up & running
    private boolean isAlive = false;

    //private String path = null;
    private Embedded embedded = null;
    private Host host = null;
    private Engine engine = null;
    private Connector connector = null;
    
    // the tomcat handler for this instance of tomcat
    protected ITomcatHandler _tomcatHandler;
                    
    /**
	 * Constructor - must be given a class implementing ITomcatHandler
	 */
	public EmbeddedTomcat(ITomcatHandler tomcatHandler) {
	    _tomcatHandler  = tomcatHandler;
	}
        
    public boolean isAlive() {
        return isAlive;
    }

    /**
     * This method Starts the Tomcat server.
     */
    public void startTomcat() throws Exception {       
            try {
                // Set the home directory
                System.setProperty("catalina.home", _tomcatHandler
                        .getTomcatPath().toString());
                // Create an embedded server
                embedded = new Embedded();
                // print all log statments to standard error
                if(ScormPlayerProperties.getString("DEBUG").equals("true")) {
                    embedded.setDebug(0);
                    embedded.setLogger(new SystemOutLogger());
                }
                // Create an engine
                engine = embedded.createEngine();
                engine.setDefaultHost(_tomcatHandler.getHostName());
                // Create a default virtual host
                host = embedded.createHost(_tomcatHandler.getHostName(),  // ie 127.0.0.1
                        _tomcatHandler.getWebAppsPath().toString() ); // ie PATH_TO_TOMCAT/webapps

                engine.addChild(host);

                // A little bit of security is needed here. We'll block all
                // addresses requesting server content, bar our own machine
                // on the local loop 127.0.0.1
                RemoteAddrValve remoteAddresses = new RemoteAddrValve();
                remoteAddresses.setDeny("");
                String[] allowedMachines = _tomcatHandler.getAllowedMachines();
                for(int i = 0; i < allowedMachines.length; i++) {
                    remoteAddresses.setAllow(allowedMachines[i]);

                }
                ((StandardHost)host).addValve(remoteAddresses);

                // next add our default contexts
                loadedContexts = new Vector();
                Context context;
                Hashtable defaultContexts = _tomcatHandler.getDefaultContexts();
                Enumeration keys = defaultContexts.keys();
                while(keys.hasMoreElements()) {
                    String path = (String)keys.nextElement();
                    String docBase = (String)defaultContexts.get(path);
                    context = embedded.createContext(path, _tomcatHandler
                            .getTomcatPath()
                            + docBase);
                    host.addChild(context);
                    loadedContexts.add(context);
                }
                // Install the assembled container hierarchy
                embedded.addEngine(engine);
                // Assemble and install a default HTTP connector
                connector = embedded.createConnector(null, _tomcatHandler
                        .getPortNumber(), false);
                embedded.addConnector(connector);
                // Start the embedded server
                embedded.start();
                isAlive = true;
            } catch(LifecycleException ex) {
                throw new Exception(ex);
            }        
    }

    /**
     * This method Stops the Tomcat server.
     */
    public void stopTomcat() throws Exception {
        // Stop the embedded server
        Iterator listElement;
        listElement = loadedContexts.iterator();
        while (listElement.hasNext()) {
            embedded.removeContext( (Context) listElement.next());
        }
        embedded.removeConnector(connector);
        embedded.removeEngine(engine);
        embedded.removeHost(host);
        embedded.stop();
        isAlive = false;
    }

}