
package uk.ac.reload.scormplayer.client.generic.contentpackaging;

import java.io.File;
import java.util.List;
import java.util.Iterator;
import org.jdom.Element;
import uk.ac.reload.diva.util.FileUtils;
import uk.ac.reload.moonunit.contentpackaging.SCORM12_Core;
import uk.ac.reload.scormplayer.properties.ScormPlayerProperties;
import uk.ac.reload.scormplayer.server.tomcat.ScormTomcatHandler;
import uk.ac.reload.testsupport.FileSupport;
import uk.ac.reload.testsupport.scorm.SCORM12_Package1;
import uk.ac.reload.testsupport.scorm.SCORM12_Package2;
import uk.ac.reload.testsupport.scorm.SCORM12_Package3;
import uk.ac.reload.testsupport.cp.CP_Package1;
import junit.framework.TestCase;


/**
 * JUnit Test class for ScormPackageHandler
 * 
 * @author Paul Sharples
 * @version $Id: ScormPackageHandlerTest.java,v 1.2 2004/07/21 16:40:30 ps3com Exp $
 */
public class ScormPackageHandlerTest extends TestCase {

    /**
     * Testing manifest in the correct location in temp folder
     */
    private File _manifestToTest;
    
    /**
     * The path to test data
     */
    private String pathTestData;
    
    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        // See if we have a testdata dir set from an Ant script
        pathTestData = System.getProperty("testdata.dir");
        if(pathTestData == null) {
            // Set a default
            pathTestData = "../junit-tests";
        }          
        // Set Properties file to test properties
		System.setProperty("scormplayer.properties.file", "uk.ac.reload.scormplayer.testproperties.rb");  
    }

    
    protected void setManifest(File man, String name) throws Exception{
        File manifestTestLocation = new File(ScormPlayerProperties.getFileProperty("coursepackage.dir") + "\\" + name , man.getName());        
        if (!manifestTestLocation.exists()){
            manifestTestLocation.getParentFile().mkdirs();
            FileUtils.copyFile(man, manifestTestLocation);
        }
        _manifestToTest = manifestTestLocation;
    }      
    
    /*
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {        
        FileSupport.deleteFolder(FileSupport.getMainTestFolder());
        super.tearDown();
    }

    public void testScormPackageHandler() throws Exception {
        setManifest(new File(pathTestData, SCORM12_Package1.fileManifest), "SCORM12_Package1");
        ScormPackageHandler scormPackageHandler = new ScormPackageHandler(_manifestToTest);
        assertNotNull("ScormPackageHandler is null", scormPackageHandler);
        assertNotNull("ScormPackageHandler document is null", scormPackageHandler.getDocument());
        assertTrue("ScormPackageHandler document should have Root Element", scormPackageHandler.getDocument().hasRootElement());
    }

    /**
     * Test the buildSettings()  In this case we should only get a reload-settings.xml
     * file generated as the items are all assets
     * @throws Exception
     */
    public void testBuildSettings() throws Exception {         
        setManifest(new File(pathTestData, SCORM12_Package1.fileManifest), "SCORM12_Package1");
        ScormPackageHandler scormPackageHandler = new ScormPackageHandler(_manifestToTest);
        scormPackageHandler.buildSettings();
        // Make sure reload-settings.xml file has been generated
        assertTrue("reload-settings.xml does not exist", ScormTomcatHandler.getSharedInstance().getTrackedSettings("SCORM12_Package1").exists());
    }

    /**
     * Test the buildSettings()  In this case we should get a reload-settings.xml
     * file generated and several sco files.
     * @throws Exception
     */
    public void testBuildSettings1() throws Exception {         
        setManifest(new File(pathTestData, SCORM12_Package3.fileManifest), "SCORM12_Package3");
        ScormPackageHandler scormPackageHandler = new ScormPackageHandler(_manifestToTest);
        scormPackageHandler.buildSettings();
        // Make sure reload-settings.xml file has been generated
        assertTrue("reload-settings.xml does not exist", ScormTomcatHandler.getSharedInstance().getTrackedSettings("SCORM12_Package3").exists());
        // now check the scos are there...
        assertTrue("S100001.xml does not exist", ScormTomcatHandler.getSharedInstance().getScoSettings("SCORM12_Package3", "S100001").exists());
        assertTrue("S100002.xml does not exist", ScormTomcatHandler.getSharedInstance().getScoSettings("SCORM12_Package3", "S100002").exists());
        assertTrue("S100003.xml does not exist", ScormTomcatHandler.getSharedInstance().getScoSettings("SCORM12_Package3", "S100003").exists());
        assertTrue("S100004.xml does not exist", ScormTomcatHandler.getSharedInstance().getScoSettings("SCORM12_Package3", "S100004").exists());
        assertTrue("S110001.xml does not exist", ScormTomcatHandler.getSharedInstance().getScoSettings("SCORM12_Package3", "S110001").exists());
        assertTrue("S110002.xml does not exist", ScormTomcatHandler.getSharedInstance().getScoSettings("SCORM12_Package3", "S110002").exists());
        assertTrue("S110003.xml does not exist", ScormTomcatHandler.getSharedInstance().getScoSettings("SCORM12_Package3", "S110003").exists());
    }
    
    /*
     * This method is tested within BuildSettings()
    public void testIterateThruManifest() {
    }
    */
    
    
    public void testGetDefaultOrganization() throws Exception {
        setManifest(new File(pathTestData, SCORM12_Package1.fileManifest), "SCORM12_Package1");
        ScormPackageHandler scormPackageHandler = new ScormPackageHandler(_manifestToTest);
        Element manifestRoot = (Element)scormPackageHandler.getDocument().getRootElement().clone();
        Element orgs = manifestRoot.getChild(SCORM12_Core.ORGANIZATIONS, manifestRoot.getNamespace());
        Element element = scormPackageHandler.getDefaultOrganization(orgs);
        assertEquals("Default organization is incorrect", "ORG-C6A9447B-2572-C268-5758-E24924528F84", element.getAttributeValue(SCORM12_Core.IDENTIFIER));
    }

    public void testGetDefaultOrganization1() throws Exception {
        setManifest(new File(pathTestData, SCORM12_Package2.fileManifest), "SCORM12_Package2");
        ScormPackageHandler scormPackageHandler = new ScormPackageHandler(_manifestToTest);
        Element manifestRoot = (Element)scormPackageHandler.getDocument().getRootElement().clone();
        Element orgs = manifestRoot.getChild(SCORM12_Core.ORGANIZATIONS, manifestRoot.getNamespace());
        Element element = scormPackageHandler.getDefaultOrganization(orgs);
        assertNull("This manifest has no organization nodes and should be null", element);
    }
    
    
    public void testGetReferencedElement() throws Exception {
        setManifest(new File(pathTestData, SCORM12_Package1.fileManifest), "SCORM12_Package1");
        ScormPackageHandler scormPackageHandler = new ScormPackageHandler(_manifestToTest);
        Element manifestRoot = (Element)scormPackageHandler.getDocument().getRootElement().clone();
        Element orgs = manifestRoot.getChild(SCORM12_Core.ORGANIZATIONS, manifestRoot.getNamespace());
        Element element = scormPackageHandler.getDefaultOrganization(orgs);        
        List itemList = element.getChildren(SCORM12_Core.ITEM, manifestRoot.getNamespace());                        
        assertEquals("There should be only two items here", 2, itemList.size());
        Iterator itemListElement = itemList.iterator();
		while (itemListElement.hasNext()) {
		    Element anItem = (Element) itemListElement.next();
		    	String idRef = anItem.getAttributeValue(SCORM12_Core.IDENTIFIERREF);
		    	String refId = scormPackageHandler.getReferencedElement(anItem).getAttributeValue(SCORM12_Core.IDENTIFIER);		        
		    	assertEquals("Element is not what the item references", idRef, refId);
		}
    }
    
    public void testIsSCORM12Manifest() throws Exception {
        setManifest(new File(pathTestData, SCORM12_Package1.fileManifest), "SCORM12_Package1");
        ScormPackageHandler scormPackageHandler = new ScormPackageHandler(_manifestToTest);
        assertTrue("Incorrect result " + _manifestToTest.toString() + " is a valid scorm 1.2 package", scormPackageHandler.isSCORM12Manifest());
    }

    public void testIsSCORM12Manifest1() throws Exception {
        setManifest(new File(pathTestData, CP_Package1.fileManifest), "CP_Package1");
        ScormPackageHandler scormPackageHandler = new ScormPackageHandler(_manifestToTest);
        assertFalse("Incorrect result " + _manifestToTest.toString() + " is not a valid scorm 1.2 package", scormPackageHandler.isSCORM12Manifest());
    }
    
}
