package uk.ac.reload.scormplayer.server.servermodels;

import junit.framework.TestCase;

import java.util.StringTokenizer;

/**
 * Description  A Junit class to test uk.ac.reload.scormplayer.server.servermodels.ScoUtils  
 * 
 * @author Paul Sharples
 * @version $Id: ScoUtilsTest.java,v 1.1 2004/07/04 22:31:40 phillipus Exp $
 */
public class ScoUtilsTest extends TestCase {
	
	//	---------------------------------------------------------------------------------------------
	
	/**
	 * Test seconds over 59, minutes over 59 
	 */
	public void testAddTimes1() {
		String firstTime = "0023:59:59";
		String secondTime = "00:00:01";
		String expectedResult = "24:00:00";		
		assertEquals("Times do not add up correctly", expectedResult, ScoUtils.addTimes(firstTime, secondTime));
	}
	
	/** 
	 * Test ALL
	 */
	public void testAddTimes2() {
		String firstTime = "10:01:00.01";
		String secondTime = "01:01:01.01";
		String expectedResult = "11:02:01.02";
		assertEquals("Times do not add up correctly", expectedResult, ScoUtils.addTimes(firstTime, secondTime));
	}
	
	/**
	 * Test milliseconds (different format - i digit)	 
	 */
	public void testAddTimes3() {
		String firstTime = "18:59:00.1";
		String secondTime = "00:01:00.9";
		String expectedResult = "19:00:01";
		assertEquals("Times do not add up correctly", expectedResult, ScoUtils.addTimes(firstTime, secondTime));
	}
	
	/**
	 * Test minutes added - no milliseconds
	 */
	public void testAddTimes4() {
		String firstTime = "18:59:00";
		String secondTime = "00:01:00";
		String expectedResult = "19:00:00";
		assertEquals("Times do not add up correctly", expectedResult, ScoUtils.addTimes(firstTime, secondTime));                
	}
	
	// ---------------------------------------------------------------------------------------------
	
	/**
	 * Test value with no milliseconds
	 */
	public void testParseTime1() {		
		String inputValue = "15:30:20";
		// if no milliseconds then method should return a zero at the end
		String expectedOutput = "15:30:20.0";
		
		int[] timeValues = ScoUtils.parseTime(inputValue);
							
		StringTokenizer st1 = new StringTokenizer(expectedOutput, ":.");
				
		for(int i = 0; i < timeValues.length; i++) {        
			assertEquals("Wrong value returned", st1.nextToken(), Integer.toString(timeValues[i]));
		}				  
	}
	
	/**
	 * Test with 1 digit millisecond
	 */
	public void testParseTime2() {		
		String inputValue = "15:30:20.1";

		String expectedOutput = "15:30:20.10";
		
		int[] timeValues = ScoUtils.parseTime(inputValue);
							
		StringTokenizer st1 = new StringTokenizer(expectedOutput, ":.");
				
		for(int i = 0; i < timeValues.length; i++) {        
			assertEquals("Wrong value returned", st1.nextToken(), Integer.toString(timeValues[i]));
		}				  
	}
	
	/**
	 * Test with 2 digit millisecond
	 */
	public void testParseTime3() {		
		String inputValue = "15:30:20.01";

		String expectedOutput = "15:30:20.1";
		
		int[] timeValues = ScoUtils.parseTime(inputValue);
							
		StringTokenizer st1 = new StringTokenizer(expectedOutput, ":.");
				
		for(int i = 0; i < timeValues.length; i++) {        
			assertEquals("Wrong value returned", st1.nextToken(), Integer.toString(timeValues[i]));
		}				  
	}

	/**
	 * Test with 3 digit hour
	 */
	public void testParseTime4() {		
		String inputValue = "015:30:20.01";

		String expectedOutput = "15:30:20.1";
		
		int[] timeValues = ScoUtils.parseTime(inputValue);
							
		StringTokenizer st1 = new StringTokenizer(expectedOutput, ":.");
				
		for(int i = 0; i < timeValues.length; i++) {        
			assertEquals("Wrong value returned", st1.nextToken(), Integer.toString(timeValues[i]));
		}				  
	}

	
	/**
	 * Test with 4 digit hour
	 */
	public void testParseTime5() {		
		String inputValue = "0015:30:20.01";

		String expectedOutput = "15:30:20.1";
		
		int[] timeValues = ScoUtils.parseTime(inputValue);
							
		StringTokenizer st1 = new StringTokenizer(expectedOutput, ":.");
				
		for(int i = 0; i < timeValues.length; i++) {        
			assertEquals("Wrong value returned", st1.nextToken(), Integer.toString(timeValues[i]));
		}				  
	}
}
